package com.sapportals.wcm.protocol.webdav;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * <p>
 * Copyright (c) SAP AG 2003
 * @author julian.reschke@greenbytes.de
 * @version $Id: BOMPeekingInputStream.java,v 1.3 2004/04/19 12:48:06 jre Exp $
 */
public class BOMPeekingInputStream extends BufferedInputStream {
  
  private final byte[] startbytes;
  private int startcnt;

  public BOMPeekingInputStream(InputStream baseStream) {
    super(baseStream);

    byte[] bytes = null;
    if (super.markSupported()) {
      try {
        bytes = new byte[4];
        super.mark(bytes.length);
        this.startcnt = super.read(bytes);
        super.reset();
      }
      catch (IOException ex) {
        //$JL-EXC$
        this.startcnt = 0;
        bytes = null;
      }
    }
    this.startbytes = bytes;
  }
  
  // BOM sniffing as defined in <http://www.w3.org/TR/REC-xml#sec-guessing>
  private static byte[] utf8bom = { (byte)0xef, (byte)0xbb, (byte)0xbf};
  private static byte[] utf16bebom = { (byte)0xfe, (byte)0xff};
  private static byte[] utf16lebom = { (byte)0xff, (byte)0xfe};

  /**
   * @return guessed encoding of input stream based on byte order mark (detects "UTF-8" and UTF-16 variants as
   * "UTF-16", otherwise <code>null</code>).
   */
  public String getEncoding() {
    if (this.startbytes == null) {
      return null;
    }
    else if (this.startcnt >= 3 && this.startbytes[0] == utf8bom[0] && this.startbytes[1] == utf8bom[1] && this.startbytes[2] == utf8bom[2]) {
      return "UTF-8";
    }
    else if (this.startcnt >= 2 && this.startbytes[0] == utf16lebom[0] && this.startbytes[1] == utf16lebom[1]) {
      return "UTF-16";
    }
    else if (this.startcnt >= 2 && this.startbytes[0] == utf16bebom[0] && this.startbytes[1] == utf16bebom[1]) {
      return "UTF-16";
    }
    else {
      return null;
    }
  }

}