package com.sapportals.wcm.protocol.webdav;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.w3c.dom.Document;

import com.sap.tc.logging.Location;
import com.sapportals.wcm.repository.Content;
import com.sapportals.wcm.repository.IResource;
import com.sapportals.wcm.repository.ResourceException;
import com.sapportals.wcm.util.content.IContent;
import com.sapportals.wcm.util.http.Base64;
import com.sapportals.wcm.util.logging.LoggingFormatter;
import com.sapportals.wcm.util.uri.IHierarchicalUri;

/**
 * <p>
 * Copyright (c) SAP AG 2004
 * @author julian.reschke@greenbytes.de
 */
public class DavMountProvider {
  
  private static final Location log = Location.getLocation(DavMountProvider.class);

  private static final Name N_dv_open_collection = new Name(Consts.NS_WDSERVLET, "open-collection");

  private DavMountProvider() {
  }
  
  public static DavMountProvider getInstance() {
    return new DavMountProvider();
  }
  
  public IContent getContentForResource(IResource res, IHierarchicalUri baseUri) throws ResourceException {
    
    Document doc = DomBuilder.newDocument(N_dv_open_collection);
    String link = baseUri.mapToAbsoluteUri(res.getRID()).toExternalForm();
    
    doc.getDocumentElement().appendChild(DomElement.makeNew(doc, Consts.N_href,link));
    
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    try {
      DomSerializer.write(new OutputStreamWriter(os, "UTF-8"), doc, true);
    }
    catch (Exception e) {
      throw new ResourceException(e);
    }

    byte[] ba = os.toByteArray();

    String etag = null;
    try {
      etag = Base64.encode(MessageDigest.getInstance("MD5").digest(ba));
    }
    catch (NoSuchAlgorithmException e1) {
      log.debugT(LoggingFormatter.extractCallstack(e1));
    }

    Content content = new Content(new ByteArrayInputStream(ba), "application/x.sap.ep.km.webdavmount+xml", os.size());
    if (etag != null) content.setEntityTag(etag);

    return content;
  }

}
