package com.sapportals.wcm.protocol.webdav;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.sapportals.wcm.WcmException;
import com.sapportals.wcm.util.uri.IUriReference;

/**
 * Holds HTTP status, WebDAV conditions and descriptions.
 *
 * <p>
 * Copyright (c) SAP AG 2002-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: DavStatus.java,v 1.10 2005/01/21 17:15:37 jre Exp $
 */

public class DavStatus {

  // WebDAV status codes

  public static final int SC_PROCESSING           = 102;
  public static final int SC_MULTI_STATUS         = 207;
  public static final int SC_UNPROCESSABLE_ENTITY = 422;
  public static final int SC_LOCKED               = 423;
  public static final int SC_FAILED_DEPENDENCY    = 424;
  public static final int SC_LOOP_DETECTED        = 506;
  public static final int SC_NOT_EXTENDED         = 510;

  // HTTP 1.1 status codes

  public static final int SC_BAD_GATEWAY = HttpServletResponse.SC_BAD_GATEWAY;
  public static final int SC_BAD_REQUEST = HttpServletResponse.SC_BAD_REQUEST;
  public static final int SC_CONFLICT = HttpServletResponse.SC_CONFLICT;
  public static final int SC_CREATED = HttpServletResponse.SC_CREATED;
  public static final int SC_FORBIDDEN = HttpServletResponse.SC_FORBIDDEN;
  public static final int SC_FOUND = HttpServletResponse.SC_MOVED_TEMPORARILY;
  public static final int SC_INTERNAL_SERVER_ERROR = HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
  public static final int SC_METHOD_NOT_ALLOWED = HttpServletResponse.SC_METHOD_NOT_ALLOWED;
  public static final int SC_MOVED_PERMANENTLY = HttpServletResponse.SC_MOVED_PERMANENTLY;
  public static final int SC_MOVED_TEMPORARILY = HttpServletResponse.SC_MOVED_TEMPORARILY;
  public static final int SC_NO_CONTENT = HttpServletResponse.SC_NO_CONTENT;
  public static final int SC_NOT_ACCEPTABLE = HttpServletResponse.SC_NOT_ACCEPTABLE;
  public static final int SC_NOT_FOUND = HttpServletResponse.SC_NOT_FOUND;
  public static final int SC_NOT_IMPLEMENTED = HttpServletResponse.SC_NOT_IMPLEMENTED;
  public static final int SC_NOT_MODIFIED = HttpServletResponse.SC_NOT_MODIFIED;
  public static final int SC_OK = HttpServletResponse.SC_OK;
  public static final int SC_PARTIAL_CONTENT = HttpServletResponse.SC_PARTIAL_CONTENT;
  public static final int SC_PRECONDITION_FAILED = HttpServletResponse.SC_PRECONDITION_FAILED;
  public static final int SC_REQUEST_ENTITY_TOO_LARGE= HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE;
  public static final int SC_UNSUPPORTED_MEDIA_TYPE = HttpServletResponse.SC_UNSUPPORTED_MEDIA_TYPE;
  public static final int SC_REQUESTED_RANGE_NOT_SATISFIABLE = HttpServletResponse.SC_REQUESTED_RANGE_NOT_SATISFIABLE;
  public static final int SC_SERVICE_UNAVAILABLE = HttpServletResponse.SC_SERVICE_UNAVAILABLE;

  //1xx series
  //
  public static final String HTTP_RESP_CONTINUE                      = "HTTP/1.1 100 Continue";
  public static final String HTTP_RESP_SWITCH_PROTOCOL               = "HTTP/1.1 101 Switching Protocols";
  public static final String HTTP_RESP_PROCESSING                    = "HTTP/1.1 102 Processing";

  //2xx series
  //
  public static final String HTTP_RESP_OK                            = "HTTP/1.1 200 OK";
  public static final String HTTP_RESP_CREATED                       = "HTTP/1.1 201 Created";
  public static final String HTTP_RESP_ACCEPTED                      = "HTTP/1.1 202 Accepted";
  public static final String HTTP_RESP_NON_AUTHORITATIVE_INFORMATION = "HTTP/1.1 203 Non-Authoritative Information";
  public static final String HTTP_RESP_NO_CONTENT                    = "HTTP/1.1 204 No Content";
  public static final String HTTP_RESP_RESET_CONTENT                 = "HTTP/1.1 205 Reset Content";
  public static final String HTTP_RESP_PARTIAL_CONTENT               = "HTTP/1.1 206 Partial Content";
  public static final String HTTP_RESP_MULTI_STATUS                  = "HTTP/1.1 207 Multi-Status";

  //3xx series
  //
  public static final String HTTP_RESP_MULTIPLE_CHOICES              = "HTTP/1.1 300 Multiple Choices";
  public static final String HTTP_RESP_MOVED_PERMANENTLY             = "HTTP/1.1 301 Moved Permanently";
  public static final String HTTP_RESP_FOUND                         = "HTTP/1.1 302 Found";
  public static final String HTTP_RESP_SEE_OTHER                     = "HTTP/1.1 303 See Other";
  public static final String HTTP_RESP_NOT_MODIFIED                  = "HTTP/1.1 304 Not Modified";
  public static final String HTTP_RESP_USE_PROXY                     = "HTTP/1.1 305 Use Proxy";
  // 306 reserved
  public static final String HTTP_RESP_TEMP_REDIRECT                 = "HTTP/1.1 307 Temporary Redirect";

  //4xx series
  //
  public static final String HTTP_RESP_BAD_REQUEST                   = "HTTP/1.1 400 Bad Request";
  public static final String HTTP_RESP_UNAUTHORIZED                  = "HTTP/1.1 401 Unauthorized";
  public static final String HTTP_RESP_PAYMENT_REQUIRED              = "HTTP/1.1 402 Payment Required";
  public static final String HTTP_RESP_FORBIDDEN                     = "HTTP/1.1 403 Forbidden";
  public static final String HTTP_RESP_NOT_FOUND                     = "HTTP/1.1 404 Not Found";
  public static final String HTTP_RESP_METHOD_NOT_ALLOWED            = "HTTP/1.1 405 Method Not Allowed";
  public static final String HTTP_RESP_NOT_ACCEPTABLE                = "HTTP/1.1 406 Not Acceptable";
  public static final String HTTP_RESP_PROXY_AUTH_REQUIRED           = "HTTP/1.1 407 Proxy Authentication Required";
  public static final String HTTP_RESP_REQUEST_TIMEOUT               = "HTTP/1.1 408 Request Timeout";
  public static final String HTTP_RESP_CONFLICT                      = "HTTP/1.1 409 Conflict";
  public static final String HTTP_RESP_GONE                          = "HTTP/1.1 410 Gone";
  public static final String HTTP_RESP_LENGTH_REQUIRED               = "HTTP/1.1 411 Length Required";
  public static final String HTTP_RESP_PRECONDITION_FAILED           = "HTTP/1.1 412 Precondition Failed";
  public static final String HTTP_RESP_REQUEST_ENTITY_TOO_LARGE      = "HTTP/1.1 413 Request Entitiy Too Large";
  public static final String HTTP_RESP_REQUEST_URI_TOO_LONG          = "HTTP/1.1 414 Request-URI Too Long";
  public static final String HTTP_RESP_UNSUPPORTED_MEDIA_TYPE        = "HTTP/1.1 415 Unsupported Media Type";
  public static final String HTTP_RESP_REQUEST_RANGE_NOT_SATISFIABLE = "HTTP/1.1 416 Request Range Not Satisfiable";
  public static final String HTTP_RESP_EXPECTATION_FAILED            = "HTTP/1.1 417 Expectation Failed";

  public static final String HTTP_RESP_UNPROCESSABLE_ENTITY          = "HTTP/1.1 422 Unprocessable Entity";
  public static final String HTTP_RESP_LOCKED                        = "HTTP/1.1 423 Locked";
  public static final String HTTP_RESP_FAILED_DEPENDENCY             = "HTTP/1.1 424 Failed Dependency";

  //5xx series
  //
  public static final String HTTP_RESP_INTERNAL_SERVER_ERROR         = "HTTP/1.1 500 Internal Server Error";
  public static final String HTTP_RESP_NOT_IMPLEMENTED               = "HTTP/1.1 501 Not Implemented";
  public static final String HTTP_RESP_BAD_GATEWAY                   = "HTTP/1.1 502 Bad Gateway";
  public static final String HTTP_RESP_SERVICE_UNAVAILABLE           = "HTTP/1.1 503 Service Unavailable";
  public static final String HTTP_RESP_GATEWAY_TIMEOUT               = "HTTP/1.1 504 Gateway Timeout";
  public static final String HTTP_RESP_HTTP_VERSION_NOT_SUPPORTED    = "HTTP/1.1 505 HTTP Version Not Supported";
  public static final String HTTP_RESP_LOOP_DETECTED                 = "HTTP/1.1 506 Loop Detected";
  public static final String HTTP_RESP_INSUFFICIENT_STORAGE          = "HTTP/1.1 507 Insufficient Storage";

  public static final DavStatus BAD_GATEWAY = new DavStatus(SC_BAD_GATEWAY);

  public static final DavStatus BAD_REQUEST = new DavStatus(SC_BAD_REQUEST);
  
  public static final DavStatus CONFLICT = new DavStatus(SC_CONFLICT);
  public static final DavStatus CONFLICT_add_must_be_new_label = new DavStatus(SC_CONFLICT, DavConditions.N_add_must_be_new_label);
  public static final DavStatus CONFLICT_bind_source_exists = new DavStatus(SC_CONFLICT, DavConditions.N_bind_source_exists);
  public static final DavStatus CONFLICT_can_overwrite = new DavStatus(SC_CONFLICT, DavConditions.N_can_overwrite);
  public static final DavStatus CONFLICT_cannot_add_to_existing_history = new DavStatus(SC_CONFLICT, DavConditions.N_cannot_add_to_existing_history);
  public static final DavStatus CONFLICT_cannot_modify_checked_in_parent = new DavStatus(SC_CONFLICT, DavConditions.N_cannot_modify_checked_in_parent);
  public static final DavStatus CONFLICT_cannot_modify_protected_property = new DavStatus(SC_CONFLICT, DavConditions.N_cannot_modify_protected_property);
  public static final DavStatus CONFLICT_cannot_modify_version_controlled_property = new DavStatus(SC_CONFLICT, DavConditions.N_cannot_modify_version_controlled_property);
  public static final DavStatus CONFLICT_collection_must_be_ordered = new DavStatus(SC_CONFLICT, DavConditions.N_collection_must_be_ordered);
  public static final DavStatus CONFLICT_label_must_exist = new DavStatus(SC_CONFLICT, DavConditions.N_label_must_exist);
  public static final DavStatus CONFLICT_must_be_checked_in = new DavStatus(SC_CONFLICT, DavConditions.N_must_be_checked_in);
  public static final DavStatus CONFLICT_must_be_checked_out = new DavStatus(SC_CONFLICT, DavConditions.N_must_be_checked_out);
  public static final DavStatus CONFLICT_name_allowed = new DavStatus(SC_CONFLICT, DavConditions.N_name_allowed);
  public static final DavStatus CONFLICT_one_version_controlled_resource_per_history_per_workspace = new DavStatus(SC_CONFLICT, DavConditions.N_one_version_controlled_resource_per_history_per_workspace);
  public static final DavStatus CONFLICT_preserve_versioning_properties = new DavStatus(SC_CONFLICT, DavConditions.N_preserve_versioning_properties);
  public static final DavStatus CONFLICT_resource_must_be_null = new DavStatus(SC_CONFLICT, DavConditions.N_resource_must_be_null);
  public static final DavStatus CONFLICT_supported_report = new DavStatus(SC_CONFLICT, DavConditions.N_supported_report);
  public static final DavStatus CONFLICT_unbind_source_exists = new DavStatus(SC_CONFLICT, DavConditions.N_unbind_source_exists);
  
  public static final DavStatus FAILED_DEPENDENCY = new DavStatus(SC_FAILED_DEPENDENCY);
  public static final DavStatus FORBIDDEN = new DavStatus(SC_FORBIDDEN);
  public static final DavStatus FORBIDDEN_cannot_copy_history = new DavStatus(SC_FORBIDDEN, DavConditions.N_cannot_copy_history);
  public static final DavStatus FORBIDDEN_cannot_modify_protected_property = new DavStatus(SC_FORBIDDEN, DavConditions.N_cannot_modify_protected_property);
  public static final DavStatus FORBIDDEN_cannot_modify_version = new DavStatus(SC_FORBIDDEN, DavConditions.N_cannot_modify_version);
  public static final DavStatus FORBIDDEN_cannot_rename_history = new DavStatus(SC_FORBIDDEN, DavConditions.N_cannot_rename_history);
  public static final DavStatus FORBIDDEN_checkout_of_checked_out_version_is_discouraged = new DavStatus(SC_FORBIDDEN, DavConditions.N_checkout_of_checked_out_version_is_discouraged);
  public static final DavStatus FORBIDDEN_checkout_of_version_with_descendant_is_discouraged = new DavStatus(SC_FORBIDDEN, DavConditions.N_checkout_of_version_with_descendant_is_discouraged);
  public static final DavStatus FORBIDDEN_cycle_allowed = new DavStatus(SC_FORBIDDEN, DavConditions.N_cycle_allowed);
  public static final DavStatus FORBIDDEN_must_be_version = new DavStatus(SC_FORBIDDEN, DavConditions.N_must_be_version);
  public static final DavStatus FORBIDDEN_name_allowed = new DavStatus(SC_FORBIDDEN, DavConditions.N_name_allowed);
  public static final DavStatus FORBIDDEN_need_privileges = new DavStatus(SC_FORBIDDEN, DavConditions.N_need_privileges);
  public static final DavStatus FORBIDDEN_number_of_matches_within_limits = new DavStatus(SC_FORBIDDEN, DavConditions.N_number_of_matches_within_limits);
  public static final DavStatus FORBIDDEN_redirect_lifetime_supported = new DavStatus(SC_FORBIDDEN, DavConditions.N_redirect_lifetime_supported);
  public static final DavStatus FORBIDDEN_search_grammar_supported = new DavStatus(SC_FORBIDDEN, DavConditions.N_search_grammar_supported);
  public static final DavStatus FORBIDDEN_search_scope_valid = new DavStatus(SC_FORBIDDEN, DavConditions.N_search_scope_valid);
  public static final DavStatus FORBIDDEN_segment_must_identify_member = new DavStatus(SC_FORBIDDEN, DavConditions.N_segment_must_identity_member);
  public static final DavStatus FORBIDDEN_supported_report = new DavStatus(SC_FORBIDDEN, DavConditions.N_supported_report);
  public static final DavStatus FORBIDDEN_unbind_source_exists = new DavStatus(SC_FORBIDDEN, DavConditions.N_unbind_source_exists);
  public static final DavStatus FORBIDDEN_workspace_location_ok = new DavStatus(SC_FORBIDDEN, DavConditions.N_workspace_location_ok);
  public static final DavStatus FOUND = new DavStatus(SC_FOUND);

  public static final DavStatus INTERNAL_SERVER_ERROR = new DavStatus(SC_INTERNAL_SERVER_ERROR);

  public static final DavStatus INTERNAL_SERVER_ERROR_filter_config = new DavStatus(SC_INTERNAL_SERVER_ERROR, DavConditions.N_SAP_filter_config_valid);
  public static final DavStatus INTERNAL_SERVER_ERROR_io_error = new DavStatus(SC_INTERNAL_SERVER_ERROR, DavConditions.N_SAP_io_error);

  public static final DavStatus LOCKED = new DavStatus(SC_LOCKED);
  public static final DavStatus LOOP_DETECTED = new DavStatus(SC_LOOP_DETECTED);

  public static final DavStatus METHOD_NOT_ALLOWED = new DavStatus(SC_METHOD_NOT_ALLOWED);
  public static final DavStatus MOVED_TEMPORARILY = new DavStatus(SC_MOVED_TEMPORARILY);
  public static final DavStatus MULTI_STATUS = new DavStatus(SC_MULTI_STATUS);

  public static final DavStatus NOT_EXTENDED = new DavStatus(SC_NOT_EXTENDED);
  public static final DavStatus NOT_FOUND = new DavStatus(SC_NOT_FOUND);
  public static final DavStatus NOT_IMPLEMENTED = new DavStatus(SC_NOT_IMPLEMENTED);

  public static final DavStatus OK = new DavStatus(SC_OK);

  public static final DavStatus PRECONDITION_FAILED = new DavStatus(SC_PRECONDITION_FAILED);
  public static final DavStatus PRECONDITION_FAILED_can_overwrite = new DavStatus(SC_PRECONDITION_FAILED, DavConditions.N_can_overwrite);

  public static final DavStatus SERVICE_UNAVAILABLE = new DavStatus(SC_SERVICE_UNAVAILABLE);

  public static final DavStatus UNPROCESSABLE_ENTITY = new DavStatus(SC_UNPROCESSABLE_ENTITY);
  public static final DavStatus UNPROCESSABLE_ENTITY_sap_request_body_valid = new DavStatus(SC_UNPROCESSABLE_ENTITY, DavConditions.N_SAP_request_body_valid);
  
  // SAP Portals RFC2518 error conditions

  public static final DavStatus BAD_REQUEST_sap_legal_depth_header_value = new DavStatus(SC_BAD_REQUEST,
      DavConditions.N_SAP_legal_depth_header_value);

  public static final DavStatus BAD_REQUEST_sap_legal_if_header_value = new DavStatus(SC_BAD_REQUEST,
      DavConditions.N_SAP_legal_if_header_value);

  public static final DavStatus BAD_REQUEST_sap_legal_request_URI = new DavStatus(SC_BAD_REQUEST,
      DavConditions.N_SAP_legal_request_URI);

  public static final DavStatus BAD_REQUEST_sap_lock_token_present = new DavStatus(SC_BAD_REQUEST,
      DavConditions.N_SAP_lock_token_present);

  public static final DavStatus BAD_REQUEST_sap_request_body_wellformed = new DavStatus(SC_BAD_REQUEST,
      DavConditions.N_SAP_request_body_wellformed);

  public static final DavStatus BAD_REQUEST_sap_required_request_body_present = new DavStatus(SC_BAD_REQUEST,
      DavConditions.N_SAP_required_request_body_present);

  public static final DavStatus CONFLICT_sap_destination_must_be_in_collection = new DavStatus(SC_CONFLICT,
      DavConditions.N_SAP_destination_must_be_in_collection);

  public static final DavStatus CONFLICT_sap_parent_collection_must_exist = new DavStatus(SC_CONFLICT,
      DavConditions.N_SAP_parent_collection_must_exist);

  public static final DavStatus CONFLICT_sap_report_applicable = new DavStatus(SC_CONFLICT, DavConditions.N_SAP_report_applicable);

  public static final DavStatus CONFLICT_sap_resource_allows_virus_content = new DavStatus(SC_CONFLICT,
      DavConditions.N_SAP_resource_allows_virus_content);

  public static final DavStatus CONFLICT_sap_resource_supports_entity_body = new DavStatus(SC_CONFLICT,
      DavConditions.N_SAP_resource_supports_entity_body);

  public static final DavStatus FORBIDDEN_sap_destination_may_not_be_ancestor_of_source = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_destination_may_not_be_ancestor_of_source);

  public static final DavStatus FORBIDDEN_sap_depth_infinity_allowed = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_depth_infinity_allowed);

  public static final DavStatus FORBIDDEN_sap_inter_workspace_bind_allowed = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_inter_workspace_bind_allowed);

  public static final DavStatus FORBIDDEN_sap_lock_token_matches = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_lock_token_matches);

  public static final DavStatus FORBIDDEN_sap_property_name_valid = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_property_name_valid);

  public static final DavStatus FORBIDDEN_sap_property_value_fits = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_property_value_fits);

  public static final DavStatus FORBIDDEN_sap_inter_workspace_move_allowed = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_inter_workspace_move_allowed);

  public static final DavStatus FORBIDDEN_sap_source_and_destination_must_differ = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_source_and_destination_must_differ);

  public static final DavStatus FORBIDDEN_sap_source_may_not_be_ancestor_of_destination = new DavStatus(SC_FORBIDDEN,
      DavConditions.N_SAP_source_may_not_be_ancestor_of_destination);



  private int status;
  private IInternationalString description;
  private Name condition;

  /**
   * @param status HTTP status
   * @param description description for status
   * @param condition additional DAV error condition code
   */
  public DavStatus(int status, IInternationalString description, Name condition) {
    this.status = status;
    this.description = description;
    this.condition = condition;
  }

  /**
   * @param status HTTP status
   * @param condition additional DAV error condition code
   */
  public DavStatus(int status, Name condition) {
    this(status, null, condition);
  }

  /**
   * @param status HTTP status
   */
  public DavStatus(int status) {
    this(status, null, null);
  }

  /**
   * @param response DAV:response element to parse
   * @throws WcmException
   */
  public DavStatus(Element response) throws WcmException {
    this(SC_OK, null, null);

    List statuslist = DomElement.getChildren(response, Consts.N_status);
    if (statuslist.size() > 1) {
      throw new IllegalArgumentException("multiple DAV:status elements in DAV:response");
    }
    else if (statuslist.size() == 1) {
      Element status = (Element)statuslist.get(0);
      com.sapportals.wcm.util.http.HttpStatus wds = new com.sapportals.wcm.util.http.HttpStatus();
      wds.parse(DomElement.getText(status));
      this.status = wds.getCode();
      this.description = new InternationalString("en", wds.getDescription());
    }
    else {
      this.status = DavStatus.SC_OK;
      this.description = null;
    }
  }

  /**
   * get a <a href="http://greenbytes.de/tech/webdav/rfc2518.html#ELEMENT_response">DAV:response</a> element for this status
   * @param doc owner document
   * @param href href to be reported
   * @return the created element
   */
  public Element getResponseElement(Document doc, IUriReference href) {
    return getResponseElement(doc, href.toExternalForm());
  }

  /**
   * get a <a href="http://greenbytes.de/tech/webdav/rfc2518.html#ELEMENT_response">DAV:response</a> element for this status
   * @param doc owner document
   * @param href href to be reported
   * @return the created element
   */
  public Element getResponseElement(Document doc, String href) {

    Element response = DomElement.makeNew(doc, Consts.N_response);
    response.appendChild(DomElement.makeNew(doc, Consts.N_href, href));
    response.appendChild(DomElement.makeNew(doc, Consts.N_status, getStatusLine(this.status)));
    Element responsedescription = getResponseDescriptionElement(doc);
    if (responsedescription != null) {
      response.appendChild(responsedescription);
    } 
    return response;
  }

  public Element getResponseDescriptionElement(Document doc) {
    Element desc = null;

    if (this.description != null || this.condition != null) {

      desc = DomElement.makeNew(doc, Consts.N_responsedescription, this.description);

      if (this.condition != null) {
        desc.insertBefore(
          DomElement.makeNew(doc, Consts.N_error,
            DomElement.makeNew(doc, this.condition)),
          desc.getFirstChild());
      }
    }

    return desc;
  }


  /**
   * get a <a href="http://greenbytes.de/tech/webdav/rfc3253.html#rfc.section.1.6">DAV:error</a> element for this {@link DavStatus}
   * @param doc owner document
   * @param desc when present, add message from exception to child sap:description element
   * @return created DAV:error element
   */
  public Element getErrorElement(Document doc, String desc) {

    Element condition = this.condition == null
      ? null
      : DomElement.makeNew(doc, this.condition, desc == null
          ? null
          : DomElement.makeSafeComment(doc, desc));
    
    Element error = DomElement.makeNew(doc, Consts.N_error, condition);
    
    return error;
  }

  /**
   * get the HTTP status
   * @return HTTP status
   */
  public int getStatus() {
    return this.status;
  }

  /**
   * get the WebDAV condition name
   * @return the condition
   */
  public Name getCondition() {
    return this.condition;
  }

  /**
   * get the description
   * @return the description
   */
  public IInternationalString getDescription() {
    return this.description;
  }

  /**
   * get the description (not language tagged)
   * @return the description
   */
  public String getDescriptionText() {
    return this.description == null ? null : this.description.getText();
  }

  /**
   * get a string representation for an HTTP status code
   * @param httpResponseCode response code
   * @return string representation
   */
  public static String getStatusLine(int httpResponseCode) {

    switch (httpResponseCode) {
      case 100:
        return HTTP_RESP_CONTINUE;
      case 101:
        return HTTP_RESP_SWITCH_PROTOCOL;
      case 102:
        return HTTP_RESP_PROCESSING;

        //2xx series
        //
      case 200:
        return HTTP_RESP_OK;
      case 201:
        return HTTP_RESP_CREATED;
      case 202:
        return HTTP_RESP_ACCEPTED;
      case 203:
        return HTTP_RESP_NON_AUTHORITATIVE_INFORMATION;
      case 204:
        return HTTP_RESP_NO_CONTENT;
      case 205:
        return HTTP_RESP_RESET_CONTENT;
      case 206:
        return HTTP_RESP_PARTIAL_CONTENT;
      case 207:
        return HTTP_RESP_MULTI_STATUS;

        //3xx series
        //
      case 300:
        return HTTP_RESP_MULTIPLE_CHOICES;
      case 301:
        return HTTP_RESP_MOVED_PERMANENTLY;
      case 302:
        return HTTP_RESP_FOUND;
      case 303:
        return HTTP_RESP_SEE_OTHER;
      case 304:
        return HTTP_RESP_NOT_MODIFIED;
      case 305:
        return HTTP_RESP_USE_PROXY;
        // 306 reserved
      case 307:
        return HTTP_RESP_TEMP_REDIRECT;

        //4xx series
        //
      case 400:
        return HTTP_RESP_BAD_REQUEST;
      case 401:
        return HTTP_RESP_UNAUTHORIZED;
      case 402:
        return HTTP_RESP_PAYMENT_REQUIRED;
      case 403:
        return HTTP_RESP_FORBIDDEN;
      case 404:
        return HTTP_RESP_NOT_FOUND;
      case 405:
        return HTTP_RESP_METHOD_NOT_ALLOWED;
      case 406:
        return HTTP_RESP_NOT_ACCEPTABLE;
      case 407:
        return HTTP_RESP_PROXY_AUTH_REQUIRED;
      case 408:
        return HTTP_RESP_REQUEST_TIMEOUT;
      case 409:
        return HTTP_RESP_CONFLICT;
      case 410:
        return HTTP_RESP_GONE;
      case 411:
        return HTTP_RESP_LENGTH_REQUIRED;
      case 412:
        return HTTP_RESP_PRECONDITION_FAILED;
      case 413:
        return HTTP_RESP_REQUEST_ENTITY_TOO_LARGE;
      case 414:
        return HTTP_RESP_REQUEST_URI_TOO_LONG;
      case 415:
        return HTTP_RESP_UNSUPPORTED_MEDIA_TYPE;
      case 416:
        return HTTP_RESP_REQUEST_RANGE_NOT_SATISFIABLE;
      case 417:
        return HTTP_RESP_EXPECTATION_FAILED;

      case 422:
        return HTTP_RESP_UNPROCESSABLE_ENTITY;
      case 423:
        return HTTP_RESP_LOCKED;
      case 424:
        return HTTP_RESP_FAILED_DEPENDENCY;

        //5xx series
        //
      case 500:
        return HTTP_RESP_INTERNAL_SERVER_ERROR;
      case 501:
        return HTTP_RESP_NOT_IMPLEMENTED;
      case 502:
        return HTTP_RESP_BAD_GATEWAY;
      case 503:
        return HTTP_RESP_SERVICE_UNAVAILABLE;
      case 504:
        return HTTP_RESP_GATEWAY_TIMEOUT;
      case 505:
        return HTTP_RESP_HTTP_VERSION_NOT_SUPPORTED;
      case 506:
        return HTTP_RESP_LOOP_DETECTED;
      case 507:
        return HTTP_RESP_INSUFFICIENT_STORAGE;

      default:
        return "HTTP/1.1 " + httpResponseCode + " Unknown";
    }
  }

  public boolean equals(Object o) {
    if (o == null) {
      return false;
    }
    else if (! (o instanceof DavStatus)) {
      return false;
    }
    else {
      DavStatus other = (DavStatus)o;

      if (this.status != other.status) {
        return false;
      }
      else {
        return this.toString().equals(other.toString());
      }
    }
  }

  public int hashCode() {
    return this.toString().hashCode();
  }

  public String toString() {
    return this.status + " " + this.description + " " + this.condition;
  }
}