package com.sapportals.wcm.protocol.webdav;

import java.io.*;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sap.tc.logging.Location;


/**
 * Utility functions for bootstrapping a {@link org.w3c.dom.Document}
 * and for parsing it from a stream or a string.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: DomBuilder.java,v 1.3 2004/06/28 18:06:05 jre Exp $
 */
public class DomBuilder {
  
  private static final Location log = Location.getLocation(DomBuilder.class);
  
  /**
   * Create an empty DOM document.
   * @return the DOM
   */
  public static Document newDocument() {
    return com.sapportals.wcm.util.xml.DomBuilder.newDocument();
  }

  /**
   * Create a DOM document including the document element node.
   * @param namespaceUri namespace of document element
   * @param documentElement local name of document element
   * @return the DOM
   */
  public static Document newDocument(String namespaceUri, String documentElement) {
    Document doc = newDocument();
    doc.appendChild(DomElement.makeNew(doc, namespaceUri, documentElement));
    return doc;
  }

  /**
   * Create a DOM document including the document element node.
   * @param name {@link com.sapportals.wcm.util.name.IName} of document element
   * @return the DOM
   */
  public static Document newDocument(com.sapportals.wcm.util.name.IName name) {
    return newDocument(name.getNamespace(), name.getName());
  }

  /**
   * Parse an input stream into a Document
   * @param input the input stream
   * @return the Document
   */

  public static Document parse(InputStream input) throws SAXException, IOException {
    Document doc = parse(new InputSource(input), true);

    try {
      doc.getDocumentElement().getNamespaceURI();
    }
    catch (java.lang.NoSuchMethodError ex) {
      log.fatalT("-------------------------------------------------------");
      log.fatalT("Couldn't invoke: getDocumentElement().getNamespaceURI()");
      log.fatalT("This is likely caused by outdated JAR files in the");
      log.fatalT("CLASSPATH preceding the DOM-Level-2 that is needed.");
      log.fatalT("Current DOM impl is: " + doc.getClass().getName());
      log.fatalT("-------------------------------------------------------");
      throw ex;
    }

    return doc;
  }

  /**
   * Parse a File into a Document
   * @param input the input stream
   * @return the Document
   */
  public static Document parse(File input) throws SAXException, IOException {
    return parse(new InputSource(new FileReader(input)), true);
  }

  /**
   * Parse a string into a Document
   * @param input the input stream
   * @return the Document
   */
  public static Document parse(String input) throws SAXException, java.io.IOException {
    return parse(new InputSource(new StringReader(input)), true);
  }

  /**
   * Parse a string into a Document
   * @param input the input stream
   * @return the Document
   */
  public static Document parse(com.sapportals.wcm.repository.XMLMarkup input) throws SAXException, java.io.IOException {
    return parse(new InputSource(new StringReader(input.toString())), true);
  }

  private static Document parse (InputSource is, boolean namespaceAware) throws SAXException {
    return com.sapportals.wcm.util.xml.DomBuilder.parse(is, namespaceAware);
  }

}