package com.sapportals.wcm.protocol.webdav;

import java.io.Writer;

import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.sap.tc.logging.Location;

import com.sapportals.wcm.util.xml.SimpleSerializer;


/**
 * Utility function for serializing the contents of a {@link org.w3c.dom.Document}
 * into a String.
 * <p>
 * Handles the Xerces-specific quirks in serializing documents with namespaces
 * correctly.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Revision: 1.2 $
 */

public class DomSerializer {

  private static final Location log = Location.getLocation(DomSerializer.class);

  private static class Trans {

    static final TransformerFactory Factory;

    static {
      TransformerFactory factory = null;
      try {
        Class fac = Class.forName(DomConfig.JAXP_TRANSFORMER_FACTORY);
        factory = (TransformerFactory)fac.newInstance();
      }
      catch (Exception ex) {
        factory = TransformerFactory.newInstance();
      }
      Factory = factory;
    }
  }

  /**
   * Serialize the node (and it's descendants) to a writer.
   * @param node start node
   * @param indent whether to produce indented XML or not
   * @throws java.io.IOException on write errors
   */
  public static void write(Writer writer, Node node, boolean indent) throws java.io.IOException {
    write (writer, node, indent, true);
  }

  /**
   * Serialize the element (and it's children) to a string.
   * @param e the Element
   * @param indent whether to produce indented XML or not
   * @return the XML string or an empty string in case of error
   */
  public static String toString (Element e, boolean indent) {
    return SimpleSerializer.toString(e, indent);
  }




  /**
   * Serialize the document to a string.
   * @param doc the document
   * @param indent whether to produce indented XML or not
   * @return the XML string or an empty string in case of error
   */
  public static String toString(Document doc, boolean indent) {
    return toString(doc, indent, true);
  }

  /**
   * Serialize the document to a string.
   * @param doc the document
   * @param indent whether to produce indented XML or not
   * @param doNsProcessing enable/disable special namespace support in the serializer
   * @return the XML string or an empty string in case of error
   */
  public static String toString (Document doc, boolean indent, boolean doNsProcessing) {
    return SimpleSerializer.toString(doc, indent, doNsProcessing);
  }

  /**
   * Add xmlns attributes for broken Xerces serializer
   */
/*
  private static boolean nsDeclInScope (Node root, Node curr,
    String namespaceURI, String prefix)
  {
    if (root != curr)
    {
      Node parent = curr.getParentNode();

      if (parent.getNodeType() != Node.ELEMENT_NODE) return false;

      Element e = (Element) parent;

      Attr nsnode = null;

      if (prefix.equals (""))
        nsnode = e.getAttributeNodeNS (Consts.NS_XMLNS, "xmlns");
      else
        nsnode = e.getAttributeNodeNS (Consts.NS_XMLNS, "xmlns:" + prefix);

      if (nsnode == null)
        return nsDeclInScope (root, e, namespaceURI, prefix);
      else
        return nsnode.getValue().equals (namespaceURI);
    }

    return false;
  }
*/
/*  private static void _NSfixupForXerces (Element root, Node curr)
  {
    if (curr.getNodeType() == Node.ELEMENT_NODE)
    {
      Element e = (Element) curr;
      String ns = e.getNamespaceURI();
      String prefix = e.getPrefix();
      if (prefix == null) prefix = "";

      if (! nsDeclInScope (root, curr, ns, prefix))
      {
        if (prefix.equals (""))
          e.setAttributeNS (Consts.NS_XMLNS, "xmlns", ns);
        else
          e.setAttributeNS (Consts.NS_XMLNS, "xmlns:" + prefix, ns);
      }

      NamedNodeMap attrs = e.getAttributes();

      for (int i = 0; i < attrs.getLength(); i++)
      {
        String ans = attrs.item(i).getNamespaceURI();
        String aprefix = attrs.item(i).getPrefix();

        if (aprefix != null && ! aprefix.equals("") &&
          ans != null &&
          ! ans.equals (Consts.NS_XMLNS) &&
          ! nsDeclInScope (root, curr, ans, aprefix))
        {
          root.setAttributeNS (Consts.NS_XMLNS, "xmlns:" + aprefix, ans);
        }
      }

      for (Node c = e.getFirstChild(); c != null; c = c.getNextSibling())
      {
        _NSfixupForXerces (root, c);
      }
    }
  }

  public static void NSfixupForXerces(Element n) {
    if (!DomConfig.FIX_XERCES_SERIALIZER)
      _NSfixupForXerces (n, n);
  }
*/

  private static void write (Writer writer, Node node, boolean indent, boolean nsProcessing)
    throws java.io.IOException
  {
    boolean isDocument = (node instanceof Document);

    switch (DomConfig.TYPE) {
/*    case DomConfig.TYPE_INQMY:
      System.setProperty("javax.xml.transform.TransformerFactory",
                         "com.inqmy.lib.jaxp.TransformerFactoryImpl");
      TransformerFactory tfac = TransformerFactory.newInstance();
      try {
        Transformer transformer = tfac.newTransformer();
        transformer.setOutputProperty(OutputKeys.ENCODING, "us-ascii");
        transformer.setOutputProperty(OutputKeys.INDENT, "no");//indent? "yes" : "no");
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");

        StreamResult result = new StreamResult(writer);
        try {
          transformer.transform(new DOMSource(doc), result);
        }
        catch (TransformerException ex) {
          m_logger.error("transforming document", ex);
        }
      }
      catch (TransformerConfigurationException ex) {
        m_logger.fatal("while instantiating transformer", ex);
      }
      break;
*/
    case DomConfig.TYPE_XERCES:
//      if (DomConfig.FIX_XERCES_SERIALIZER)
      {
        SimpleSerializer ser = new SimpleSerializer (writer, indent ?  "  " : null);
        ser.setNsAware (nsProcessing);
        ser.serialize (node);
        // m_logger.error ("-> " + sw.getBuffer().toString());
//      }
//      else
//      {
//        OutputFormat of = new OutputFormat ("xml", null, indent);
//        of.setOmitXMLDeclaration (true);
//        of.setPreserveSpace (true);
//        XMLSerializer ser = new XMLSerializer (writer, of);
//        if (isDocument) {
//          Document doc = (Document)node;
//          NSfixupForXerces (doc.getDocumentElement());
//          ser.serialize (doc);
//       }
//        else if (node instanceof Element) {
//          Element elem = (Element)node;
//          NSfixupForXerces (elem);
//          ser.serialize (elem);
//        }
//        else {
//          ser.serialize ((DocumentFragment)node);
//        }
      }
      break;

    default:
      if (DomConfig.FIX_XERCES_SERIALIZER)
      {
        SimpleSerializer ser = new SimpleSerializer (writer, indent ?  "  " : null);
        ser.setNsAware (nsProcessing);
        ser.serialize (node);
        // m_logger.error ("-> " + sw.getBuffer().toString());
      }
      else
      {
        Transformer transformer = getTransformer();
        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperty(OutputKeys.ENCODING, "iso-8859-1");
        transformer.setOutputProperty(OutputKeys.INDENT, indent? "yes" : "no");
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, isDocument? "no" : "yes");

        StreamResult result = new StreamResult(writer);
        try {
          transformer.transform(new DOMSource(node), result);
        }
        catch (TransformerException ex) {
          log.errorT("transforming document" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
        }
      }
      break;
    }
  }

  private static Transformer getTransformer() {
    try {
      return Trans.Factory.newTransformer();
    }
    catch (TransformerConfigurationException ex) {
      log.fatalT("while instantiating transformer" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }
}