package com.sapportals.wcm.protocol.webdav;

/** Represents a {@link String} in a particular
 *  language (specified using a language tag as defined
 *  in <a href="http://www.ietf.org/rfc/rfc1766.txt">RFC1766</a>).
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Revision: 1.3 $
 */
public interface IInternationalString {

  /**
   * Gets the text content.
   * @return the text content
   */
  public String getText();

  /**
   * Gets the language tag.
   * @return the language tag (or <code>null</code> when language unspecified)
   */
  public String getLanguageTag();

  /**
   * Check whether the language matches the given locale
   * @param loc locale to match with
   * @param exact when <code>true</code>, compare the complete tag. Otherwise, just compare the language.
   * @return match or not
   */
  public boolean matchesLocale (java.util.Locale loc, boolean exact);

}



