package com.sapportals.wcm.protocol.webdav;

import com.sapportals.wcm.service.urlgenerator.IUriMapper;
import com.sapportals.wcm.util.uri.IHierarchicalUri;
import com.sapportals.wcm.util.uri.RID;

/**
 * Extends IUriMapper with methods to hide query parameters
 * <p>
 * Copyright (c) SAP AG 2002-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Revision: 1.2 $
 */
public interface IWebUriMapper extends IUriMapper {

  /**
   * Determine if the mapper hides the query part in the resource name
   * @return if mapper hides the query part in the resource name
   */
  public boolean hidesQuery();

  /**
   * Hides the query part in the resource name if enabled. Otherwise
   * returns the uri unchanged.
   * @return uri with query part hidden
   */
  public IHierarchicalUri hideQuery(IHierarchicalUri uri);

  /**
   * Discovers the query part in the resource name if enabled. Otherwise
   * returns the rid unchanged.
   * @return rid with query part discovered
   */
  public RID discoverQuery(RID rid);

}


