package com.sapportals.wcm.protocol.webdav;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;

/**
 * Immmutable version of {@link java.util.Properties}
 * <p>
 * Copyright (c) SAP AG 2003-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: ImmutableProperties.java,v 1.2 2004/02/19 18:31:05 jre Exp $
 */

public final class ImmutableProperties extends Properties {
  
  private boolean initDone = false;
  
  private ImmutableProperties(Properties props) {
    super();
    for (Iterator it = props.entrySet().iterator(); it.hasNext(); ) {
      Map.Entry entry = (Map.Entry)it.next();
      super.setProperty((String)entry.getKey(), (String)entry.getValue());
    }
    this.initDone = true;
  }
  
  public static ImmutableProperties getInstance(Properties props) {
    return new ImmutableProperties(props);
  }

  public Object put(Object key, Object value) {
    // TODO: do not reply on specific implementation of java.util.Properties.setProperty
    if (this.initDone) {
      throw new UnsupportedOperationException();
    }
    else {
      return super.put(key, value);
    }
  }

  public Object remove(Object key) {
    throw new UnsupportedOperationException();
  }

  public void putAll(Map t) {
    throw new UnsupportedOperationException();
  }

  public void clear() {
    throw new UnsupportedOperationException();
  }

  public Set entrySet() {
    return Collections.unmodifiableSet(super.entrySet());
  }

  public Set keySet() {
    return Collections.unmodifiableSet(super.keySet());
  }

  public Collection values() {
    return Collections.unmodifiableCollection(super.values());
  }

  public synchronized Object setProperty(String key, String value) {
    throw new UnsupportedOperationException();
  }

  public synchronized void load(InputStream inStream) throws IOException {
    throw new UnsupportedOperationException();
  }  
}