package com.sapportals.wcm.protocol.webdav;

import java.util.HashSet;
import java.util.Set;

import com.sap.tc.logging.Location;
import com.sapportals.wcm.repository.IBindableResource;
import com.sapportals.wcm.repository.IResource;
import com.sapportals.wcm.repository.ResourceException;
import com.sapportals.wcm.util.logging.LoggingFormatter;
import com.sapportals.wcm.util.uri.IUri;
import com.sapportals.wcm.util.uri.RID;

/** Generic link/bind loop checker.
 *
 * <p>
 * Copyright SAP AG 2002-2004
 * @author julian.reschke@greenbytes.de
 * @version $Revision: 1.2 $
 */
public class LoopChecker {

  private static final Location log = Location.getLocation(LoopChecker.class);

  private Set ids = new HashSet(17);

  public LoopChecker() {
  }

  public boolean add(RID rid, IUri id) {

    boolean r1 = this.ids.add(rid);
    boolean r2 = id == null ? true : this.ids.add(id);

    log.debugT("adding: " + rid + " " + id + " --> " + (r1 && r2));

    return r1 && r2;
  }

  public boolean add(RID rid) {
    return add(rid, null);
  }

  public boolean add(IResource resource) {

    try {
      RID rid = resource.getRID();
      IBindableResource bnd = (IBindableResource)resource.as(IBindableResource.class);
      IUri id = bnd == null ? null : bnd.getGloballyUniqueId();

      return add(rid, id);
    }
    catch (ResourceException ex) {
      log.debugT("getting RID or unique resource identifier - " + LoggingFormatter.extractCallstack(ex));
      return false;
    }
  }

  public boolean contains(RID rid) {
    return this.ids.contains(rid);
  }

  public boolean contains(IUri id) {
    return this.ids.contains(id);
  }

  public boolean contains(IResource resource) {
    try {
      RID rid = resource.getRID();
      IBindableResource bnd = (IBindableResource)resource.as(IBindableResource.class);
      IUri id = bnd == null ? null : bnd.getGloballyUniqueId();

      if (bnd == null || id == null) {
        return this.ids.contains(rid);
      }
      else {
        return this.ids.contains(rid) || this.ids.contains(id);
      }
    }
    catch (ResourceException ex) {
      log.debugT(LoggingFormatter.extractCallstack(ex));
      return false;
    }
  }
}



