package com.sapportals.wcm.protocol.webdav;

import org.w3c.dom.Node;

/**
 * An <a href="http://www.w3.org/TR/REC-xml#NT-Name">XML Name</a>,
 * consisting of namespace name and local name.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: Name.java,v 1.2 2004/02/19 18:31:05 jre Exp $
 */

 public class Name extends com.sapportals.wcm.util.name.Name implements Comparable {

  /**
   * Create Name from namespace name and local name
   * @param namespace namespace name
   * @param name local name
   * @returns newly created Name
   */
  public Name(String namespace, String name) {
    super(namespace, name);
  }

  public Name(String expandedName) throws Exception {
    super(expandedName);
  }

  /**
   * Create Name from DOM node
   * @param node DOM node, supplying both namespace name and local name
   * @returns newly created Name
   */
  public Name (Node node) {
    super (node.getNamespaceURI(), node.getLocalName());
  }

  public int compareTo(Object other) {
    if (other instanceof Name) {
      return compareTo((Name)other);
    }
    return this.equals(other)? 0 : -1;
  }

  public int compareTo(Name other) {
    return this.toString().compareTo (other.toString());
  }

  /**
   * gets the namespace name
   * @return the namespace name
   */
  public final String getNamespaceUri() {
    return super.getNamespace();
  }

  /**
   * gets the local name
   * @return the local name
   */
  public final String getLocalName() {
    return super.getName();
  }


  public final String getStartTag() {
    if (super.getNamespaceOrNull() != null) {
      return "<nsx:" + super.getName() + " xmlns:nsx='" + super.getNamespace() + "'>";
    }
    else {
      return "<" + super.getName() + " xmlns=''>";
    }
  }

  public final String getEmptyTag() {
    return "<" + super.getName() + " xmlns='" + super.getNamespace() + "'/>";
  }

  public final String getEndTag() {
    if (super.getNamespaceOrNull() != null) {
      return "</nsx:" + super.getName() + ">";
    }
    else {
      return "</" + super.getName() + ">";
    }
  }

  /**
   * Compare namespace name and local name with namespace and name of a DOM node.
   * @param node DOM node to compare with
   * @return whether both match
   */
  public final boolean equalsNode(Node node) {
    if (node == null) return false;

    String ns1 = super.getNamespace();
    String ns2 = node.getNamespaceURI();
    if (ns2 == null) ns2 = "";

    if (ns1.equals (ns2)) {
      return super.getName().equals (node.getLocalName());
    }
    else {
      return false;
    }
  }
}