package com.sapportals.wcm.protocol.webdav;

import com.sapportals.wcm.repository.IPropertyName;

/** Holds a property name and a description for the property.
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Revision: 1.2 $
 */
public class PropertyDescription {

  private final IPropertyName name;
  private final IInternationalString description;
  private final String asString;

  /**
   * @param name property name
   * @param description (internationalized) description
   * @returns newly created PropertyDescription
   */
  public PropertyDescription(IPropertyName name, IInternationalString description) {
    this.name = name;
    this.description = description;
    this.asString = name.toString();
  }

  /**
   * @param name property name
   * @param languageTag language tag
   * @param description description
   * @returns newly created PropertyDescription
   */
  public PropertyDescription(IPropertyName name, String languageTag, String description) {
    this.name = name;
    this.description = new InternationalString (languageTag, description);
    this.asString = name.toString();
  }

  /**
   * gets the property name
   * @return property name
   */
  public IPropertyName getName() {
    return this.name;
  }

  /**
   * gets the description
   * @return description
   */
  public IInternationalString getDescription() {
    return this.description;
  }

  public boolean equals(Object other) {
    if (other instanceof PropertyDescription) {
      return this.asString.equals (((PropertyDescription) other).asString);
    }
    else {
      return false;
    }
  }

  public int hashCode() {
    return this.asString.hashCode();
  }
}



