package com.sapportals.wcm.protocol.webdav;

import java.io.IOException;
import java.io.InputStream;

/**
 * Wraps an input stream, implementing a size limit.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author julian.reschke@greenbytes.de
 * @version $Id: SizeConstrainedInputStream.java,v 1.2 2004/02/19 18:31:05 jre Exp $
 */
public class SizeConstrainedInputStream extends InputStream {

  private final InputStream baseStream;
  private final long limit;
  private long position;

  public SizeConstrainedInputStream(InputStream baseStream, long limit) {
    this.baseStream = baseStream;
    this.limit = limit;
    this.position = 0;
  }

  public int available() throws IOException {
    return this.baseStream.available();
  }

  public void close() throws IOException {
    this.baseStream.close();
  }

  public int read() throws IOException {
    int c = this.baseStream.read();
    count(1);
    return c;
  }

  public int read(byte[] b) throws IOException {
    return (int)count(this.baseStream.read(b));
  }

  public int read(byte[] b, int off, int len) throws IOException {
    return (int)count(this.baseStream.read(b, off, len));
  }

  public long skip(long n) throws IOException {
    return count(this.baseStream.skip(n));
  }

  public class SizeExceededException extends IOException {

    public SizeExceededException(long limit) {
      super("configured size limit of " + limit + " exceeded.");
    }

  }

  // private

  private long count(long cnt) throws java.io.IOException {
    if (cnt >= 0 && (this.position += cnt) > this.limit) {
      throw new SizeExceededException(this.limit);
    }

    return cnt;
  }

}