package com.sapportals.wcm.protocol.webdav.cache;

import com.sapportals.wcm.protocol.webdav.ISimpleMap;

/** IGenerationCache is a simple map which keeps generations
 * of key to value associations. The IGenerationCache has a
 * time in milliseconds as the duration between to generations.
 * <p>
 * The idea is that the map keeps 2 generations. Each time interval
 * (generation time) a new generation is born while the oldest one
 * dies. Map entries, which are not accessed, age and will eventually
 * be removed from the map. Entries which are accessed however get
 * a rejuvenation treatment become part of the youngest generation
 * again.
 * <p>
 * Copyright (c) SAP AG 2001
 * @author stefan.eissing@greenbytes.de
 * @version $Revision: 1.1.1.1 $
 */

public interface IGenerationCache extends ISimpleMap {

  /**
   * @return the time in milliseconds between two generations
   */
  public long getGenerationTime();

  /**
   * Sets the time between two generations.
   * @param generationTime in milliseconds
   */
  public void setGenerationTime(long generationTime);

}



