package com.sapportals.wcm.protocol.webdav.cache;

import com.sapportals.wcm.protocol.webdav.ISimpleMap;
import com.sapportals.wcm.util.uri.RID;

/**
 * Hash of RIDs to objects with configurable {@link ISimpleMap}.
 * <p>
 * This Cache is multithread-safe and can take any implementation
 * of {@link ISimpleMap} as its base map. The main task is to adjust trailing
 * '/' in RIDs so that a cache value can be found under both flavours.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Revision: 1.1.1.1 $
 */

public class URICache implements IDiscardable {

  private final ISimpleMap m_map;

  /** No default constructor */
  private URICache() {
    m_map = null;
  }

  /** New URICache with given map as base.
   * @param map the ISimpleMap to use internally, need not be multithread-safe
   */
  public URICache(ISimpleMap map) {
    m_map = map;
  }

  public synchronized void clear() {
    m_map.clear();
  }

  public synchronized void discard() {
    if (m_map instanceof IDiscardable) {
      ((IDiscardable)m_map).discard();
    }
  }

  public synchronized void put(RID uri, Object o) {
    m_map.put(uri, o);
  }

  public synchronized Object get(RID uri) {
    return m_map.get(uri);
  }

  public synchronized void remove(RID uri) {
    m_map.remove(uri);
  }

}
