package com.sapportals.wcm.protocol.webdav.dompath;

/**
 *
 * <p>
 * Copyright(c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: AdditiveExpression.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class AdditiveExpression implements IExpression {

  private final int UNKNOWN = 0;
  private final int ADD = 1;
  private final int SUBSTRACT = 2;

  private final IExpression m_left;
  private final IExpression m_right;
  private final int m_operator;

  AdditiveExpression(IExpression left, String op, IExpression right) {
    m_left = left;
    m_right = right;
    m_operator = (op.equals("+")? ADD :
      (op.equals("-")? SUBSTRACT : UNKNOWN));
    if (m_operator == UNKNOWN) {
      throw new IllegalArgumentException("unknown additive operator: "+op);
    }
  }

  public IContext evaluate(IContext context) {
    return context.newInstance(evaluateDouble(context));
  }

  public boolean evaluateBoolean(IContext context) {
    return evaluateDouble(context) != 0.0;
  }

  private double evaluateDouble(IContext context) {
    double left = m_left.evaluate(context).toNumber();
    double right = m_right.evaluate(context).toNumber();

    double result = 0.0;
    switch (m_operator) {
    case ADD:
      result = left + right;
      break;
    case SUBSTRACT:
      result = left - right;
      break;
    }
    return result;
  }

  public int getType() {
    return IContext.TYPE_NUMBER;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(m_left).append((m_operator == ADD)? " + " : " - ").append(m_right);
    return sb.toString();
  }
}
