package com.sapportals.wcm.protocol.webdav.dompath;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @version $Id: AttributeStep.java,v 1.2 2004/03/03 17:44:19 jre Exp $
 */
class AttributeStep implements IStep {

  private final INodeTest m_test;
  private IStep m_next;
  private IExpression m_predicate;

  protected AttributeStep(INodeTest test) {
    m_test = test;
  }

  public Node[] match(Node[] nodes, IContext context) {

    Result result = new Result(context);
    for (int i = 0; i < nodes.length; ++i) {
      Node e = nodes[i];
      if (e.hasAttributes()) {
        NamedNodeMap map = e.getAttributes();
        if (map != null) {
          int len = map.getLength();
          for (int j = 0; j < len; ++j) {
            Node attr = map.item(j);
            //System.err.println("looking at attribute "+attr);
            if (m_test.matches(attr)) {
              String ns = attr.getNamespaceURI();
              if (Consts.NS_XMLNS.equals(ns)
                  || (ns == null && "xmlns".equals(attr.getLocalName()))
                  || "xmlns".equals(attr.getNodeName())) {
                // ignore namespace declarations
              }
              else {
                //System.err.println("matched attribute "+attr);
                result.add(attr);
              }
            }
          }
        }
      }
    }

    if (m_predicate != null) {
      context = m_predicate.evaluate(result.toContext());
      return (m_next != null)?
        m_next.match(context.getNodes(), context)
      : context.getNodes();
    }
    else {
      return (m_next != null)?
        m_next.match(result.getNodes(), context)
      : result.getNodes();
    }
  }

  public Node matchFirst(Node context) {

    NamedNodeMap map = context.getAttributes();
    if (map != null) {
      int len = map.getLength();
      for (int i = 0; i < len; ++i) {

        Node attr = map.item(i);
        if (m_test.matches(attr)) {

          String ns = attr.getNamespaceURI();
          if (Consts.NS_XMLNS.equals(ns)
              || (ns == null && "xmlns".equals(attr.getLocalName()))
              || "xmlns".equals(attr.getNodeName())) {
            // ignore namespace declarations
            //
          }
          else {
            // matches and is not a namespace declaration
            //
            if (m_next == null) {
              return attr;
            }
            else {
              Node result = m_next.matchFirst(attr);
              if (result != null) {
                return result;
              }
            }
          }
        }
      }
    }

    return null;
  }

  public void setNext(IStep next) {
    m_next = next;
  }

  public void setPredicate(IExpression expression) {
    m_predicate = expression;
  }

  public boolean containsPredicate() {
    return (m_predicate != null
            || (m_next != null && m_next.containsPredicate()));
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append('@').append(m_test.toString());
    if (m_predicate != null) {
      sb.append(m_predicate.toString());
    }
    if (m_next != null) {
      sb.append('/').append(m_next.toString());
    }
    return sb.toString();
  }
}
