package com.sapportals.wcm.protocol.webdav.dompath;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


/**
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: ChildStep.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class ChildStep implements IStep {

  private final INodeTest m_test;
  private final boolean m_isNameTest;

  private IStep m_next;
  private IExpression m_predicate;

  protected ChildStep(INodeTest test) {
    m_test = test;
    m_isNameTest = (m_test instanceof NameTest);
  }

  public Node[] match(Node[] nodes, IContext context) {

    Result result = new Result(context);
    if (m_predicate != null) {
      Result intermediate = new Result(context);
      for (int i = 0; i < nodes.length; ++i) {
        intermediate.clear();
        Node e = nodes[i];
        //System.err.println("child context node "+e);
        for (Node node = e.getFirstChild(); node != null; node = node.getNextSibling()) {
          if (m_test.matches(node)) {
            intermediate.add(node);
          }
        }

        //System.err.println("child predicate on "+list);
        IContext eval = m_predicate.evaluate(intermediate.toContext());
        if (eval.getSize() > 0) {
          result.add(eval.getNodes());
        }
      }
    }
    else {
      for (int i = 0; i < nodes.length; ++i) {
        Node e = nodes[i];
        //System.err.println("child context node "+e);
        for (Node node = e.getFirstChild(); node != null; node = node.getNextSibling()) {
          if (m_test.matches(node)) {
            result.add(node);
          }
        }
      }
    }

    return (m_next != null)?
      m_next.match(result.getNodes(), context)
    : result.getNodes();
  }

  public Node matchFirst(Node context) {
    if (m_isNameTest) {
      switch (context.getNodeType()) {
      case Node.DOCUMENT_NODE:
        Element e = ((Document)context).getDocumentElement();
        if (e != null && m_test.matches(e)) {
          return (m_next != null)? m_next.matchFirst(e) : e;
        }
        break;
      case Node.ELEMENT_NODE:
        for (Node node = context.getFirstChild();
             node != null;
             node = node.getNextSibling()) {
          if (node.getNodeType() == Node.ELEMENT_NODE
              && m_test.matches(node)) {
            if (m_next == null) {
              return node;
            }
            else {
              Node found = m_next.matchFirst(node);
              if (found != null) {
                return found;
              }
            }
          }
        }
        break;
      default:
        break;
      }
    }
    else {
      for (Node node = context.getFirstChild();
           node != null;
           node = node.getNextSibling()) {
        if (m_test.matches(node)) {
          return (m_next != null)? m_next.matchFirst(node) : node;
        }
      }
    }

    return null;
  }

  public void setNext(IStep next) {
    m_next = next;
  }

  public void setPredicate(IExpression expression) {
    m_predicate = expression;
  }

  public boolean containsPredicate() {
    return (m_predicate != null
            || (m_next != null && m_next.containsPredicate()));
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(m_test.toString());
    if (m_predicate != null) {
      sb.append(m_predicate.toString());
    }
    if (m_next != null) {
      sb.append('/').append(m_next.toString());
    }
    return sb.toString();
  }
}
