package com.sapportals.wcm.protocol.webdav.dompath;

import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Node;

/**
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @version $Id: FixedNodeList.java,v 1.2 2004/09/13 14:26:09 jre Exp $
 */
class FixedNodeList implements java.util.List {

  private class FixedNodeIterator implements java.util.ListIterator {

    private int m_index;

    public FixedNodeIterator() {
      m_index = 0;
    }

    public FixedNodeIterator(int start) {
      m_index = start;
    }

    public boolean hasNext() {
      return m_index < m_nodes.length;
    }

    public java.lang.Object next() {
      return m_nodes[m_index++];
    }

    public int previousIndex() {
      return m_index - 1;
    }

    public boolean hasPrevious() {
      return m_index > 0;
    }

    public java.lang.Object previous() {
      return m_nodes[--m_index];
    }

    public int nextIndex() {
      return m_index;
    }

    public void remove() {
      throw new java.lang.UnsupportedOperationException();
    }

    public void set(java.lang.Object A) {
      throw new java.lang.UnsupportedOperationException();
    }

    public void add(java.lang.Object A) {
      throw new java.lang.UnsupportedOperationException();
    }
  }

  private final Node[] m_nodes;

  public FixedNodeList(Node[] nodes) {
    m_nodes = nodes;
  }

  public int size() {
    return m_nodes.length;
  }

  public boolean isEmpty() {
    return m_nodes.length == 0;
  }

  public java.lang.Object[] toArray(java.lang.Object[] A) {
    return m_nodes;
  }

  public boolean contains(java.lang.Object A) {
    for (int i = 0; i < m_nodes.length; ++i) {
      if (m_nodes[i].equals(A)) {
        return true;
      }
    }
    return false;
  }

  public java.util.Iterator iterator() {
    return new FixedNodeIterator();
  }

  public java.lang.Object[] toArray() {
    return m_nodes;
  }

  public boolean addAll(java.util.Collection A) {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean add(java.lang.Object A) {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean remove(java.lang.Object A) {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean containsAll(java.util.Collection collection) {

    Iterator iter = collection.iterator();
    while (iter.hasNext()) {
      if (!contains(iter.next())) {
        return false;
      }
    }

    return true;
  }

  public void clear() {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean addAll(int A, java.util.Collection B) {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean removeAll(java.util.Collection A) {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean retainAll(java.util.Collection A) {
    throw new java.lang.UnsupportedOperationException();
  }

  public java.lang.Object set(int A, java.lang.Object B) {
    throw new java.lang.UnsupportedOperationException();
  }

  public boolean equals(java.lang.Object other) {

    if (other instanceof List) {

      List olist = (List)other;
      if (m_nodes.length != olist.size()) {
        return false;
      }

      for (int i = 0; i < m_nodes.length; ++i) {
        if (!m_nodes[i].equals(olist.get(i))) {
          return false;
        }
      }
      return true;
    }
    return false;
  }

  public int hashCode() {
    int hash = 1;
    for (int i = 0; i < m_nodes.length; ++i) {
      Node node = m_nodes[i];
      hash = 31*hash + (node != null? 0 : node.hashCode());
    }
    return hash;
  }

  public java.lang.Object get(int index) {
    return m_nodes[index];
  }

  public int lastIndexOf(java.lang.Object A) {
    for (int i = m_nodes.length-1; i >= 0; --i) {
      if (m_nodes[i].equals(A)) {
        return i;
      }
    }
    return -1;
  }

  public void add(int A, java.lang.Object B) {
    throw new java.lang.UnsupportedOperationException();
  }

  public java.lang.Object remove(int A) {
    throw new java.lang.UnsupportedOperationException();
  }

  public int indexOf(java.lang.Object A) {
    for (int i = 0; i < m_nodes.length; ++i) {
      if (m_nodes[i].equals(A)) {
        return i;
      }
    }
    return -1;
  }

  public java.util.ListIterator listIterator() {
    return new FixedNodeIterator();
  }

  public java.util.ListIterator listIterator(int index) {
    return new FixedNodeIterator(index);
  }

  public java.util.List subList(int A, int B) {
    throw new java.lang.UnsupportedOperationException();
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("[");
    for (int i = 0;  i < m_nodes.length; ++i) {
      if (i != 0) {
        sb.append(", ");
      }
      sb.append(m_nodes[i]);
    }
    sb.append("]");
    return sb.toString();
  }
}

