package com.sapportals.wcm.protocol.webdav.dompath;

/**
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: MultiExpression.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class MultiExpression implements IExpression {

  private final int UNKNOWN = 0;
  private final int MULTIPLY = 1;
  private final int DIV = 2;
  private final int MOD= 3;

  private final IExpression m_left;
  private final IExpression m_right;
  private final int m_operator;

  MultiExpression(IExpression left, String op, IExpression right) {
    m_left = left;
    m_right = right;
    m_operator = (op.equals("*")? MULTIPLY :
      (op.equals("div")? DIV :
      (op.equals("mod")? MOD : UNKNOWN)));
    if (m_operator == UNKNOWN) {
      throw new IllegalArgumentException("unknown multiplication operator: "+op);
    }
  }

  public IContext evaluate(IContext context) {
    double left = m_left.evaluate(context).toNumber();
    double right = m_right.evaluate(context).toNumber();

    double result = 0.0;
    switch (m_operator) {
    case MULTIPLY:
      result = left * right;
      break;
    case DIV:
      result = left / right;
      break;
    case MOD:
      result = ((int)left) % ((int)right);
      break;
    }
    return context.newInstance(result);
  }

  public boolean evaluateBoolean(IContext context) {
    return evaluate(context).toBoolean();
  }

  public int getType() {
    return IContext.TYPE_NUMBER;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    string(m_left, sb);
    switch (m_operator) {
    case MULTIPLY:
      sb.append(" * ");
      break;
    case DIV:
      sb.append(" div ");
      break;
    case MOD:
      sb.append(" mod ");
      break;
    }
    string(m_right, sb);
    return sb.toString();
  }

  private void string(IExpression expr, StringBuffer sb) {
    if (expr instanceof AdditiveExpression) {
      sb.append("(").append(expr).append(")");
    }
    else {
      sb.append(expr);
    }
  }
}
