package com.sapportals.wcm.protocol.webdav.dompath;

import org.w3c.dom.Node;


/**
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: ParentStep.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class ParentStep implements IStep {

  private final INodeTest m_test;
  private IStep m_next;
  private IExpression m_predicate;

  protected ParentStep(INodeTest test) {
    m_test = test;
  }

  public Node[] match(Node[] nodes, IContext context) {

    Result result = new Result(context);
    for (int j = 0; j < nodes.length; ++j) {
      Node parent = context.getParent(nodes[j]);
      if (m_test == null || m_test.matches(parent)) {
        result.add(parent);
      }
    }

    if (m_predicate != null) {
      context = m_predicate.evaluate(result.toContext());
      return (m_next != null)?
        m_next.match(context.getNodes(), context)
      : context.getNodes();
    }
    else {
      return (m_next != null)?
        m_next.match(result.getNodes(), context)
      : result.getNodes();
    }
  }

  public Node matchFirst(Node context) {

    context = PathContext.GetParent(context);
    return (m_next != null)? m_next.matchFirst(context) : context;
  }

  public void setNext(IStep next) {
    m_next = next;
  }

  public void setPredicate(IExpression expression) {
    m_predicate = expression;
  }

  public boolean containsPredicate() {
    return (m_predicate != null
            || (m_next != null && m_next.containsPredicate()));
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("..");
    if (m_predicate != null) {
      sb.append(m_predicate.toString());
    }
    if (m_next != null) {
      sb.append('/').append(m_next.toString());
    }
    return sb.toString();
  }
}
