package com.sapportals.wcm.protocol.webdav.dompath;

/**
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: PredicateExpression.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class PredicateExpression implements IExpression {

  private final IExpression m_expr;
  private PredicateExpression m_next;

  PredicateExpression(IExpression expr) {
    m_expr = expr;
  }

  public IContext evaluate(IContext context) {
    //System.err.println("evaluate predicate: "+this);
    context.resetPosition();
    Result result = new Result(context);
    switch (m_expr.getType()) {

    case IContext.TYPE_NUMBER:
      while (context.nextPosition()) {
        IContext eval = m_expr.evaluate(context);
        if (eval.toNumber() == context.getPosition()) {
          result.add(context.getCurrentNode());
        }
      }
      break;

    default:
      while (context.nextPosition()) {
        if (m_expr.evaluateBoolean(context)) {
          result.add(context.getCurrentNode());
        }
      }
      break;
    }

    if (m_next != null && !result.isEmpty()) {
      return m_next.evaluate(result.toContext());
    }
    return result.toContext();
  }

  public boolean evaluateBoolean(IContext context) {
    //System.err.println("evaluateBoolean predicate: "+this);
    if (m_next != null) {
      return evaluate(context).toBoolean();
    }
    else {
      context.resetPosition();

      switch (m_expr.getType()) {

      case IContext.TYPE_NUMBER:
        while (context.nextPosition()) {
          IContext eval = m_expr.evaluate(context);
          if (eval.toNumber() == context.getPosition()) {
            return true;
          }
        }
        break;

      default:
        while (context.nextPosition()) {
          if (m_expr.evaluateBoolean(context)) {
            return true;
          }
        }
        break;
      }

      return false;
    }
  }

  void setNext(PredicateExpression next) {
    m_next = next;
  }

  public int getType() {
    return IContext.TYPE_NODESET;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append('[').append(m_expr.toString()).append(']');
    if (m_next != null) {
      sb.append(m_next.toString());
    }
    return sb.toString();
  }
}
