package com.sapportals.wcm.protocol.webdav.dompath;

/**
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: RelationalExpression.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class RelationalExpression implements IExpression {

  private final int UNKNOWN = 0;
  private final int GT = 1;
  private final int GE = 2;
  private final int LT = 3;
  private final int LE = 4;

  private final IExpression m_left;
  private final IExpression m_right;
  private final int m_operator;

  RelationalExpression(IExpression left, String op, IExpression right) {
    m_left = left;
    m_right = right;
    m_operator = (op.equals(">")? GT : (op.equals(">=")? GE :
      (op.equals("<")? LT : (op.equals("<=")? LE : UNKNOWN))));

    if (m_operator == UNKNOWN) {
      throw new IllegalArgumentException("unknown compare operator: "+op);
    }
  }

  public IContext evaluate(IContext context) {
    return context.newInstance(evaluateBoolean(context));
  }

  public boolean evaluateBoolean(IContext context) {
    IContext left = m_left.evaluate(context);
    IContext right = m_right.evaluate(context);

    boolean result = false;
    switch (m_operator) {
    case GT:
      result = left.greaterThan(right);
      break;
    case GE:
      result = left.greaterOrEqual(right);
      break;
    case LT:
      result = right.greaterThan(left);
      break;
    case LE:
      result = right.greaterOrEqual(left);
      break;
    }
    return result;
  }

  public int getType() {
    return IContext.TYPE_BOOLEAN;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(m_left);
    switch (m_operator) {
    case GT:
      sb.append(" > ");
      break;
    case GE:
      sb.append(" >= ");
      break;
    case LT:
      sb.append(" < ");
      break;
    case LE:
      sb.append(" <= ");
      break;
    }
    sb.append(m_right);
    return sb.toString();
  }

}
