package com.sapportals.wcm.protocol.webdav.dompath;

import org.w3c.dom.Node;

/**
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @version $Id: RootStep.java,v 1.2 2004/03/03 17:44:19 jre Exp $
 */
class RootStep implements IStep {

  // private final INodeTest m_test;
  private IStep m_next;
  private IExpression m_predicate;

  protected RootStep(INodeTest test) {
    // m_test = test;
  }

  public Node[] match(Node[] nodes, IContext context) {

    Result result = new Result(context);
    for (int i = 0; i < nodes.length; ++i) {
      Node node = nodes[i];
      if (node.getNodeType() == Node.DOCUMENT_NODE) {
        result.add(node);
      }
      else {
        Node doc = node.getOwnerDocument();
        result.add(doc);
      }
    }

    if (m_predicate != null) {
      context = m_predicate.evaluate(result.toContext());
      return (m_next != null)?
        m_next.match(context.getNodes(), context) : context.getNodes();
    }
    else {
      return (m_next != null)?
        m_next.match(result.getNodes(), context) : result.getNodes();
    }
  }

  public Node matchFirst(Node context) {

    if (context.getNodeType() == Node.DOCUMENT_NODE) {
      return (m_next != null)? m_next.matchFirst(context) : context;
    }
    else {
      context = (Node)context.getOwnerDocument();
      return (m_next != null && context != null)? m_next.matchFirst(context) : context;
    }
  }

  public void setNext(IStep next) {
    m_next = next;
  }

  public void setPredicate(IExpression expression) {
    m_predicate = expression;
  }

  public boolean containsPredicate() {
    return (m_predicate != null
            || (m_next != null && m_next.containsPredicate()));
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append('/');
    if (m_predicate != null) {
      sb.append(m_predicate.toString());
    }
    if (m_next != null) {
      sb.append(m_next.toString());
    }
    return sb.toString();
  }
}
