package com.sapportals.wcm.protocol.webdav.dompath;

import org.w3c.dom.Node;

/**
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: SelfOrAncestorStep.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
class SelfOrAncestorStep implements IStep {

  private final boolean m_includeSelf;
  private final INodeTest m_test;
  private IStep m_next;
  private IExpression m_predicate;

  protected SelfOrAncestorStep(INodeTest test) {
    this(test, true);
  }

  protected SelfOrAncestorStep(INodeTest test, boolean includeSelf) {
    m_test = test;
    m_includeSelf = includeSelf;
  }

  public Node[] match(Node[] nodes, IContext context) {

    Result result = new Result(context);
    if (m_predicate != null) {
      Result intermediate = new Result(context);
      for (int i = 0; i < nodes.length; ++i) {
        intermediate.clear();
        Node node = nodes[i];
        if (m_includeSelf) {
          if (m_test == null) {
            if (node.getNodeType() == Node.ELEMENT_NODE)
              intermediate.add(node);
          }
          else if (m_test.matches(node)) {
            intermediate.add(node);
          }
        }
        addAncestors(node, intermediate);

        IContext eval = m_predicate.evaluate(intermediate.toContext());
        result.add(eval.getNodes());
      }
    }
    else {
      for (int i = 0; i < nodes.length; ++i) {
        Node node = nodes[i];
        if (m_includeSelf) {
          if (m_test == null) {
            if (node.getNodeType() == Node.ELEMENT_NODE)
              result.add(node);
          }
          else if (m_test.matches(node)) {
            result.add(node);
          }
        }
        addAncestors(node, result);
      }
    }

    return (m_next != null)?
      m_next.match(result.getNodes(), context) : result.getNodes();
  }

  public Node matchFirst(Node context) {

    Node found = null;
    if (m_test != null || context.getNodeType() == Node.ELEMENT_NODE) {
      if (m_next != null) {
        if (m_includeSelf) {
          if (m_test == null || m_test.matches(context)) {
            found = m_next.matchFirst(context);
          }
        }
        if (found == null) {
          found = matchFirstAncestor(context);
        }
      }
      else if (m_includeSelf) {
        if (m_test == null || m_test.matches(context)) {
          found = context;
        }
      }
    }

    return found;
  }

  public void setNext(IStep next) {
    m_next = next;
  }

  public void setPredicate(IExpression expression) {
    m_predicate = expression;
  }

  public boolean containsPredicate() {
    return (m_predicate != null
            || (m_next != null && m_next.containsPredicate()));
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append('/');
    if (m_next != null) {
      sb.append(m_next.toString());
    }
    else {
      sb.append('/');
    }
    if (m_predicate != null) {
      sb.append(m_predicate.toString());
    }
    return sb.toString();
  }

  private void addAncestors(Node node, Result result) {
    for (Node parent = PathContext.GetParent(node); parent != null;
         parent = PathContext.GetParent(parent)) {
      if (m_test == null || m_test.matches(parent)) {
        result.add(parent);
      }
    }
  }

  private Node matchFirstAncestor(Node node) {
    for (Node parent = PathContext.GetParent(node); parent != null;
         parent = PathContext.GetParent(parent)) {
      if (m_test == null || m_test.matches(parent)) {
        Node found = m_next.matchFirst(parent);
        if (found != null) return found;
      }
    }
    return null;
  }
}
