package com.sapportals.wcm.protocol.webdav.dompath;

import org.w3c.dom.Node;


/**
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: SelfStep.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */

class SelfStep implements IStep {

  private final INodeTest m_test;
  private IStep m_next;
  private IExpression m_predicate;

  protected SelfStep(INodeTest test) {
    m_test = test;
  }

  public Node[] match(Node[] nodes, IContext context) {

    if (m_test != null || m_predicate != null) {
      Result result = new Result(context);

      for (int i = 0; i < nodes.length; ++i) {
        Node node = nodes[i];
        if (m_test != null) {
          if (m_test.matches(node)) {
            if (m_predicate != null) {
              IContext eval = m_predicate.evaluate(context.newInstance(node));
              result.add(eval.getNodes());
            }
            else {
              result.add(node);
            }
          }
        }
        else {
          if (m_predicate != null) {
            IContext eval = m_predicate.evaluate(context.newInstance(node));
            result.add(eval.getNodes());
          }
          else {
            // cannot happen
            throw new IllegalStateException("should not be reached");
          }
        }
      }

      return (m_next != null)?
        m_next.match(result.getNodes(), context) : result.getNodes();
    }
    else {
      return (m_next != null)?
        m_next.match(context.getNodes(), context) : context.getNodes();
    }
  }

  public Node matchFirst(Node context) {

    if (m_test != null && !m_test.matches(context)) {
      return null;
    }
    return (m_next != null)? m_next.matchFirst(context) : context;
  }

  public void setNext(IStep next) {
    m_next = next;
  }

  public void setPredicate(IExpression expression) {
    m_predicate = expression;
  }

  public boolean containsPredicate() {
    return (m_predicate != null
            || (m_next != null && m_next.containsPredicate()));
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append('.');
    if (m_predicate != null) {
      sb.append(m_predicate.toString());
    }
    if (m_next != null) {
      sb.append('/').append(m_next.toString());
    }
    return sb.toString();
  }

}
