package com.sapportals.wcm.protocol.webdav.dompath;

import java.util.HashMap;
import java.util.Map;

/**
 * Definition of package internal constants.
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @version $Id: UnicodeString.java,v 1.2 2004/03/03 17:44:19 jre Exp $
 */

public final class UnicodeString {

  private final int MIN_ESCAPE = 0xd800;
  private final int MAX_ESCAPE = 0xdbff;

  private final String m_s;
  private final int m_jlength;
  private final int m_ulength;
  private final boolean m_hasEscapes;

  public UnicodeString(String s) {
    m_s = s;
    m_jlength = m_s.length();
    m_ulength = ulength();
    m_hasEscapes = (m_jlength != m_ulength);
  }

  public final int length() {
    return m_ulength;
  }

  public final String substr(int from) {
    if (m_hasEscapes) {
      int jfrom = jposition(from);
      return m_s.substring(jfrom);
    }
    else {
      return m_s.substring(from);
    }
  }

  public final String substr(int from, int to) {
    if (m_hasEscapes) {
      int jfrom = jposition(from);
      int jto = jposition(to);
      return m_s.substring(jfrom, jto);
    }
    else {
      return m_s.substring(from, to);
    }
  }

  public final String toString() {
    return m_s;
  }

  public final String translate(String from, String to) {
    Map map = buildTranslationMap(from, to);

    StringBuffer sb = new StringBuffer(m_jlength);
    String trans;
    char[] composite = new char[2];

    for (int i = 0; i < m_jlength; ++i) {
      int c = m_s.charAt(i);
      if (c < MIN_ESCAPE || c > MAX_ESCAPE || (i+1 >= m_jlength)) {
        trans = (String)map.get(new Character((char)c));
        if (trans != null)
          sb.append(trans);
        else
          sb.append((char)c);
      }
      else {
        composite[0] = (char)c;
        composite[1] = m_s.charAt(++i);
        trans = (String)map.get(new String(composite));
        if (trans != null)
          sb.append(trans);
        else
          sb.append(composite);
      }
    }
    //System.out.println("translate ("+m_s+") is "+sb.toString()+" map "+map);
    return sb.toString();
  }

  public final boolean equals(Object other) {
    if (other == null) {
      return false;
    }
    else {
      if (other instanceof UnicodeString) {
        return m_s.equals(other.toString());
      }
      else {
        return m_s.equals(other);
      }
    }
  }

  private final int jposition(int upos) {
    for (int i = 0; i < m_jlength; ++i) {
      int c = m_s.charAt(i);
      if (c < MIN_ESCAPE || c > MAX_ESCAPE) {
        if (--upos < 0) {
          return i;
        }
      }
    }
    return 0;
  }

  private final int ulength() {
    int ulength = 0;
    for (int i = 0; i < m_jlength; ++i) {
      int c = m_s.charAt(i);
      if (c < MIN_ESCAPE || c > MAX_ESCAPE) ++ulength;
    }
    return ulength;
  }

  private final Map buildTranslationMap(String from, String to) {
    int flen = from.length();
    int tlen = to.length();
    Map map = new HashMap(flen*5/3);
    char[] composite = new char[2];
    int tindex = 0;
    StringBuffer sb = new StringBuffer(tlen);

    for (int i = 0; i < flen; ++i) {
      int c = from.charAt(i);
      String target = "";
      if (tindex < tlen) {
        sb.setLength(0);
        int ct = to.charAt(tindex++);
        sb.append((char)ct);
        if (ct < MIN_ESCAPE || ct > MAX_ESCAPE || tindex >= tlen) {
          target = sb.toString();
        }
        else {
          sb.append(to.charAt(tindex++));
          target = sb.toString();
        }
      }

      if (c < MIN_ESCAPE || c > MAX_ESCAPE || (i+1 >= flen)) {
        map.put(new Character((char)c), target);
      }
      else {
        composite[0] = (char)c;
        composite[1] = from.charAt(++i);
        map.put(new String(composite), target);
      }
    }
    return map;
  }
}
