/*
 * Copyright (c) 2004 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: HTMLForm.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
package com.sapportals.wcm.protocol.webdav.html;

import java.util.ArrayList;
import java.util.List;

import com.sapportals.wcm.util.uri.IHierarchicalUri;

/**
 * Represents HTML Form.<p>
 *
 * Copyright (c) SAP AG 2001-2004
 *
 * @author stefan.eissing@greenbytes.de
 * @version $Id: HTMLForm.java,v 1.1.1.1 2004/02/18 13:00:22 sync Exp $
 */
final class HTMLForm {

  public static final String URLENCODING = "application/x-www-form-urlencoded";
  public static final String MULTIPART = "multipart/form-data";
  public static final String DEFAULT_ENC_TYPE = URLENCODING;
  
  public static HTMLForm getInstance(String id, IHierarchicalUri action, String method, String encType) {
    return new HTMLForm(id, action, method, encType);
  }
    
  public final String id;
  public final IHierarchicalUri action;
  public final String method;
  public final String encType;
  public final List inputs;
    
  private HTMLForm(String id, IHierarchicalUri action, String method, String encType) {
    this.id = id;
    this.action = action;
    this.method = method;
    this.encType = encType;
    this.inputs = new ArrayList(7);
  }
    
  public boolean isValidLogin() {
    boolean hasPassword = false;
    boolean hasText = false;
    boolean hasSingleText = true;
    for (int i = 0, n = this.inputs.size(); i < n; ++i) {
      HTMLInput input = (HTMLInput)this.inputs.get(i);
      String type = input.getType();
      if (HTMLInput.PASSWORD.equals(type)) {
        hasPassword = true;
      }
      else if (HTMLInput.TEXT.equals(type)) {
        if (hasText) {
          hasSingleText = false;
        }
        hasText = true;
      }
    }
    return (hasPassword && hasText && hasSingleText);
  }
    
  public void addInput(String name, String type, String value) {
    this.inputs.add(HTMLInput.getInstance(name, type, value));
  }
}
  
