/*
 * Copyright (c) 2004 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: IHTMLLogin.java,v 1.2 2004/03/03 17:44:19 jre Exp $
 */
package com.sapportals.wcm.protocol.webdav.html;

import java.util.List;

import com.sapportals.wcm.repository.ResourceException;
import com.sapportals.wcm.util.http.HttpRequest;
import com.sapportals.wcm.util.http.UserInfo;
import com.sapportals.wcm.util.uri.IHierarchicalUri;

/**
 * Represents a HTML login form. <p>
 *
 * Copyright (c) SAP AG 2001-2004
 *
 * @author stefan.eissing@greenbytes.de
 * @version $Id: IHTMLLogin.java,v 1.2 2004/03/03 17:44:19 jre Exp $
 */
public interface IHTMLLogin {

  /**
   * Get id of login form iff set. Otherwise returns <code>null</code>.
   * @return id of login form
   */
  String getID();
  
  /**
   * Get uri of login form action.
   * @return uri of form action
   */
  IHierarchicalUri getAction();
  
  /**
   * Get method of login form submittal.
   * @return method of login form submittal
   */
  String getMethod();
  
  /**
   * Get encoding mime type for submitted form data.
   * @return encoding mime type for submitted form data
   */
  String getEncType();
  
  /**
   * Get list of {@link HTMLInput}s of login form.
   * @return list of {@link HTMLInput}s of login form
   */
  List getInputs();
  
  /**
   * Create a http request object suitable for sending the login form data
   * @param ui username and password to use
   * @return http request object suitable for sending the login form data
   */
  HttpRequest createRequest(UserInfo ui) throws ResourceException;
}
