package com.sapportals.wcm.protocol.webdav.repository;

/**
 * Map WCM lock tokens to URIs and vice versa.
 * <p>
 * Works by tunneling non "opaquelocktoken:" URIs through the
 * "opaquelocktoken:" URI scheme using a special UUID.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author julian.reschke@greenbytes.de
 * @version $Revision: 1.1.1.1 $
 */

public class WDLockTokenMapper {

  private static final String LOCKTOKEN_SCHEME = "opaquelocktoken:";
  private static final String UUID = "58D07A8B-986A-4C8A-A93D-F545055499DD-";
  private static final String INVALID_UUID = "58D07A8B-986A-4C8A-A93D-F545055499DE-";

  public static String getUriFromToken(String lockToken) {
    if (lockToken.startsWith(UUID)) {
      return lockToken.substring(UUID.length());
    }
    else {
      return LOCKTOKEN_SCHEME + lockToken;
    }
  }

  public static String getTokenFromUri(String uri) {
    if (uri.startsWith(LOCKTOKEN_SCHEME)) {
      return uri.substring(LOCKTOKEN_SCHEME.length());
    }
    else {
      return UUID + uri;
    }
  }

  public static String getTokenFromCodedUri(String uri) {
    int len = uri.length();
    if (len <= 2)
      throw new IllegalArgumentException("coded uri must have length > 2: " + uri);
    if (uri.charAt(0) != '<' || uri.charAt(len - 1) != '>') {
      throw new IllegalArgumentException("coded uri must be surrounded with <>: " + uri);
    }

    uri = uri.substring(1, len - 1);
    if (uri.startsWith(LOCKTOKEN_SCHEME)) {
      return uri.substring(LOCKTOKEN_SCHEME.length());
    }
    else {
      return UUID + uri;
    }
  }
  
  /**
   * @return a lock token that represents an unknown lock token (each will be generated exactly once, thus it will never
   * match any existing lock token (either valid or not)
   */
  public static String createInvalidLockToken() {
    return getNextInvalidToken();
  }
  
  /**
   * @param locktoken
   * @return whether the lock token is invalid (being a place holder token for a lock for which the real lock token is unknown)
   */
  public static boolean isInvalid(String locktoken) {
    return locktoken.startsWith(INVALID_UUID);
  }
  
  // ---- generation of unique "invalid" lock tokens

  static String pref = new com.sapportals.wcm.util.uuid.UUID().toString() + ":";
  static long lockTokenCounter = 1;

  private static  String getNextInvalidToken() {
    synchronized (pref) {
      return INVALID_UUID + pref + (lockTokenCounter++);
    }
  }

}
