package com.sapportals.wcm.protocol.webdav.repository;

import com.sap.security.api.UMFactory;
import com.sap.tc.logging.Location;
import com.sapportals.portal.security.usermanagement.*;
import com.sapportals.wcm.WcmException;
import com.sapportals.wcm.repository.IResource;
import com.sapportals.wcm.repository.IResourceContext;
import com.sapportals.wcm.repository.ResourceException;
import com.sapportals.wcm.repository.ResourceFactory;
import com.sapportals.wcm.service.IServiceTypesConst;
import com.sapportals.wcm.service.urlgenerator.IURLGeneratorService;
import com.sapportals.wcm.util.uri.IRidSet;
import com.sapportals.wcm.util.uri.RID;
import com.sapportals.wcm.util.usermanagement.WPUMFactory;

/**
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @version $Revision: 1.3 $
 */

public class WDPrincipals {

  private static final Location log = Location.getLocation(WDPrincipals.class);

  private static class My {

    static final IURLGeneratorService generator;
    static final IUserFactory userFactory;
    static final IGroupFactory groupFactory;
    static final IRoleFactory roleFactory;
    static final IUser anonymous;
    static final IGroup everybody;
    static final IGroup authenticated;
    static final IGroup anonymous_group;
    static final IRidSet principalCollectionSet;

    static {
      userFactory = WPUMFactory.getUserFactory();
      groupFactory = WPUMFactory.getGroupFactory();
      roleFactory = WPUMFactory.getRoleFactory();

      IURLGeneratorService l_generator = null;
      IUser l_anonymous = null;
      IGroup l_everybody = null;
      IGroup l_authenticated = null;
      IGroup l_anonymous_group = null;
      IRidSet l_collections = null;

      try {
        com.sap.security.api.IGroupFactory gf = UMFactory.getGroupFactory();
        try {
          com.sap.security.api.IGroup g6 = gf.getGroup(com.sap.security.api.IGroupFactory.AUTHENTICATED_USERS_UNIQUEID);
          l_authenticated = groupFactory.getEP5Group(g6);
        }
        catch (Exception ex) {
          log.errorT("getting authenticated users group" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
        }
        
        try { 
          com.sap.security.api.IGroup g6 = gf.getGroup(com.sap.security.api.IGroupFactory.ANONYMOUS_USERS_UNIQUEID);
          l_anonymous_group = groupFactory.getEP5Group(g6);
        }
        catch (Exception ex) {
          log.errorT("getting anonymous users group" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
        }

        try {      
          l_anonymous = userFactory.getUser(IUser.ANONYMOUS);
        }
        catch (Exception ex) {
          log.errorT("getting anonymous user" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
        }
        
        try {
          l_everybody = groupFactory.getGroup(IGroup.EVERYBODY);
        }
        catch (Exception ex) {
          log.errorT("getting group everybody" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
        }
        
        l_generator = (IURLGeneratorService)ResourceFactory.getInstance().getServiceFactory().getService(IServiceTypesConst.URLGENERATOR_SERVICE);
        l_collections = l_generator.getPrincipalCollectionRidSet();
      }
      catch (Exception ex) {
        log.errorT("getting URLGenerator user management" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      }

      anonymous = l_anonymous;
      everybody = l_everybody;
      generator = l_generator;
      principalCollectionSet = l_collections;
      authenticated = l_authenticated;
      anonymous_group = l_anonymous_group;
    }
  }

  public static IRidSet getPrincipalCollectionSet() {
    return My.principalCollectionSet;
  }

  public static RID getRid(IUMPrincipal principal) {
    try {
      switch (principal.getType()) {

        case IUMPrincipal.IUSER:
          return My.generator.getUserResourceRID((IUser)principal);

        case IUMPrincipal.IGROUP:
          return My.generator.getGroupResourceRID((IGroup)principal);

        case IUMPrincipal.IROLE:
          return My.generator.getRoleResourceRID((IRole)principal);

        default:
          break;
      }
    }
    catch (WcmException ex) {
      log.warningT("cannot get RID for principal "+principal + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
    }
    return null;
  }

  public static RID getUserRid (String uid) {
    try {
      return My.generator.getUserResourceRID(uid);
    }
    catch (WcmException ex) {
      log.warningT("cannot get RID for uid "+uid + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static RID getGroupRid (String gid) {
    try {
      return My.generator.getGroupResourceRID(gid);
    }
    catch (WcmException ex) {
      log.warningT("cannot get RID for gid "+gid + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static RID getRoleRid (String rid) {
    try {
      return My.generator.getRoleResourceRID(rid);
    }
    catch (WcmException ex) {
      log.warningT("cannot get RID for rid "+rid + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static IUMPrincipal getPrincipal(RID rid, IResourceContext ctx) {
    try {
      IResource res = ResourceFactory.getInstance().getResource(rid, ctx);

      if (res != null) {
        IUMPrincipal prin = (IUMPrincipal) res.as(IUMPrincipal.class);
        if (prin == null) {
          log.warningT("found, but not a IUMPrincipal: " + rid);
        }
        return prin;
      }
      else {
        log.warningT("not found: " + rid);
        return null;
      }
    }
    catch (ResourceException ex) {
      log.warningT("cannot get resource: "+rid + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static IUser getUserByID (String id) {
    try {
      return My.userFactory.getUser(id);
    }
    catch (com.sapportals.portal.security.usermanagement.UserManagementException ex) {
      log.warningT("cannot determine user: "+id + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static IGroup getGroupByID (String id) {
    try {
      return My.groupFactory.getGroup(id);
    }
    catch (com.sapportals.portal.security.usermanagement.UserManagementException ex) {
      log.warningT("cannot determine group: "+id + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static IRole getRoleByID (String id) {
    try {
      return My.roleFactory.getRole(id);
    }
    catch (com.sapportals.portal.security.usermanagement.UserManagementException ex) {
      log.warningT("cannot determine role: "+id + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      return null;
    }
  }

  public static IUMPrincipal getAnonymous() {
    return My.anonymous_group;
  }

  public static IUMPrincipal getEverybody() {
    return My.everybody;
  }

  public static IUMPrincipal getAuthenticated() {
    return My.authenticated;
  }

}