package com.sapportals.wcm.protocol.webdav.repository;

import java.util.*;

import com.sap.tc.logging.Location;
import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.Name;
import com.sapportals.wcm.repository.IPropertyName;
import com.sapportals.wcm.repository.PropertyDef;
import com.sapportals.wcm.repository.PropertyName;
import com.sapportals.wcm.repository.ResourceException;
import com.sapportals.wcm.repository.enum.PropertyType;

/**
 * Generic property mapping information.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author stefan.eissing@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: WDPropertyDef.java,v 1.7 2004/06/14 14:22:58 jre Exp $
 */

public class WDPropertyDef {

  private static final Location log = Location.getLocation(WDPropertyDef.class);

  public static final IPropertyName CommentName;
  public static final IPropertyName SupportedLivePropertySetName;

  public static final WDPropertyDef creationdate;
  public static final WDPropertyDef current_user_privilege_set;
  public static final WDPropertyDef displayname;
  public static final WDPropertyDef getcontentlanguage;
  public static final WDPropertyDef getcontentlength;
  public static final WDPropertyDef getcontenttype;
  public static final WDPropertyDef getetag;
  public static final WDPropertyDef getlastmodified;
  public static final WDPropertyDef ishidden;
  public static final WDPropertyDef lockdiscovery;
  public static final WDPropertyDef orderingtype; // deprecate me
  public static final WDPropertyDef ordering_type;
  public static final WDPropertyDef redirect_lifetime;
  public static final WDPropertyDef reftarget;
  public static final WDPropertyDef resourcetype;
  public static final WDPropertyDef supportedlock;
  
  public static final WDPropertyDef sap_property_displayname_set;
  public static final WDPropertyDef sap_appproperties;
  public static final WDPropertyDef sap_personalized_appproperties;
  public static final WDPropertyDef sap_repository;

  // deltaV properties

  public static final WDPropertyDef auto_update;
  public static final WDPropertyDef auto_version;
  public static final WDPropertyDef checked_in;
  public static final WDPropertyDef checked_out;
  public static final WDPropertyDef checkin_fork;
  public static final WDPropertyDef checkout_fork;
  public static final WDPropertyDef checkout_set;
  public static final WDPropertyDef comment;
  public static final WDPropertyDef creator_displayname;
  public static final WDPropertyDef label_name_set;
  public static final WDPropertyDef predecessor_set;
  public static final WDPropertyDef root_version;
  public static final WDPropertyDef successor_set;
  public static final WDPropertyDef supported_live_property_set;
  public static final WDPropertyDef supported_method_set;
  public static final WDPropertyDef supported_query_grammar_set;
  public static final WDPropertyDef supported_report_set;
  public static final WDPropertyDef eclipsed_set;
  public static final WDPropertyDef version_controlled_binding_set;
  public static final WDPropertyDef version_history;
  public static final WDPropertyDef version_history_collection_set;
  public static final WDPropertyDef version_name;
  public static final WDPropertyDef version_set;
  public static final WDPropertyDef workspace;
  public static final WDPropertyDef workspace_checkout_set;
  public static final WDPropertyDef workspace_collection_set;

  // ACL properties

  public static final WDPropertyDef acl;
  public static final WDPropertyDef acl_restrictions;
  public static final WDPropertyDef acl_semantics;
  public static final WDPropertyDef group;
  public static final WDPropertyDef group_membership;
  public static final WDPropertyDef group_member_set;
  public static final WDPropertyDef inherited_acl_set;
  public static final WDPropertyDef owner;
  public static final WDPropertyDef principal_collection_set;
  public static final WDPropertyDef supported_privilege_set;

  // Principal properties

  public static final WDPropertyDef alternate_URI_set;
  public static final WDPropertyDef principal_URL;
  public static final WDPropertyDef principal_details;

  // BIND properties
  public static final WDPropertyDef parent_set;
  public static final WDPropertyDef resource_id;

  /** Unmodifiable Set of all WDPropertyDef objects */
  public static final Set all;
  /** Unmodifiable Set of all WDPropertyDef objects visible on a PROPFIND/allprop request */
  public static final Set allprop;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on plain resources */
  public static final Set liveResourceProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on plain resources on workspace aware servers */
  public static final Set liveWorkspaceAwareResourceProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on resources inside workspaces */
  public static final Set liveWorkspaceControlledResourceProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on plain resources on version history aware servers */
  public static final Set liveVersionHistoryAwareResourceProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on link resources */
  public static final Set liveLinkProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on checked out resources */
  public static final Set liveCheckedOutProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on checked in resources */
  public static final Set liveCheckedInProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on version controlled resources */
  public static final Set liveVersionControlledProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on version controlled collection resources */
  public static final Set liveVersionControlledCollectionProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on collection version resources */
  public static final Set liveCollectionVersionProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on ordered collection version resources */
  public static final Set liveOrderedCollectionVersionProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on version resources */
  public static final Set liveVersionProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on version history resources */
  public static final Set liveVersionHistoryProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on working resources */
  public static final Set liveWorkingResourceProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on workspace resources */
  public static final Set liveWorkspaceResourceProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on resource with ACL */
  public static final Set liveAclProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on resource of type principal */
  public static final Set livePrincipalProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on resource that support BIND's live properties */
  public static final Set liveBindProp;
  /** Unmodifiable Set of all live WDPropertyDef objects supported on ordered collection resource */
  public static final Set liveOrderedCollectionProp;
  /** Unmodifiable Set of all known live WDPropertyDef objects */
  public static final Set liveProp;

  private static final Set all_internal;
  private static final Set allprop_internal;
  private static final Map lookupMap;
  private static final Map inverseNameMap;
  private static final Set hideInternal;

  static {
    IPropertyName l_Comment = null;
    IPropertyName l_SupportedLivePropertySet = null;
    WDPropertyDef l_creationdate = null;
    WDPropertyDef l_current_user_privilege_set = null;
    WDPropertyDef l_displayname = null;
    WDPropertyDef l_getcontentlanguage = null;
    WDPropertyDef l_getcontentlength = null;
    WDPropertyDef l_getcontenttype = null;
    WDPropertyDef l_getetag = null;
    WDPropertyDef l_getlastmodified = null;
    WDPropertyDef l_group = null;
    WDPropertyDef l_ishidden = null;
    WDPropertyDef l_lockdiscovery = null;
    WDPropertyDef l_orderingtype = null;
    WDPropertyDef l_ordering_type = null;
    WDPropertyDef l_owner = null;
    WDPropertyDef l_redirect_lifetime = null;
    WDPropertyDef l_reftarget = null;
    WDPropertyDef l_resourcetype = null;
    WDPropertyDef l_supportedlock = null;
    WDPropertyDef l_auto_update = null;
    WDPropertyDef l_auto_version = null;
    WDPropertyDef l_checked_in = null;
    WDPropertyDef l_checked_out = null;
    WDPropertyDef l_checkin_fork = null;
    WDPropertyDef l_checkout_fork = null;
    WDPropertyDef l_creator_displayname = null;
    WDPropertyDef l_comment = null;
    WDPropertyDef l_supported_query_grammar_set = null;
    WDPropertyDef l_supported_live_property_set = null;
    WDPropertyDef l_supported_method_set = null;
    WDPropertyDef l_supported_report_set = null;
    WDPropertyDef l_predecessor_set = null;
    // WDPropertyDef l_precursor_set = null;
    WDPropertyDef l_successor_set = null;
    WDPropertyDef l_checkout_set = null;
    WDPropertyDef l_version_name = null;
    WDPropertyDef l_root_version = null;
    WDPropertyDef l_eclipsed_set = null;
    WDPropertyDef l_version_controlled_binding_set = null;
    WDPropertyDef l_version_history = null;
    WDPropertyDef l_version_history_collection_set = null;
    WDPropertyDef l_workspace = null;
    WDPropertyDef l_workspace_checkout_set = null;
    WDPropertyDef l_workspace_collection_set = null;
    WDPropertyDef l_version_set = null;
    WDPropertyDef l_label_name_set = null;

    WDPropertyDef l_acl = null;
    WDPropertyDef l_alternate_URI_set = null;
    WDPropertyDef l_acl_restrictions = null;
    WDPropertyDef l_acl_semantics = null;
    WDPropertyDef l_group_membership = null;
    WDPropertyDef l_group_member_set = null;
    WDPropertyDef l_inherited_acl_set = null;
    WDPropertyDef l_principal_collection_set = null;
    WDPropertyDef l_principal_URL = null;
    WDPropertyDef l_principal_details = null;
    WDPropertyDef l_supported_privilege_set = null;

    // BIND
    WDPropertyDef l_parent_set = null;
    WDPropertyDef l_resource_id = null;

    WDPropertyDef l_sap_appproperties = null;
    WDPropertyDef l_sap_personalized_appproperties = null;
    WDPropertyDef l_sap_property_displayname_set = null;
    WDPropertyDef l_sap_repository = null;

    try {
      l_Comment = new PropertyName(Consts.NS_WEBDAV, Consts.EN_comment);
      l_SupportedLivePropertySet = new PropertyName(Consts.NS_WEBDAV, Consts.EN_supported_live_property_set);

      l_creationdate = new WDPropertyDef(Consts.N_creationdate, PropertyName.createCreationDate(),
        PropertyType.DATE, PropertyDef.createCreationDate().isReadonly(), PropertyDef.createCreationDate().isHidden());

      l_displayname = new WDPropertyDef(Consts.N_displayname, PropertyName.createDisplayname(),
        false, PropertyDef.createDisplayname().isHidden());

      l_getcontentlanguage = new WDPropertyDef(Consts.N_getcontentlanguage, PropertyName.createContentLanguage(),
        PropertyType.STRING, false, PropertyDef.createContentLanguage().isHidden());
      l_getcontentlength = new WDPropertyDef(Consts.N_getcontentlength, PropertyName.createContentLength(),
        PropertyType.LONG, true, PropertyDef.createContentLength().isHidden());
      l_getcontenttype = new WDPropertyDef(Consts.N_getcontenttype);
      l_getetag = new WDPropertyDef(Consts.N_getetag, PropertyName.createETag(),
        true, PropertyDef.createETag().isHidden());
      l_getlastmodified = new WDPropertyDef(Consts.N_getlastmodified, PropertyName.createLastModified(), PropertyType.DATE,
        false, PropertyDef.createLastModified().isHidden());

      l_ishidden = new WDPropertyDef(Consts.N_ishidden, PropertyName.createHidden(),
        PropertyType.BOOLEAN, false, PropertyDef.createHidden().isHidden());

      l_lockdiscovery = new WDPropertyDef (Consts.N_lockdiscovery);

      l_orderingtype = new WDPropertyDef (Consts.N_orderingtype);
      l_ordering_type = new WDPropertyDef (Consts.N_ordering_type);

      l_redirect_lifetime = new WDPropertyDef (Consts.N_redirect_lifetime);
      l_reftarget = new WDPropertyDef (Consts.N_reftarget);
      l_resourcetype = new WDPropertyDef (Consts.N_resourcetype);

      l_supportedlock = new WDPropertyDef (Consts.N_supportedlock);

      l_auto_update = new WDPropertyDef(Consts.N_auto_update);
      l_auto_version = new WDPropertyDef(Consts.N_auto_version);
      l_checked_in = new WDPropertyDef(Consts.N_checked_in);
      l_checked_out = new WDPropertyDef(Consts.N_checked_out);
      l_checkin_fork = new WDPropertyDef(Consts.N_checkin_fork);
      l_checkout_fork = new WDPropertyDef(Consts.N_checkout_fork);
      l_creator_displayname = new WDPropertyDef(Consts.N_creator_displayname);
      l_comment = new WDPropertyDef(Consts.N_comment, l_Comment, false, false);
      l_supported_query_grammar_set = new WDPropertyDef(Consts.N_supported_query_grammar_set);
      l_supported_live_property_set = new WDPropertyDef(Consts.N_supported_live_property_set, null, true, true);
      l_supported_method_set = new WDPropertyDef(Consts.N_supported_method_set);
      l_supported_report_set = new WDPropertyDef(Consts.N_supported_report_set);
      l_predecessor_set = new WDPropertyDef(Consts.N_predecessor_set);
      l_successor_set = new WDPropertyDef(Consts.N_successor_set);
      l_checkout_set = new WDPropertyDef(Consts.N_checkout_set);
      l_eclipsed_set = new WDPropertyDef(Consts.N_eclipsed_set);
      l_version_controlled_binding_set = new WDPropertyDef(Consts.N_version_controlled_binding_set);
      l_version_name = new WDPropertyDef(Consts.N_version_name);
      l_version_history = new WDPropertyDef(Consts.N_version_history); //, new
																																			 // PropertyName(Consts.N_version_history),
																																			 // true);
      l_version_history_collection_set = new WDPropertyDef(Consts.N_version_history_collection_set);
      l_workspace = new WDPropertyDef(Consts.N_workspace);
      l_workspace_checkout_set = new WDPropertyDef(Consts.N_workspace_checkout_set);
      l_workspace_collection_set = new WDPropertyDef(Consts.N_workspace_collection_set);
      l_version_set = new WDPropertyDef(Consts.N_version_set);
      l_root_version = new WDPropertyDef(Consts.N_root_version);
      l_label_name_set = new WDPropertyDef(Consts.N_label_name_set);

      l_acl = new WDPropertyDef(Consts.N_acl);
      l_acl_restrictions = new WDPropertyDef(Consts.N_acl_restrictions);
      l_acl_semantics = new WDPropertyDef(Consts.N_acl_semantics);
      l_alternate_URI_set = new WDPropertyDef(Consts.N_alternate_URI_set);
      l_current_user_privilege_set = new WDPropertyDef(Consts.N_current_user_privilege_set);
      l_group = new WDPropertyDef(Consts.N_group);
      l_group_membership = new WDPropertyDef(Consts.N_group_membership);
      l_group_member_set = new WDPropertyDef(Consts.N_group_member_set);
      l_inherited_acl_set = new WDPropertyDef(Consts.N_inherited_acl_set);
      l_owner = new WDPropertyDef(Consts.N_owner);
      l_principal_collection_set = new WDPropertyDef(Consts.N_principal_collection_set);
      l_principal_URL =
        new WDPropertyDef(Consts.N_principal_URL, new PropertyName(Consts.NS_SAP, Consts.EN_principal_URL), true, true);
      l_principal_details = new WDPropertyDef(Consts.N_SAP_principal_details);
      l_supported_privilege_set = new WDPropertyDef(Consts.N_supported_privilege_set);

      l_parent_set = new WDPropertyDef(Consts.N_parent_set);
      l_resource_id = new WDPropertyDef(Consts.N_resource_id);

      l_sap_appproperties = new WDPropertyDef(Consts.N_SAP_appproperties, null, true, true);
      l_sap_personalized_appproperties = new WDPropertyDef(Consts.N_SAP_personalized_appproperties, null, true, true);
      l_sap_property_displayname_set = new WDPropertyDef(Consts.N_SAP_property_displayname_set, null, true, true);
      l_sap_repository = new WDPropertyDef(Consts.N_SAP_repository, null, true, true);
    }
    catch (ResourceException ex) {
      log.fatalT("initializing system PropertyDef" + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
      throw new RuntimeException ("initializing system PropertyDef");
    }

    CommentName = l_Comment;
    SupportedLivePropertySetName = l_SupportedLivePropertySet;
    creationdate = l_creationdate;
    displayname = l_displayname;
    getcontentlanguage = l_getcontentlanguage;
    getcontentlength = l_getcontentlength;
    getcontenttype = l_getcontenttype;
    getetag = l_getetag;
    getlastmodified = l_getlastmodified;
    group = l_group;
    ishidden = l_ishidden;
    lockdiscovery = l_lockdiscovery;
    orderingtype = l_orderingtype;
    ordering_type = l_ordering_type;
    owner = l_owner;
    redirect_lifetime = l_redirect_lifetime;
    reftarget = l_reftarget;
    resourcetype = l_resourcetype;
    supportedlock = l_supportedlock;
    auto_version = l_auto_version;
    auto_update = l_auto_update;
    checked_in = l_checked_in;
    checked_out = l_checked_out;
    checkin_fork = l_checkin_fork;
    checkout_fork = l_checkout_fork;
    creator_displayname = l_creator_displayname;
    comment = l_comment;
    supported_query_grammar_set = l_supported_query_grammar_set;
    supported_live_property_set = l_supported_live_property_set;
    supported_method_set = l_supported_method_set;
    supported_report_set = l_supported_report_set;
    predecessor_set = l_predecessor_set;
    // precursor_set = l_precursor_set;
    successor_set = l_successor_set;
    checkout_set = l_checkout_set;
    eclipsed_set = l_eclipsed_set;
    version_controlled_binding_set = l_version_controlled_binding_set;
    version_name = l_version_name;
    version_history_collection_set = l_version_history_collection_set;
    workspace = l_workspace;
    workspace_checkout_set = l_workspace_checkout_set;
    workspace_collection_set = l_workspace_collection_set;
    // deltaV VHR
    version_set = l_version_set;
    version_history = l_version_history;
    root_version = l_root_version;
    label_name_set = l_label_name_set;

    acl = l_acl;
    acl_restrictions = l_acl_restrictions;
    acl_semantics = l_acl_semantics;
    alternate_URI_set = l_alternate_URI_set;
    current_user_privilege_set = l_current_user_privilege_set;
    group_membership = l_group_membership;
    group_member_set = l_group_member_set;
    inherited_acl_set = l_inherited_acl_set;
    principal_collection_set = l_principal_collection_set;
    principal_URL = l_principal_URL;
    principal_details = l_principal_details;
    supported_privilege_set = l_supported_privilege_set;

    // BIND
    parent_set = l_parent_set;
    resource_id = l_resource_id;

    // SAP extensions
    sap_appproperties = l_sap_appproperties;
    sap_personalized_appproperties = l_sap_personalized_appproperties;
    sap_property_displayname_set = l_sap_property_displayname_set;
    sap_repository = l_sap_repository;

    // all props that appear in PROPFIND/allprop
    Set set = new HashSet();
    set.add(creationdate);
    set.add(displayname);
    set.add(getcontentlanguage);
    set.add(getcontentlength);
    set.add(getcontenttype);
    set.add(getetag);
    set.add(getlastmodified);
    set.add(lockdiscovery);
    set.add(redirect_lifetime);
    set.add(reftarget);
    set.add(resourcetype);
    set.add(supportedlock);
    allprop_internal = set;
    allprop = Collections.unmodifiableSet(allprop_internal);

    // all properties, including allprop
    set = new HashSet(set);
    set.add(acl);
    set.add(acl_restrictions);
    if (Consts.SUPPORT_ACL_PROTOCOL_OLD) set.add(acl_semantics);
    set.add(alternate_URI_set);
    set.add(auto_update);
    set.add(auto_version);
    set.add(checked_in);
    set.add(checked_out);
    set.add(checkin_fork);
    set.add(checkout_fork);
    set.add(checkout_set);
    set.add(creator_displayname);
    set.add(comment);
    set.add(current_user_privilege_set);
    set.add(eclipsed_set);
    set.add(group);
    set.add(group_membership);
    set.add(group_member_set);
    set.add(inherited_acl_set);
    set.add(ishidden);
    set.add(label_name_set);
    set.add(owner);
    set.add(ordering_type);
    set.add(parent_set);
    set.add(predecessor_set);
    set.add(principal_collection_set);
    set.add(principal_details);
    set.add(principal_URL);
    set.add(resource_id);
    set.add(root_version);
    set.add(successor_set);
    set.add(supported_live_property_set);
    set.add(supported_method_set);
    set.add(supported_query_grammar_set);
    set.add(supported_privilege_set);
    set.add(supported_report_set);
    set.add(version_controlled_binding_set);
    set.add(version_history);
    set.add(version_history_collection_set);
    set.add(version_name);
    set.add(version_set);
    set.add(workspace);
    set.add(workspace_checkout_set);
    set.add(workspace_collection_set);
    set.add(sap_appproperties);
    set.add(sap_personalized_appproperties);
    set.add(sap_property_displayname_set);
    set.add(sap_repository);
    all_internal = set;
    all = Collections.unmodifiableSet(all_internal);

    // all live properties of a plain resource
    set = new HashSet();
    set.add(comment);
    set.add(creationdate);
    set.add(creator_displayname);
    set.add(displayname);
    set.add(getcontentlanguage);
    set.add(getcontentlength);
    set.add(getcontenttype);
    set.add(getetag);
    set.add(getlastmodified);
    set.add(lockdiscovery);
    set.add(resourcetype);
    set.add(supportedlock);
    set.add(supported_live_property_set);
    set.add(supported_method_set);
    set.add(supported_privilege_set);
    set.add(supported_report_set);
    set.add(sap_appproperties);
    set.add(sap_personalized_appproperties);
    set.add(sap_property_displayname_set);
    set.add(sap_repository);
    liveResourceProp = Collections.unmodifiableSet(set);

    // additional properties for resources on workspace aware deltav servers
    set = new HashSet();
    set.add(workspace_collection_set);
    liveWorkspaceAwareResourceProp = Collections.unmodifiableSet(set);

    // additional properties for resources in workspaces
    set = new HashSet();
    set.add(workspace);
    liveWorkspaceControlledResourceProp = Collections.unmodifiableSet(set);

    // additional properties for resources on version history aware deltav servers
    set = new HashSet();
    set.add(version_history_collection_set);
    liveVersionHistoryAwareResourceProp = Collections.unmodifiableSet(set);

    // all live properies of a acl-controlled resource
    set = new HashSet();
    set.add(acl);
    set.add(acl_restrictions);
    if (Consts.SUPPORT_ACL_PROTOCOL_OLD) set.add(acl_semantics);
    set.add(current_user_privilege_set);
    set.add(group);
    set.add(inherited_acl_set);
    set.add(owner);
    liveAclProp = Collections.unmodifiableSet(set);

    // all live properties of a principal resource
    set = new HashSet();
    set.add(alternate_URI_set);
    set.add(group_membership);
    set.add(group_member_set);
    set.add(principal_details);
    set.add(principal_URL);
    livePrincipalProp = Collections.unmodifiableSet(set);

    // all live properties of a link resource
    set = new HashSet();
    set.add(redirect_lifetime);
    set.add(reftarget);
    liveLinkProp = Collections.unmodifiableSet(set);

    // all live properties of a checked-out resource
    set = new HashSet();
    set.add(checked_out);
    set.add(checkin_fork);
    set.add(checkout_fork);
    set.add(predecessor_set);
    // set.add(precursor_set);
    liveCheckedOutProp = Collections.unmodifiableSet(set);

    // all live properties of a checked-in resource
    set = new HashSet();
    set.add(checked_in);
    liveCheckedInProp = Collections.unmodifiableSet(set);

    // all live properties of a version-controlled resource
    set = new HashSet();
    set.add(auto_version);
    set.addAll(liveCheckedInProp);
    set.addAll(liveCheckedOutProp);
    set.add(version_history); // not the case if the repository doesn't support it!
    liveVersionControlledProp = Collections.unmodifiableSet(set);

    // all additional properties of a version-controlled collection resource
    set = new HashSet();
    set.add(eclipsed_set);
    liveVersionControlledCollectionProp = Collections.unmodifiableSet(set);

    // all live properties of a version history resource
    set = new HashSet();
    set.add(root_version);
    set.add(version_set);
    liveVersionHistoryProp = Collections.unmodifiableSet(set);

    // all live properties of a version resource
    set = new HashSet();
    set.add(predecessor_set);
    set.add(successor_set);
    set.add(checkout_set);
    set.add(version_name);
    set.add(checkin_fork);
    set.add(checkout_fork);
    set.add(label_name_set);
    set.add(version_history);
    liveVersionProp = Collections.unmodifiableSet(set);

    // all live properties of a collection version
    set = new HashSet();
    set.add(version_controlled_binding_set);
    liveCollectionVersionProp = Collections.unmodifiableSet(set);

    // all additional live properties of a collection version
    set = new HashSet();
    set.add(ordering_type);
    liveOrderedCollectionVersionProp = Collections.unmodifiableSet(set);

    // all live properties of a working resource
    set = new HashSet();
    set.add(auto_update);
    liveWorkingResourceProp = Collections.unmodifiableSet(set);

    // all live properties of a workspace resource
    set = new HashSet();
    set.add(workspace_checkout_set);
    liveWorkspaceResourceProp = Collections.unmodifiableSet(set);

    // all live properties of a resource that supports the BIND properties
    set = new HashSet();
    set.add(parent_set);
    set.add(resource_id);
    liveBindProp = Collections.unmodifiableSet(set);

    // all live properties of ordered collections
    set = new HashSet();
    set.add(ordering_type);
    liveOrderedCollectionProp = Collections.unmodifiableSet(set);

    // all live properties
    set = new HashSet();
    set.addAll(liveAclProp);
    set.addAll(liveBindProp);
    set.addAll(liveCheckedInProp);
    set.addAll(liveCheckedOutProp);
    set.addAll(liveLinkProp);
    set.addAll(liveOrderedCollectionProp);
    set.addAll(livePrincipalProp);
    set.addAll(liveResourceProp);
    set.addAll(liveVersionControlledProp);
    set.addAll(liveVersionProp);
    set.addAll(liveCollectionVersionProp);
    set.addAll(liveWorkspaceResourceProp);
    set.addAll(liveWorkingResourceProp);
    liveProp = Collections.unmodifiableSet(set);

    lookupMap = new HashMap(all.size());
    inverseNameMap = new HashMap(all.size());
    Iterator iter = all.iterator();
    while (iter.hasNext()) {
      WDPropertyDef pd = (WDPropertyDef)iter.next();
      lookupMap.put(pd.getName(), pd);
      if (pd.getInternalName() != null) {
        inverseNameMap.put(pd.getInternalName(), pd);
      }
    }

    set = new HashSet();
    set.add(PropertyName.createContentType()); // must be hidden because it shouldn't be reported but doesn't directly map to DAV:getcontenttype
    hideInternal = Collections.unmodifiableSet(set);
  }

  public static WDPropertyDef lookup (com.sapportals.wcm.util.name.IName name) {
    //WDPropertyDef def = (WDPropertyDef) lookupMap.get(name);
    //if (def != null) m_logger.error (name + " -> " + def.isProtected());
    return (WDPropertyDef)lookupMap.get(name);
  }

  public static WDPropertyDef mapInverse(com.sapportals.wcm.util.name.IName name) {
    return (WDPropertyDef)inverseNameMap.get(name);
  }

  public static boolean hideInternal(com.sapportals.wcm.util.name.IName name) {
    return hideInternal.contains(name);
  }

  private final IPropertyName name;
  private final IPropertyName internalName;
  private final PropertyType type;
  private final boolean isProtected;
  private final boolean isHidden;

  public WDPropertyDef(Name name, IPropertyName internalName, PropertyType type,
    boolean isProtected, boolean isHidden)
    throws ResourceException
  {
    this.name = new PropertyName (name);
    this.internalName = internalName;
    this.type = type;
    this.isProtected = isProtected;
    this.isHidden = isHidden;
  }

  public WDPropertyDef(Name name, IPropertyName internal, boolean isProtected,
    boolean isHidden) throws ResourceException {

    this(name, internal, null, isProtected, isHidden);
  }

  public WDPropertyDef(Name name)
    throws ResourceException
  {
    this(name, null, false, false);
  }

  public final IPropertyName getName() {
    return this.name;
  }

  public final IPropertyName getInternalName() {
    return this.internalName;
  }

  public final PropertyType getType() {
    return this.type;
  }

  public final boolean isHidden() {
    return (this.internalName == null || this.isHidden);
  }

  public final boolean isProtected() {
    return (this.internalName == null || this.isProtected);
  }

  public final boolean visibleOnAllProp()
  {
    return allprop_internal.contains(this);
  }

  public final boolean equals(Object other) {
    if (other != null) {
      return this.name.equals(((WDPropertyDef)other).name);
    }
    return false;
  }

  public final int hashCode() {
    return this.name.hashCode();
  }

}
