package com.sapportals.wcm.protocol.webdav.repository.landscape;

import java.util.HashMap;
import java.util.Properties;

import com.sap.security.toolkit.login.Credentials;
import com.sap.tc.logging.Location;
import com.sapportals.portal.security.usermanagement.AbstractBackendConnection;

import com.sapportals.wcm.service.landscape.ISystem;

/**
 * Helper for getting user mappings.
 * <p>
 * This class is not thread-safe.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Revision: 1.2 $
 */
class BackendConnection extends AbstractBackendConnection {

  private static final Location log = Location.getLocation(BackendConnection.class);

  private static final String USER_KEY     = "user";
  private static final String PASSWORD_KEY = "mappedpassword";

  private final ISystem m_system;
  private final Credentials m_credentials;
  private String m_remoteUser;
  private String m_remotePassword;

  public BackendConnection(ISystem system, Properties credentials) {
    m_system = system;
    m_credentials = new Credentials("", credentials, "");
    if (log.beDebug()) {
      log.debugT("backend connection for system "+m_system.getId());
    }
  }

  public void setLogonData(HashMap data) {
    if (log.beDebug()) {
      log.debugT("got logon data: "+data+" for system "+m_system.getId());
    }

    m_remoteUser = (String)data.get(USER_KEY);
    m_remotePassword = (String)data.get(PASSWORD_KEY);
  }

  public String getRemoteApplication() {
    return m_system.getId();
  }

  public void connect() {
    // dummy
  }

  public void logoff() {
    // dummy
  }

  public String getUser() {
    return m_remoteUser;
  }

  public String getPassword() {
    return m_remotePassword;
  }
  
  public void reset() {
    m_remoteUser = null;
    m_remotePassword = null;
  }
}

