package com.sapportals.wcm.protocol.webdav.repository.landscape;

import java.util.List;

import com.sapportals.portal.security.usermanagement.IUser;

import com.sapportals.wcm.util.http.UserInfo;
import com.sapportals.wcm.util.uri.IHierarchicalUri;

/**
 * Informations about landscape systems (servers).
 * <p>
 * Copyright (c) SAP AG 2002
 * @author stefan.eissing@greenbytes.de
 * @version $Id: IServerInfo.java,v 1.1.1.1 2004/02/18 13:00:23 sync Exp $
 */
public interface IServerInfo {

  /**
   * Get the URI of the server
   * @return the server's uri
   */
  IHierarchicalUri getUri();

  /**
   * Get the default userinfo (name/password) to
   * use for authentication at the server. If no
   * default info is available, UserInfo.ANONYMOUS
   * is returned.
   * @return default userinfo for server
   */
  UserInfo getDefaultUserMapping();

  /**
   * Determine the userinfo (name/password) which
   * should be used for authentication. UserInfo.ANONYMOUS
   * is returned when no info is available.
   * @param user to get info for
   * @return userinfo for authentication
   */
  UserInfo getUserMappingFor(IUser user);

  int getMaxHttpConnections();

  /**
   * If this server is in the same user domain.
   * @return if server is in same user domain
   */
  boolean isSameUserDomain();
  
  /**
   * Get list of alias {@link com.sapportals.wcm.util.uri.IHierarchicalUri}s
   * for this system.
   * @return list of alias {@link com.sapportals.wcm.util.uri.IHierarchicalUri}s
   * for this system.
   */
  List getAliasUris();
}
