package com.sapportals.wcm.protocol.webdav.repository.landscape;

import com.sap.tc.logging.Location;

import com.sapportals.wcm.WcmException;
import com.sapportals.wcm.repository.ResourceFactory;
import com.sapportals.wcm.service.IServiceTypesConst;
import com.sapportals.wcm.service.landscape.ILandscapeService;
import com.sapportals.wcm.service.landscape.ISystem;
import com.sapportals.wcm.service.landscape.ISystemFactory;

/**
 * Hands out IServerInfo.
 * <p>
 * Copyright (c) SAP AG 2002
 * @author stefan.eissing@greenbytes.de
 * @version $Id: ServerInfoBroker.java,v 1.1.1.1 2004/02/18 13:00:23 sync Exp $
 */
public class ServerInfoBroker {

  private static final Location log = Location.getLocation(ServerInfoBroker.class);

  /**
   * Determine the IServerInfo for the server with the given id.
   * @param system_id of server as used in landscape service
   * @return info about server or <code>null</code> if server unknown
   */
  public static synchronized IServerInfo getInfoAbout(String system_id) {
    // sei: implementation note
    // we do not cache this information for now (although it's easy to do).
    // When the framework can restart repositories and change configuration
    // settings on the fly, caching might be counterproductive.
    //
    try {
      ISystemFactory systemFactory = getSystemFactory();
      if (systemFactory != null) {
        ISystem system = systemFactory.getSystem(system_id);
        if (system != null) {
          return new ServerInfo(system, systemFactory);
        }
      }
    }
    catch (Exception ex) {
      log.errorT("retrieving system: "+system_id + " - " + com.sapportals.wcm.util.logging.LoggingFormatter.extractCallstack(ex));
    }

    return null;
  }

  //--------------- private -----------------------------------------------------

  private static ISystemFactory getSystemFactory() throws WcmException {
    ILandscapeService lservice = getService();
    if (lservice != null) {
      return lservice.getSystemFactory();
    }
    return null;
  }

  private static ILandscapeService getService() throws WcmException {
    return (ILandscapeService)ResourceFactory.getInstance().getServiceFactory().getService(IServiceTypesConst.LANDSCAPE_SERVICE);
  }
}
