package com.sapportals.wcm.protocol.webdav.repository.monitor;

import com.sapportals.wcm.util.http.IRequester;

/**
 * Guards time usage.
 * <p>
 * Copyright (c) SAP AG 2002-2004
 * @author stefan.eissing@greenbytes.de
 * @version $Id: ITimeGuard.java,v 1.1.1.1 2004/02/18 13:00:23 sync Exp $
 */
public interface ITimeGuard {

  /**
   * Check if time limit is exceeded.
   * @throws TimeExceededException if time limit is exceeded
   */
  void check() throws TimeExceededException;
  
  /**
   * Check if time limit was/will be exceeded at the given time.
   * Time is measured in milliseconds since the Epoch, as returned
   * by {@link System#currentTimeMillis()}.
   * @param time to check against
   * @throws TimeExceededException if time limit is exceeded
   */
  void check(long time) throws TimeExceededException;
  
  /**
   * Deactivate this time guard in the context it was created.
   */
  void deactivate();
  
  /**
   * Return the amount of time in milliseconds remaining until check would
   * throw a TimeExceededException. The value maybe negative if the check
   * would have failed before.
   * @return time remaining in milliseconds until check would fail
   */
  long getRemainingMS();

  /**
   * Adjust the SO timeout value for the requester, so that it does not
   * exceed the timeout value for this ITimeGuard object.
   * @param requester to adjust
   */
  void adjustSOTimeout(IRequester requester);
}
