package com.sapportals.wcm.protocol.webdav.server.loadtests;

import java.io.*;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright (c) SAP AG 2001-2005
 * @author stefan.eissing@greenbytes.de
 */

public class BigCollectionTest extends TestMember
{
  private static final String[] desc=new String[] {
    "00 BC MKCOL big collection",
    "01 BC PUT into big collection",
    "02 BC PROPFIND big collection",
    "03 BC COPY big collection",
    "04 BC DELETE copy of big collection",
    "BC cleanup",
  };

  public static int NrOfTests = desc.length;

  private static final String COL  = "/bigcollection";
  private static final String COL2 = "/big2";

  private static int m_numfiles = 3000;

  public BigCollectionTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest() {
    try
    {
      IResponse response;
      clear(request);
      switch (nr)
      {
      case 0:
        request.setMethod("MKCOL");
        request.setURI(localpart + COL);
        response = requester.perform(request);
        checkResponseCode(response, 201);
        break;
      case 1:
        putFiles(localpart + COL);
        break;
      case 2:
        request.setMethod("PROPFIND");
        request.setURI(localpart + COL);
        request.setHeader("Depth", "1");
        request.setBody("<propfind xmlns='DAV:'><allprop/></propfind>");
        response = requester.perform(request);
        checkResponseCode(response, 207);
        break;
      case 3:
        request.setMethod("COPY");
        request.setURI(localpart + COL);
        request.setHeader("Destination", m_base + localpart + COL2);
        response = requester.perform(request);
        checkResponseCode(response, 201);
        break;
      case 4:
        request.setMethod("DELETE");
        request.setURI(localpart + COL2);
        response = requester.perform(request);
        checkResponseCode(response, 204);
        break;
      default:
        request.setMethod("DELETE");
        request.setURI(localpart + COL);
        response = requester.perform(request);
        checkResponseCode(response, 204);
        break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

  private void putFiles(String coluri) throws Exception {
    InputStream is = this.getClass().getResourceAsStream("bigc.input.copyme");
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    copy(is, bos);
    byte[] content = bos.toByteArray();

    IResponse response;
    for (int i = 0; i < m_numfiles; ++i) {
      clear(request);
      request.setMethod("PUT");
      request.setURI(coluri + "/rfc"+i+".txt");
      request.setContentType("text/plain");
      request.setContentLength(content.length);
      ByteArrayInputStream bis = new ByteArrayInputStream(content);
      request.setBody(bis);

      response = requester.perform(request);
      checkResponseCode(response, 201);
    }
  }


  private void copy(InputStream is, OutputStream os) throws IOException {
    byte[] buffer = new byte[64*1024];
    int len;
    while ((len = is.read(buffer)) != -1) {
      os.write(buffer, 0, len);
    }
  }

}
