package com.sapportals.wcm.protocol.webdav.server.loadtests;

import java.io.FileOutputStream;
import java.io.PrintStream;

import com.sapportals.wcm.util.http.IContext;
import com.sapportals.wcm.util.http.IRequesterFactory;
import com.sapportals.wcm.util.http.RequesterFactory;
import com.sapportals.wcm.util.http.UserInfo;
import com.sapportals.wcm.util.uri.IHierarchicalUri;
import com.sapportals.wcm.util.uri.UriFactory;

public class Main {
  
  public static void main(String[] args) throws Exception {
    if (args.length < 2) {
      usage("wrong number of arguments");
    }
    if ("crawl".equals(args[0])) {
      String tmp = args[args.length - 1];
      IHierarchicalUri start = UriFactory.parseHierarchicalUri(tmp);
      IHierarchicalUri base = start;
      int depth = 100;
      int workers = 4;
      String username = "anonymous";
      String password = null;
      String logfile = null;
      for (int i = 1, n = args.length - 1; i < n; ++i) {
        tmp = args[i];
        if (tmp.startsWith("--base=")) {
          tmp = tmp.substring("--base=".length());
          base = UriFactory.parseHierarchicalUri(tmp);
        }
        else if (tmp.startsWith("--depth=")) {
          tmp = tmp.substring("--depth=".length());
          depth = Integer.parseInt(tmp);
        }
        else if (tmp.startsWith("--workers=")) {
          tmp = tmp.substring("--workers=".length());
          workers = Integer.parseInt(tmp);
        }
        else if (tmp.startsWith("--http-user=")) {
          username = tmp.substring("--http-user=".length());
        }
        else if (tmp.startsWith("--http-pass=")) {
          password = tmp.substring("--http-pass=".length());
        }
        else if (tmp.startsWith("--logfile=")) {
          logfile = tmp.substring("--logfile=".length());
        }
        else {
          usage("unknown option: "+tmp);
        }
      }
      
      crawl(start, base, username, password, depth, workers, logfile, true);
    }
    else {
      usage("unknown command: "+args[0]);
    }
    System.exit(0);
  }

  //-------------- private -----------------------------------------------------
  
  private static void usage(String message) {
    if (message != null) {
      System.err.println(message);
    }
    System.err.println("usage: cmd crawl [ options ] url");
    System.err.println("       to crawl url via WebDAV PROPFIND, where options are");
    System.err.println(" --base=url");
    System.err.println("       defaults to main url, no resource outside base is crawled");
    System.err.println(" --depth=n");
    System.err.println("       defaults to 100, maximum link depth to follow");
    System.err.println(" --workers=n");
    System.err.println("       defaults to 4, number of threads to use");
    System.err.println(" --http-user=username");
    System.err.println("       username to use for authentication");
    System.err.println(" --http-pass=password");
    System.err.println("       password to use for authentication");
    System.err.println(" --logfile=filename");
    System.err.println("       file to log each crawled url to");
    System.exit(2);
  }
  
  private static void crawl(IHierarchicalUri start,
                            IHierarchicalUri base,
                            String username,
                            String password,
                            int maxDepth,
                            int workers,
                            String logfile,
                            boolean verbose) throws Exception {
    IRequesterFactory factory = RequesterFactory.newInstance();
    factory.setBaseURL(base.toString());
    IContext context = factory.getContext("user");
    if (context.isNew()) {
      context.setUserInfo(UserInfo.valueOf(username, password));
    }
    PrintStream log = null;
    if (logfile != null) {
      FileOutputStream flog = new FileOutputStream(logfile, false);
      log = new PrintStream(flog, true);
    }
    SummaryObserver observer = SummaryObserver.getInstance(start, maxDepth, workers, verbose);
    WebDAVCrawler crawler = WebDAVCrawler.getInstance(factory, context, base);
          
    observer.outputIntermediateTo(System.out, 1000);
    observer.setLog(log);
    observer.start();
    crawler.walk(start, maxDepth, workers, observer);
    observer.finish();
          
    observer.printSummary(System.out);
    if (log != null) {
      log.close();
    }
  }
}
