package com.sapportals.wcm.protocol.webdav.server.loadtests;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.sapportals.wcm.util.http.*;

/**
 * Generates various tests for a certain class.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2005
 * @author stefan.eissing@greenbytes.de
 */

public class MyTestSuite extends TestCase
{
  private static boolean m_initialized = false;
  private static String m_host = null, m_host2 = null;

  // base collection for tests
  private static String m_base = null, m_base2 = null;

  private static IRequesterFactory m_factory = null, m_factory2 = null;
  private static IRequester m_requester = null, m_requester2 = null;
  private static IContext m_context = null, m_context2 = null;

  private static boolean m_first = true;

  private static String m_user;
  private static String m_password;

  public MyTestSuite(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    SetUp();
  }

  public static void SetUp() throws Exception
  {
    if (m_first) {
      m_first = false;
      init();
    }
  }

  private synchronized static void init() {
    if (!m_initialized) {
      m_initialized = true;

      m_host = System.getProperty ("host", "http://localhost:8080");
      m_base = System.getProperty ("base", "/wcm/docs");
      m_host2 = System.getProperty ("host2", "http://localhost:8081");
      m_base2 = System.getProperty ("base2", "/wcm/docs");

      m_user = System.getProperty("user", "user");
      m_password = System.getProperty("password", "user");

      try{
        m_factory = RequesterFactory.newInstance();
        m_factory.setSoTimeout(180000);
        m_factory.setBaseURL(m_host);
        m_context = m_factory.getContext("user");

        if (m_context.isNew()) {
          m_context.setUserInfo(UserInfo.valueOf(m_user, m_password));
        }
        m_requester = m_factory.getRequester(m_context);

        m_factory2 = RequesterFactory.newInstance();
        m_factory2.setBaseURL(m_host2);
        m_context2 = m_factory2.getContext("user");

        if (m_context2.isNew()) {
          m_context2.setUserInfo(UserInfo.valueOf(m_user, m_password));
        }
        m_requester2 = m_factory2.getRequester(m_context2);

      }catch (Throwable ex) {
        ex.printStackTrace(System.err);
      }
    }
  }

  public static TestSuite suite(Class memberClass, String name)
  {
    if (!TestMember.class.isAssignableFrom(memberClass))
      throw new IllegalArgumentException("wrong class: "+memberClass);

    init();
    TestSuite suite = new TestSuite(name);
    try{
      Class[] consArgs = new Class[] {Integer.class};
      Constructor cons = memberClass.getConstructor(consArgs);
      Field field = memberClass.getField("NrOfTests");
      int maxTests = field.getInt(null);
      for (int i=0; i<maxTests; i++)
      {
        Object[] args = new Object[] {new Integer(i)};
        TestMember instance = (TestMember)cons.newInstance(args);
        instance.init(m_factory, m_context, m_requester, m_host, m_base,
                      m_factory2, m_context2, m_requester2, m_host2, m_base2);
        suite.addTest(instance);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
    }
    return suite;
  }

  public static IRequester getRequester() {
    init();
    return m_requester;
  }

  public static IRequester getRequester2() {
    init();
    return m_requester2;
  }

  public static String getHost() {
    init();
    return m_host;
  }

  public static String getBaseURI() {
    init();
    return m_base;
  }

  public static String getHost2() {
    init();
    return m_host2;
  }

  public static String getBase2URI() {
    init();
    return m_base2;
  }

  protected void runTest() {
  }
}
