package com.sapportals.wcm.protocol.webdav.server.loadtests;

import java.io.IOException;
import java.io.InputStream;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Navigation UI benchmark.
 *
 * <p>
 * Copyright (c) SAP AG 2003-2005
 * @author julian.reschke@greenbytes.de
 */

public class NavigationTest extends TestMember
{
  private static final String[] desc = new String[]
  {
    "00 NAV setup",
    "01 NAV GET benchmark",
    "   NAV cleanup",
  };

  public static int NrOfTests = desc.length;

  public NavigationTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  private static final String UA = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)";
  
  private static String navigationCollection = System.getProperty("navigation", null);
  private static boolean onDemand = "true".equalsIgnoreCase(System.getProperty("onDemand"));
  private static int children = 32;
  
  protected void runTest()
  {
    try
    {
      IResponse response;
      
      requester.setSoTimeout(600000);

      switch (nr)
      {
        case 0: {
            long start = System.currentTimeMillis();
            String tmp = System.getProperty("children", "32");
            children = Integer.parseInt(tmp);
            
            if ( ! onDemand) {
              clear(request);
              request.setMethod("DELETE");
              request.setURI(localpart + "/navbench");
              response = requester.perform(request);
              checkResponseCode(response, 204, 404);
            }

            clear(request);
            request.setMethod("MKCOL");
            request.setURI(localpart + "/navbench");
            response = requester.perform(request);
            checkResponseCode(response, 201, 503, 405);
            assertTrue("could not create collection navbench", 
                onDemand || response.getStatus() == 201);
            
            if (response.getStatus() == 201) {
              for (int i = 0; i < children; i++) {
                clear(request);
                request.setMethod("PUT");
                request.setURI(localpart + "/navbench/" + i);
                response = requester.perform(request);
                checkResponseCode(response, 201);
              }

              System.out.println("setup took " + (System.currentTimeMillis() - start) + " ms");
            }
            else {
              System.out.println("using existing collection");
            }
          }
          break;

        case 1: {
            long start = System.currentTimeMillis();
            long navDuration = 0L;
            long succeeded = 0, total = 0;
            boolean imagesCached = false;
            String lastmod = null;
            String tmp = System.getProperty("iterations", "50");
            int iterations = Integer.parseInt(tmp);

            for (int i = 0; i < iterations; i++) {
              System.out.println("Starting iteration " + i + "...");
              System.out.flush();
              clear(request);
              request.setMethod("GET");
              request.setURI(navigationCollection + "/navbench");
              request.setHeader("user-agent", UA);
              request.expectsResponseStream(true);
              long navStart = System.currentTimeMillis();
              response = requester.perform(request);
              getContent(response.getStream());
              navDuration += (System.currentTimeMillis() - navStart);
              total += 1;
              if (response.getStatus() < 300) {
                succeeded += 1;
              }
              else {
                System.out.println("GET on " 
                    + navigationCollection + "/navbench"
                    + "returned: " + response.getStatus()); 
              }

              System.out.println("done.\n");
              System.out.flush();
              if (! imagesCached) {
                System.out.println("Starting iteration " + i + "...");
                System.out.flush();
                clear(request);
                request.setMethod("GET");
                request.setURI(localpart + "/../../etc/public/mimes/images/plaintext.gif");
                request.setHeader("user-agent", UA);
                if (lastmod != null) request.setHeader("if-modified-since", lastmod);
                request.expectsResponseStream(true);
                response = requester.perform(request);
                getContent(response.getStream());
                if (response.getStatus() < 300) {
                  if (response.getHeader("last-modified") != null) {
                    lastmod = response.getHeader("last-modified");
                  }
                  if (response.getHeader("expires") != null) {
                    imagesCached = true;
                    System.out.println("images cached");
                  }
                }
                clear(request);
                request.setMethod("GET");
                request.setURI(localpart + "/../../etc/public/mimes/images/standard_folder.gif");
                request.setHeader("user-agent", UA);
                if (lastmod != null) request.setHeader("if-modified-since", lastmod);
                request.expectsResponseStream(true);
                response = requester.perform(request);
                getContent(response.getStream());
                clear(request);
                request.setMethod("GET");
                request.setURI(localpart + "/../../etc/public/mimes/images/pop_up.gif");
                request.setHeader("user-agent", "foobar");
                if (lastmod != null) request.setHeader("if-modified-since", lastmod);
                request.expectsResponseStream(true);
                response = requester.perform(request);
                getContent(response.getStream());
              }
            }

            System.out.println("GETs (" + succeeded + "/" + total + ", "
                + children + " children) took " + (System.currentTimeMillis() - start) + " ms");
            double navAverage = ((double)navDuration) / total;
            System.out.println("GET on navigation control average: "+navAverage+" ms");
          }
          break;

        default:
          if ( ! onDemand) {
            clear(request);
            request.setMethod("DELETE");
            request.setURI(localpart + "/navbench");
            request.setHeader("x-greenbytes-memory-footprint", "true"); 
            response = requester.perform(request);
            checkResponseCode(response, 204);
          }
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

  private String getContent(InputStream is) throws IOException {
    if (is == null)
      return null;

    byte[] buffer = new byte[64 * 1024];
    // sei: do we really need this? We do not want to test browser performance...
    //ByteArrayOutputStream bos = new ByteArrayOutputStream();
    //int len;
    while ((/*len =*/ is.read(buffer)) != -1) {
      //bos.write(buffer, 0, len);
    }
    is.close();

    return "";//bos.toString();
  }

}
