package com.sapportals.wcm.protocol.webdav.server.loadtests;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.sapportals.wcm.IWcmConst;
import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DomSerializer;
import com.sapportals.wcm.protocol.webdav.Name;
import com.sapportals.wcm.util.http.*;

/**
 * Abstract class. All implementing classes have access to a check of a
 * normalized multistatus responsecode. To compare the results the path to
 * the response-files is needed. It is stored in the system property
 * 'statusPath' and has the default
 * "c:/home/projects/wcm/testcases/JUnitTestResults/".
 *
 * <p>
 * Copyright (c) SAP AG 2001-2005
 * @author martin.boettcher@greenbytes.de
 */
public abstract class TestMember extends TestCase
{
  public static final String FAILURE_EXT = ".failure";
  public static final String PROP_STATUS_PATH = "statusPath";

  // Limited HTTP Extension Framework support (RFC2774)
  
  public static final String HEADER_EXT = "Ext";
  public static final String HEADER_MAN = "Man";
  public static final String HEADER_OPT = "Opt";

  public static final String NS_HTTPEXT_CACHESERVER = Consts.NS_WDSERVLET + IWcmConst.NAMESPACE_SEPARATOR + "cache-server";
  public static final String NS_HTTPEXT_DEBUG = Consts.NS_WDSERVLET + IWcmConst.NAMESPACE_SEPARATOR + "debug";
  
  public static final Name HTTPEXT_N_NO_CACHE_REDIRECT = new Name(NS_HTTPEXT_CACHESERVER, "no-cache-redirect");
  public static final Name HTTPEXT_N_PRETENDED_ADDRESS = new Name(NS_HTTPEXT_CACHESERVER, "pretended-address");
  public static final Name HTTPEXT_N_SPECIFIC_EXPIRY = new Name(NS_HTTPEXT_CACHESERVER, "specific-expiry");
  public static final Name HTTPEXT_N_TRACE = new Name(NS_HTTPEXT_DEBUG, "trace");
  
  
  // Extensions

  public static final String SERVLET_TRACE_HEADER = "x-greenbytes";
  public static final String SERVLET_TRACE_HEADER2 = "x-litmus";
  public static final String SERVLET_TRACE_HEADER3 = "x-litmus-second";  
  
  public static final String RESOURCE_CONTEXT_PRINCIPAL_FAKE = "x-greenbytes-principal-fake";

  private String testName;

  protected int nr=0;
  protected IRequesterFactory m_factory = null, m_factory2 = null;
  protected IContext m_context = null, m_context2 = null;
  protected IRequester requester = null, m_requester2 = null;
  protected HttpRequest request = null, m_request2 = null;
  protected String m_base = null, m_base2 = null;
  protected String localpart = null, m_localpart2 = null;
  protected String m_statusPath = null;
  protected boolean m_allowRedirects;

  /**
   * @param nr the current number of the desired test in the subclass.
   * @param s the name of the testclass.
   */
  public TestMember(Integer nr, String s) {
    super(s);
    this.testName = s;
    this.nr = nr.intValue();
  }

  public void clear(HttpRequest request) {
    request.clear();
    request.setHeader(Consts.SERVLET_TRACE_HEADER, testName + " [" + this.m_base + this.localpart + "]");
    request.setHeader(HEADER_OPT, '"' + HTTPEXT_N_TRACE.getNamespaceUri() + '"' + " ;ns=00");
    request.setHeader("00-" + HTTPEXT_N_TRACE.getLocalName(), testName + " [" + this.m_base + this.localpart + "]");
    request.setHeader(HEADER_MAN, '"' + HTTPEXT_N_NO_CACHE_REDIRECT.getNamespaceUri() + '"' + " ;ns=01");
    request.setHeader("01-" + HTTPEXT_N_NO_CACHE_REDIRECT.getLocalName(), "");
  }

  public void init(IRequesterFactory factory, IContext context, IRequester requester, String base, String localpart,
                   IRequesterFactory factory2, IContext context2, IRequester requester2, String base2, String localpart2) {
    m_factory = factory;
    m_factory2 = factory2;
    m_context = context;
    m_context2 = context2;
    m_base = base;
    this.localpart = localpart;
    m_localpart2 = localpart2;
    this.requester = requester;
    m_requester2 = requester2;
    this.request = new HttpRequest(localpart);
    m_request2 = new HttpRequest(m_localpart2);
    clear(request);
    clear(m_request2);
    request.setURI(localpart);
    m_request2.setURI(m_localpart2);

    m_statusPath = System.getProperty(PROP_STATUS_PATH,
      "c:/projects/WCM3/testcases/JUnitTestResults/");

    m_allowRedirects = "true".equalsIgnoreCase(System.getProperty("REDIRECTS", "false"));
    
    if (!m_statusPath.endsWith("/"))
      m_statusPath += "/";

  }

//  private void writeToFile(String contents, String filename)
//  {
//    System.err.println("Creating response file: " + filename);
//    try{
//      java.io.FileOutputStream f = new java.io.FileOutputStream(filename);
//      f.write( contents.getBytes() );
//      f.close();
//    }catch(Exception e){
//      System.err.println("could not write response to file: " + e.toString() );
//    }
//  }
//
//  private String readFromFile (java.io.InputStream f)
//  {
//    if (f == null) return null;
//
//    try{
//      ByteArrayOutputStream bos = new ByteArrayOutputStream();
//      byte[] buffer = new byte[64 * 1024];
//      int len = 0;
//      while ((len = f.read(buffer)) != -1) {
//        bos.write(buffer, 0, len);
//      }
//      f.close();
//      return bos.toString();
//    }
//    catch(Exception e){
//      System.err.println("could not read response from file: " + e.toString() );
//      return null;
//    }
//  }

  protected void checkResponseCode(IResponse response, int expected) {
    checkResponseCode(response, expected, -1, -1);
  }

  protected void checkResponseCode(IResponse response, int expected1, int expected2) {
    checkResponseCode(response, expected1, expected2, -1);
  }

  protected void checkResponseCode(IResponse response, int expected1,
                                   int expected2, int expected3) {
    int code = response.getStatus();
    if (code != expected1 && code != expected2 && code != expected3) {
      StringBuffer sb = new StringBuffer();
      sb.append("expected response codes (").append(expected1);
      if (expected2 >= 0) {
        sb.append(", ").append(expected2);
        if (expected3 >= 0) {
          sb.append(", ").append(expected3);
        }
      }
      sb.append(") got ").append(code);
      Document doc = response.getDocument();
      if (doc != null) {
        sb.append(" with body\n");
        sb.append(DomSerializer.toString(doc, true));
      }
      assertTrue (sb.toString(), false);
    }
  }

  /**
   * to be implemented by the subclasses.
   */
  protected abstract void runTest() throws Exception;

  protected void logComment(String comment) {
    System.out.print("{" + comment + "}");
  }
}
