package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.io.PrintWriter;
import java.io.StringWriter;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright SAP AG 2002-2003
 * @author martin.boettcher@greenbytes.de
 */

public class GetTest extends TestMember
{
  private static final String[] fns  = new String[] {
    "/",
    "",
    "/voll",
    "/abc.xyz",
    "/Worddatei.doc",
    "/ANSI.TXT",
    "/Euro%e2%82%ac.txt",
    "/Mappe1.xls",
    "/my_dog_&_cat.txt",
    "/Test!.ppt",
    "/Umlaute%20%c3%84%c3%96%c3%9c%c3%a4%c3%b6%c3%bc%c3%9f.txt",
    "/Unicode.TXT",
    "/UTF8.TXT",
    "/ANSI.TXT"
  };

  private static final String[] desc = new String[] {
    "00 GET with root",
    "01 GET WITH 'empty' URI",
    "02 GET with collection",
    "03 GET with unknown file",
    "04 GET for " + fns[ 4],
    "05 GET for " + fns[ 5],
    "06 GET for " + fns[ 6],
    "07 GET for " + fns[ 7],
    "08 GET for " + fns[ 8],
    "09 GET for " + fns[ 9],
    "10 GET for " + fns[10],
    "11 GET for " + fns[11],
    "12 GET for " + fns[12],
    "13 GET RANGE for " + fns[13]
  };

  public static int NrOfTests = desc.length;

  public GetTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    try {
      IResponse response;
      clear(m_request);
      m_request.setMethod("GET");
      m_request.setURI(m_localpart + fns[nr]);
      switch (nr) {
        case 0 :
          response = m_requester.perform(m_request);
          checkResponseCode(response, 200);
          break;
        case 1 :
        case 2 :
          response = m_requester.perform(m_request);
          checkResponseCode(response, 200);
          break;
        case 3 :
          response = m_requester.perform(m_request);
          checkResponseCode(response, 404);
          break;
        case 4 :
        case 5 :
        case 6 :
        case 7 :
        case 8 :
        case 9 :
        case 10 :
        case 11 :
        case 12 :
          response = m_requester.perform(m_request);
          checkResponseCode(response, 200);
          break;
        case 13 :
          m_request.setHeader("Range", "bytes=0-19");
          m_request.expectsResponseStream(true);
          response = m_requester.perform(m_request);
          checkResponseCode(response, 206);
          break;
      }
    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
//    System.err.println( this.getName() + " took " + (System.currentTimeMillis()-start) + "msec.");
  }
}
