package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.io.PrintWriter;
import java.io.StringWriter;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright SAP AG 2002-2003
 * @author martin.boettcher@greenbytes.de
 */

public class MkColTest extends TestMember
{
  private static final String[] fns  = new String[] {
    "/removeme",
    "/removeme.flatdir",
    "/removeme.deepdir"
  };

  private static final String[] desc = new String[] {
    "01 MKCOL make 100 files",
    "02 MKCOL make directory with 100 files",
    "03 MKCOL make 100 empty directories"
  };

  public static int NrOfTests = desc.length;

  public MkColTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    try
    {
      IResponse response;
      clear(m_request);
      switch (nr)
      {
      case 0:
        m_request.setMethod("PUT");
        m_request.setBody("Test");
        for (int i=0; i<100; i++) {
          m_request.setURI(m_localpart + fns[0] + i);
          response = m_requester.perform( m_request );
          checkResponseCode(response, 200, 201);
        }
        break;
      case 1:
        m_request.setMethod("MKCOL");
        m_request.setURI(m_localpart + fns[1]);
        response = m_requester.perform( m_request );
        checkResponseCode(response, 200, 201);
        for (int i=0; i<100; i++){
          m_request.clear();
          m_request.setMethod("PUT");
          m_request.setBody("Test");
          m_request.setURI(m_localpart + fns[1] + "/" + fns[0] + i);
          response = m_requester.perform( m_request );
          checkResponseCode(response, 200, 201);
        }
        break;
      case 2:
        m_request.clear();
        m_request.setMethod("MKCOL");
        m_request.setURI(m_localpart + fns[2]);
        response = m_requester.perform( m_request );
        checkResponseCode(response, 200, 201);
        for (int i=0; i<10; i++) {
          m_request.clear();
          m_request.setMethod("MKCOL");
          m_request.setURI(m_localpart + fns[2] + "/" + fns[1] + i);
          response = m_requester.perform( m_request );
          checkResponseCode(response, 200, 201);
          for (int j=0; j<10; j++) {
            m_request.clear();
            m_request.setMethod("MKCOL");
            m_request.setURI(m_localpart + fns[2] + "/" + fns[1] + i + "/" + fns[1] + j);
            response = m_requester.perform( m_request );
            checkResponseCode(response, 200, 201);
          }
        }
        break;
      }
    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
//    System.err.println( this.getName() + " took " + (System.currentTimeMillis()-start) + "msec.");
  }
}
