package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.sapportals.wcm.util.http.*;

/**
 * Generates various tests for a certain class.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: MyTestSuite.java,v 1.1.1.1 2004/02/18 12:58:51 sync Exp $
 */

public class MyTestSuite extends TestCase
{
  private static boolean m_initialized = false;
  private static String m_host = null;

  // base collection for tests
  private static String m_base = null;

  private static IRequesterFactory m_factory = null;
  private static IRequester m_requester = null;
  private static IContext m_context = null;

  private static boolean m_first = true;

  private static String m_user;
  private static String m_password;

  public MyTestSuite(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    SetUp();
  }

  public static void SetUp() throws Exception
  {
    if (m_first) {
      m_first = false;
      init();
    }
  }

  private static void init() {
    if (!m_initialized) {
      synchronized(MyTestSuite.class) {
        if (!m_initialized) {
          m_initialized = true;

          m_host = System.getProperty ("host", "http://localhost:8080");
          m_base = System.getProperty ("base", "/wcm/docs/cm/test/Collection%231/Collection%231/Collection%231/Collection%231/Collection%231/Collection%231/Collection%232/Collection%231/Collection%231/Collection%231/Collection%232/Collection%231/Collection%231/Collection%231/Collection%231/Collection%231"); //wcmdb

          m_user = System.getProperty("user", "user");
          m_password = System.getProperty("password", "user");

          try{
            Class factoryClass = Class.forName("com.sapportals.wcm.util.http.slim.SlimRequesterFactory");
            m_factory = (IRequesterFactory)factoryClass.newInstance();
            m_factory.setBaseURL(m_host);

            m_context = m_factory.getContext("user");

            if (m_context.isNew()) {
              m_context.setUserInfo(UserInfo.valueOf(m_user, m_password));
            }
            m_requester = m_factory.getRequester(m_context);
          }catch (Throwable ex) {
            ex.printStackTrace(System.err);
          }
        }
      }
    }
  }

  public static TestSuite suite(Class memberClass, String name)
  {
    if (!TestMember.class.isAssignableFrom(memberClass))
      throw new IllegalArgumentException("wrong class: "+memberClass);

    init();
    TestSuite suite = new TestSuite(name);
    try{
      Class[] consArgs = new Class[] {Integer.class};
      Constructor cons = memberClass.getConstructor(consArgs);
      Field field = memberClass.getField("NrOfTests");
      int maxTests = field.getInt(null);
      for (int i=0; i<maxTests; i++)
      {
        Object[] args = new Object[] {new Integer(i)};
        TestMember instance = (TestMember)cons.newInstance(args);
        instance.init(m_requester, m_host, m_base);
        suite.addTest(instance);
      }
    }catch (Exception ex) {
      ex.printStackTrace(System.err);
    }
    return suite;
  }

  public static IRequester getRequester() {
    init();
    return m_requester;
  }

  public static String getHost() {
    init();
    return m_host;
  }

  public static String getBaseURI() {
    init();
    return m_base;
  }

  protected void runTest() {
  }
}
