package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.io.PrintWriter;
import java.io.StringWriter;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright SAP AG 2002-2003
 * @author martin.boettcher@greenbytes.de
 */

public class PropfindDeepTest extends TestMember
{
  private static final String[] fns  = new String[] {
    "/voll",
    "/",
    "/voll",
    "/"
  };

  private static final String[] desc = new String[] {
    "00 PROPFIND DEPTH 1 for collection",
    "01 PROPFIND DEPTH 1 for root",
    "02 PROPFIND DEPTH infinity for collection",
    "03 PROPFIND DEPTH infinity for root"
  };

  public static int NrOfTests = desc.length;

  public PropfindDeepTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    try
    {
      IResponse response;
      clear(m_request);
      m_request.setMethod("PROPFIND");
      if (nr<2)
        m_request.setHeader("Depth", "1");
      else
        m_request.setHeader("Depth", "infinity");

      m_request.setURI(m_localpart + fns[nr]);
      switch (nr)
      {
      case 0:
      case 1:
        response = m_requester.perform( m_request );
        checkResponseCode(response, 207);
        break;
      case 2:
      case 3:
        response = m_requester.perform( m_request );
        checkResponseCode(response, 403);
      }
    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
//    System.err.println( this.getName() + " took " + (System.currentTimeMillis()-start) + "msec.");
  }
}
