package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.io.PrintWriter;
import java.io.StringWriter;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright SAP AG 2002-2003
 * @author martin.boettcher@greenbytes.de
 */

public class PropfindFlatTest extends TestMember
{
  private static final String[] fns  = new String[] {
    "/",
    "/abc.xyz",
    "/Worddatei.doc",
    "/ANSI.TXT",
    "/Euro%e2%82%ac.txt",
    "/Mappe1.xls",
    "/my_dog_&_cat.txt",
    "/Test!.ppt",
    "/Umlaute%20%c3%84%c3%96%c3%9c%c3%a4%c3%b6%c3%bc%c3%9f.txt",
    "/Unicode.TXT",
    "/UTF8.TXT"
  };

  private static final String[] desc = new String[] {
    "00 PROPFIND root",
    "01 PROPFIND with unknown file",
    "02 PROPFIND for " + fns[ 2],
    "03 PROPFIND for " + fns[ 3],
    "04 PROPFIND for " + fns[ 4],
    "05 PROPFIND for " + fns[ 5],
    "06 PROPFIND for " + fns[ 6],
    "07 PROPFIND for " + fns[ 7],
    "08 PROPFIND for " + fns[ 8],
    "09 PROPFIND for " + fns[ 9],
    "10 PROPFIND for " + fns[10]
  };

  public static int NrOfTests = desc.length;

  public PropfindFlatTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    try
    {
      IResponse response;
      clear(m_request);
      m_request.setMethod("PROPFIND");
      m_request.setHeader("Depth", "0");
      m_request.setURI(m_localpart + fns[nr]);
      switch (nr)
      {
      case 1:
        response = m_requester.perform( m_request );
        checkResponseCode(response, 404);
        break;
      case 0:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
        response = m_requester.perform( m_request );
        checkResponseCode(response, 207);
        break;
      }
    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
//    System.err.println( this.getName() + " took " + (System.currentTimeMillis()-start) + "msec.");
  }
}
