package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.io.PrintWriter;
import java.io.StringWriter;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.Name;
import com.sapportals.wcm.protocol.webdav.dasl.BasicSearch;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright SAP AG 2002-2003
 * @author martin.boettcher@greenbytes.de
 */

public class QueryTest extends TestMember
{
  private static final String[] fns  = new String[] {
    "/",
    "/",
  };

  private static final String[] desc = new String[] {
    "00 QUERY DEPTH 1",
    "01 QUERY DEPTH infinity"
  };

  private static Name pTime = new Name("greenbytes.de", "time");
  // private static final DateFormat DF = new DateFormat();
  public static int NrOfTests = desc.length;

  public QueryTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    long start = System.currentTimeMillis();
    try
    {
      clear(m_request);

      String body = buildBasic("<resourcetype/>",
                               m_localpart + fns[nr],
                               nr==0 ? 1 : Integer.MAX_VALUE,
                               exprAnd(
                                   exprGreaterEq(pTime, "1082882678875"),
                                   exprLessEq(pTime, "1062882678875")
                               ),
                               null,
                               0);
      m_request.setMethod("SEARCH");
      m_request.setURI(m_localpart + fns[nr]);
      m_request.setBody(body);
      IResponse response = m_requester.perform(m_request);
      checkResponseCode(response, 207);
      System.err.println("Response size=" + response.getDocument().toString().length());
    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
    System.err.println( this.getName() + " took " + (System.currentTimeMillis()-start) + "msec.");
  }

  private String exprGreaterEq(Name propname, String lit) {
    return nestString(BasicSearch.N_gte, prop(propname) + literal(lit));
  }

  private String exprLessEq(Name propname, String lit) {
    return nestString(BasicSearch.N_lte, prop(propname) + literal(lit));
  }

  private String exprAnd(String cont1, String cont2) {
    return nestString(BasicSearch.N_and, cont1 + cont2);
  }

  private String nestString(Name name, String contents) {
    return name.getStartTag() + contents + name.getEndTag();
  }

  private String prop(Name propname) {
    return nestString(Consts.N_prop, propname.getEmptyTag());
  }

  private String literal(String lit) {
    return nestString(BasicSearch.N_literal, lit);
  }

  private static String buildBasic(String select, String href, int depth,
                                     String where, String orderby, int limit) {
      StringBuffer sb = new StringBuffer();
      String depthString = (depth == 1? "1" : (depth == 0? "0" : "infinity"));

      sb.append("<searchrequest xmlns='DAV:'>");
      sb.append("  <basicsearch>");
      sb.append("    <select>");
      if (select != null) {
        sb.append("      <prop>");
        sb.append("        ").append(select);
        sb.append("      </prop>");
      }
      else {
        sb.append("<allprop/>");
      }
      sb.append("    </select>");
      sb.append("    <from>");
      sb.append("      <scope>");
      sb.append("        <href>").append(href).append("</href>");
      sb.append("        <depth>").append(depthString).append("</depth>");
      //if (depth == DEEP_INCLUDING_VERSIONS) sb.append("<include-versions/>");
      sb.append("      </scope>");
      sb.append("    </from>");

      if (where != null) {
        sb.append("    <where>");
        sb.append("      ").append(where);
        sb.append("    </where>");
      }

      if (orderby != null) {
        sb.append("    <orderby>");
        sb.append("      ").append(orderby);
        sb.append("    </orderby>");
      }

      if (limit > 0) {
        sb.append("    <limit>");
        sb.append("      <nresults>").append(limit).append("</nresults>");
        sb.append("    </limit>");
      }

      sb.append("  </basicsearch>");
      sb.append("</searchrequest>");

      return sb.toString();
    }

}
