package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.sapportals.wcm.protocol.webdav.DomSerializer;
import com.sapportals.wcm.util.http.HttpRequest;
import com.sapportals.wcm.util.http.IRequester;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Abstract class. All implementing classes have access to a check of a
 * normalized multistatus responsecode. To compare the results the path to
 * the response-files is needed. It is stored in the system property
 * 'statusPath' and has the default
 * "c:/home/projects/wcm/testcases/JUnitTestResults/".
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author martin.boettcher@greenbytes.de
 * @version $Id: TestMember.java,v 1.1.1.1 2004/02/18 12:58:51 sync Exp $
 */
public abstract class TestMember extends TestCase
{
  public static final String FAILURE_EXT = ".failure";
  public static final String PROP_STATUS_PATH = "statusPath";

  private String m_testName;

  protected int nr=0;
  protected IRequester m_requester = null;
  protected HttpRequest m_request = null;
  protected String m_base = null;
  protected String m_localpart = null;
  protected String m_statusPath = null;

  /**
   * @param nr the current number of the desired test in the subclass.
   * @param s the name of the testclass.
   */
  public TestMember(Integer nr, String s) {
    super(s);
    m_testName = s;
    this.nr = nr.intValue();
  }

  public void clear(HttpRequest request) {
    request.clear();
    request.setHeader(com.sapportals.wcm.protocol.webdav.Consts.SERVLET_TRACE_HEADER,
      m_testName);
  }

  public void init(IRequester requester, String base, String localpart) {
    m_base = base;
    m_localpart = localpart;
    m_requester = requester;
    m_request = new HttpRequest(m_localpart);
    clear(m_request);
    m_request.setURI(m_localpart);

    m_statusPath = System.getProperty(PROP_STATUS_PATH,
      "c:/projects/WCM3/testcases/JUnitTestResults/");

    if (!m_statusPath.endsWith("/"))
      m_statusPath += "/";

  }

//  private void writeToFile(String contents, String filename)
//  {
//    System.err.println("Creating response file: " + filename);
//    try{
//      java.io.FileOutputStream f = new java.io.FileOutputStream(filename);
//      f.write( contents.getBytes() );
//      f.close();
//    }catch(Exception e){
//      System.err.println("could not write response to file: " + e.toString() );
//    }
//  }
//
//  private String readFromFile (java.io.InputStream f)
//  {
//    if (f == null) return null;
//
//    try{
//      ByteArrayOutputStream bos = new ByteArrayOutputStream();
//      byte[] buffer = new byte[64 * 1024];
//      int len = 0;
//      while ((len = f.read(buffer)) != -1) {
//        bos.write(buffer, 0, len);
//      }
//      f.close();
//      return bos.toString();
//    }
//    catch(Exception e){
//      System.err.println("could not read response from file: " + e.toString() );
//      return null;
//    }
//  }

  protected void checkResponseCode(IResponse response, int expected) {
    checkResponseCode(response, expected, -1, -1);
  }

  protected void checkResponseCode(IResponse response, int expected1, int expected2) {
    checkResponseCode(response, expected1, expected2, -1);
  }

  protected void checkResponseCode(IResponse response, int expected1,
                                   int expected2, int expected3) {
    int code = response.getStatus();
    if (code != expected1 && code != expected2 && code != expected3) {
      StringBuffer sb = new StringBuffer();
      sb.append("expected response codes (").append(expected1);
      if (expected2 >= 0) {
        sb.append(", ").append(expected2);
        if (expected3 >= 0) {
          sb.append(", ").append(expected3);
        }
      }
      sb.append(") got ").append(code);
      Document doc = response.getDocument();
      if (doc != null) {
        sb.append(" with body\n");
        sb.append(DomSerializer.toString(doc, true));
      }
      assertTrue (sb.toString(), false);
    }
  }

  /**
   * to be implemented by the subclasses.
   */
  protected abstract void runTest();
}
