package com.sapportals.wcm.protocol.webdav.server.sqlbench;

import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright SAP AG 2002-2003
 * @author martin.boettcher@greenbytes.de
 */

public class TraverseTest extends TestMember
{
  public static int NrOfTests = 1;

  public TraverseTest(Integer nr)
  {
    super(nr, "PROPFIND");
  }

  private Node getChild(Node parent, String localname) {
    NodeList nl = parent.getChildNodes();
    for (int i=0; i< nl.getLength(); i++) {
      Node m = nl.item(i);
      if (m.getLocalName().equals(localname)) return m;
    }
    return null;
  }

  private void checkNode(String file, Node n) throws Exception{
    String href = getChild(n, "href").getFirstChild().getNodeValue();
    boolean isCollection = getChild(getChild(getChild(getChild(n, "propstat"), "prop"), "resourcetype"), "collection")!=null;
    if (!href.equals(file)) {
      if (!isCollection) {
        checkFile(href);
      }
      else
        checkDirectory(href);
    }
  }

  private static int sdirs, sfiles;
  private static long sbytes;

  private void checkFile(String file) throws Exception{
    sfiles++;
    if (((sdirs+sfiles) % 1000==0)) System.out.println(sdirs+sfiles + " entries (" + sdirs + "/" + sfiles + ") with " + sbytes + " bytes checked");

    IResponse response;
    clear(m_request);
    m_request.expectsResponseStream(true);
    m_request.setMethod("GET");
    m_request.setURI(file);
    response = m_requester.perform( m_request );
    checkResponseCode(response, 200);

    if (response.hasStream()) {
      int l = 0;
      byte[] buffer = new byte[8192];
      InputStream is = response.getStream();
      for (int i = 0; i!=-1; i=is.read(buffer)) {
        l+=i;
      }
      is.close();
      sbytes += l;
    }
  }

  private void checkDirectory(String file) throws Exception{
    sdirs++;
    if (((sdirs+sfiles) % 1000==0)) System.out.println(sdirs+sfiles + " entries (" + sdirs + "/" + sfiles + ") with " + sbytes + " bytes checked");

    IResponse response;
    clear(m_request);
    m_request.setMethod("PROPFIND");
    m_request.setHeader("Depth", "1");
    m_request.setURI(file);
    response = m_requester.perform( m_request );
    checkResponseCode(response, 207);

    if (response.hasDocument()) {
      NodeList children = response.getDocument().getElementsByTagNameNS("DAV:", "response");
      for (int i=0; i<children.getLength(); i++) {
        checkNode(file, children.item(i));
      }
    }
  }

  protected void runTest()
  {
    try{
      checkDirectory(m_localpart + "/");
    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
    System.out.println("checked directories: " + sdirs);
    System.out.println("checked files:       " + sfiles);
    System.out.println("read contentbytes:   " + sbytes);
//    System.err.println( this.getName() + " took " + (System.currentTimeMillis()-start) + "msec.");
  }
}
