package com.sapportals.wcm.protocol.webdav.server.tests;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.sapportals.wcm.WcmException;
import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.DomSerializer;
import com.sapportals.wcm.protocol.webdav.dompath.DomPath;
import com.sapportals.wcm.util.http.HttpRequest;
import com.sapportals.wcm.util.http.IRequester;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for ACL inheritance.
 * <p>
 * Copyright (c) SAP AG 2002-2004
 * @author martin.boettcher@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: AclInheritanceTest.java,v 1.7 2005/02/18 12:46:59 sei Exp $
 */

public class AclInheritanceTest extends TestMember {

  private static final String[] desc = new String[]
  {
    "00 AI Setting up ACL",
    "01 AI PROPFIND principal-collection-set",
    "02 AI REPORT principal-match self, no props",
    "03 AI MKCOL new collection",
    "04 AI new collection, read user",
    "05 AI read user/write user",
    "06 AI read/write user",
    "07 AI PUT new file in acl controlled collection",
    "08 AI PROPFIND supported-privilege-set on file",
    "09 AI enable versioning on file",
    "10 AI PROPFIND supported-privilege-set on versionhistory",
    "11 AI PROPFIND supported-privilege-set on version",
    "12 AI revoke permissions, check inheritance",
    "13 AI PROPFIND supported-privilege-set on version after delete of VCR",
    "   AI cleanup",
  };

  public static final int NrOfTests = desc.length;

  private static final DomPath DP_hrefs = new DomPath(
      "/D:multistatus/D:response/D:href", DomPath.DEFAULT_NAMESPACE_BINDINGS);

  private static final String COLLECTION = "/aclfolder";
  private static final String FILE = "/aclfolder/aclfile";

  private static Set principalCollectionSet;
//  private static String selfCollection;
  private static String selfURI;
  private static String self2URI;
  private static String vhruri = null;
  private static String checkedInUri = null;
  private static String orgACLs = null;

  private static IRequester requester2 = null;
  private static HttpRequest request2 = new HttpRequest();

  public AclInheritanceTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest() {
    if (nr == 0) super.logIdentifier("ACLINHERITANCE");

    try {
      if (requester2 == null) {
        requester2 = getSecondRequester();
      }

      String respfile = "ACLINHERITANCE-" + TestMember.THREEDIGITS.format(nr) + ".txt";
      IResponse response;

      clear(request);
      clear(request2);

      switch (nr) {
        case 0:
          break;

        case 1:
          principalCollectionSet = DomElement.getHrefs(
            getProperty(getLocalpart() + "/", Consts.N_principal_collection_set));
          break;

        case 2:
          List principals = new ArrayList();

          for (Iterator it = principalCollectionSet.iterator(); it.hasNext(); /**/) {
            String url = (String) it.next();
            String base = getBase();
            if (url.startsWith(base)) url = url.substring(base.length());

            clear(request);
            request.setMethod("REPORT");
            request.setURI(url);
            request.setHeader("Depth", "0");
            request.setBody("<principal-match xmlns='DAV:'><self/></principal-match>");
            request.setContentType("text/xml");
            response = requester.perform(request);
            checkResponseCode(response, 207);
            Document doc = response.getDocument();
            assertTrue("document missing", doc != null);
            Node[] hrefs = DP_hrefs.matchNodes(doc);
            for (int j = 0; j < hrefs.length; ++j) {
//              selfCollection = url;
              principals.add(DomElement.getText(hrefs[j]));
            }
          }
          assertTrue("unexpected number of matching principals: " + principals,
                     principals.size() == 1);
          selfURI = (String) principals.get(0);

          principals.clear();
          for (Iterator it = principalCollectionSet.iterator(); it.hasNext(); /**/) {
            String url = (String) it.next();
            String base = getBase();
            if (url.startsWith(base)) url = url.substring(base.length());

            clear(request);
            request2.setMethod("REPORT");
            request2.setURI(url);
            request2.setHeader("Depth", "0");
            request2.setBody("<principal-match xmlns='DAV:'><self/></principal-match>");
            request2.setContentType("text/xml");
            response = requester2.perform(request2);
            checkResponseCode(response, 207);
            Document doc = response.getDocument();
            assertTrue("document missing", doc != null);
            Node[] hrefs = DP_hrefs.matchNodes(doc);
            for (int j = 0; j < hrefs.length; ++j) {
//              selfCollection = url;
              principals.add(DomElement.getText(hrefs[j]));
            }
          }
          assertTrue("unexpected number of matching principals for 2nd user: " + principals,
                     principals.size() == 1);
          self2URI = (String) principals.get(0);
          break;


        case 3:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + COLLECTION);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request2);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          checkMultistatus(
              "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
              respfile);
          break;

        case 4:
          setReadOnly(COLLECTION);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          checkMultistatus(
              "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
              respfile);
          break;

        case 5:
          clear(request);
          request.setMethod("ACL");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          request.setContentType("text/xml");
          request.setBody("<acl xmlns='DAV:'>"
                          + "<ace><principal><href>" + selfURI + "</href></principal>"
                          + "  <grant><privilege><read/></privilege></grant>"
                          + "</ace>"
                          + "<ace><principal><href>" + selfURI + "</href></principal>"
                          + "  <grant><privilege><write/></privilege></grant>"
                          + "</ace>"
                          + "<ace><principal><href>" + selfURI + "</href></principal>"
                          + "  <grant><privilege><write-acl/></privilege></grant>"
                          + "</ace>"
                          + "</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          checkMultistatus(
              "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
              respfile);
          break;

        case 6:
          setReadWrite(COLLECTION);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          checkMultistatus(
              "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
              respfile);
          break;

        case 7:
          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + FILE);
          request.setContentType("text/plain");
          request.setBody("this is a text file.");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          checkMultistatus(
              "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
              respfile);
          break;

        case 8:
          orgACLs = DomSerializer.toString(getProperty(getLocalpart() + FILE, Consts.N_acl), false);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          checkMultistatus(
              "<propfind xmlns='DAV:'><prop><supported-privilege-set/></prop></propfind>",
              respfile);
          break;

        case 9:
          clear(request);
          request.setMethod("VERSION-CONTROL");
          request.setURI(getLocalpart() + FILE);
          response = requester.perform(request);
          checkResponseCode(response, 200, 201);
          if (response.getStatus() == 201) {
            checkedInUri = getRelativeUriFromLocationHeader(response);
          }
          else {
            checkedInUri = getRelativeUriFromPropertyHrefs(
              getProperty(getLocalpart() + FILE, Consts.N_checked_in));
          }

          Element vhelem = getProperty(getLocalpart() + FILE, Consts.N_version_history);
          vhruri = super.getRelativeUriFromPropertyHrefs(vhelem);
          break;

        case 10:  if (vhruri != null) {
            String got = DomSerializer.toString(getProperty(vhruri, Consts.N_acl), false);
            assertTrue("expected ACLS: " + orgACLs + ", got: " + got, orgACLs.equals(got));
          }
          break;

        case 11:  if (checkedInUri != null) {
            String got = DomSerializer.toString(getProperty(checkedInUri, Consts.N_acl), false);
            assertTrue("expected ACLS: " + orgACLs + ", got: " + got, orgACLs.equals(got));
          }
          break;
        
        case 12: if (checkedInUri != null && vhruri != null) {
            // give permissions on VCR
            setReadWrite(FILE);

            // get old values
            String origVhrACLs = DomSerializer.toString(getProperty(vhruri, Consts.N_acl), false);
            String origVersionACLs = DomSerializer.toString(getProperty(checkedInUri, Consts.N_acl), false);
            
            // see that the other use can access the version
            clear(request);
            request.setMethod("GET");
            request.setURI(checkedInUri);
            response = requester2.perform(request);
            checkResponseCode(response, 200);
            
            // revoke permissions on VCR
            setAllOnlyMyself(FILE);
            
            // see that the other use can't access the version anymore
            clear(request);
            request.setMethod("GET");
            request.setURI(checkedInUri);
            response = requester2.perform(request);
            checkResponseCode(response, 403);

            // check current values
            String got = DomSerializer.toString(getProperty(vhruri, Consts.N_acl), false);
            assertTrue("ACLs did not change on VHR: " + got, ! origVhrACLs.equals(got));
            
            got = DomSerializer.toString(getProperty(checkedInUri, Consts.N_acl), false);
            assertTrue("ACLs did not change on version: " + got, ! origVersionACLs.equals(got));
          }
          break;

        case 13: if (super.vhrPreserved()) {
            String origVhrACLs = vhruri == null ? null : DomSerializer.toString(removeInherited(getProperty(vhruri, Consts.N_acl)), false);
            String origVersionACLs = checkedInUri == null ? null : DomSerializer.toString(removeInherited(getProperty(checkedInUri, Consts.N_acl)), false);
            
            clear(request);
            request.setMethod("DELETE");
            request.setURI(getLocalpart() + FILE);
            response = requester.perform(request);
            checkResponseCode(response, 204);
            
            if (vhruri != null) {
              clear(request);
              request.setMethod("GET");
              request.setURI(vhruri);
              response = requester.perform(request);
              checkResponseCode(response, 200, 404);
              if (response.getStatus() != 404) {                
                String got = DomSerializer.toString(getProperty(vhruri, Consts.N_acl), false);
                assertTrue("ACLs did change on VHR. Was: " + origVhrACLs + ", Got: " + got, origVhrACLs.equals(got));
              }
            }
            
            if (checkedInUri != null) {
              clear(request);
              request.setMethod("GET");
              request.setURI(checkedInUri);
              response = requester.perform(request);
              checkResponseCode(response, 200, 404);              
              if (response.getStatus() != 404) {             
                String got = DomSerializer.toString(removeInherited(getProperty(checkedInUri, Consts.N_acl)), false);
                assertTrue("ACLs did change on version. Was: " + origVersionACLs + ", Got: " + got, origVersionACLs.equals(got));
              }
            }
            
          }
          break;
          
        default: {
            setReadWrite(COLLECTION);            
            setReadWrite(FILE, true);            
  
            clear(request);
            request.setMethod("DELETE");
            request.setURI(getLocalpart() + COLLECTION);
            response = requester.perform(request);
            checkResponseCode(response, 204);
            
            if (super.vhrPreserved()) {
              clear(request);
              request.setMethod("DELETE");
              request.setURI(vhruri);
              response = requester.perform(request);
              checkResponseCode(response, 204);
            }
          }
          break;
      }

    }
    catch (Exception ex) {
      StringWriter sw = new StringWriter();
      ex.printStackTrace(new PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

  private void setReadOnly(String uri) throws WcmException {
    clear(request);
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'>"
                    + "<ace><principal><href>" + self2URI + "</href></principal>"
                    + "  <grant><privilege><read/></privilege></grant>"
                    + "</ace>"
                    + "<ace><principal><href>" + selfURI + "</href></principal>"
                    + "  <grant><privilege><write-acl/></privilege></grant>"
                    + "</ace>"
                    + "</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

//  private void setWriteNoDelete(String uri) throws WcmException {
//    clear(request);
//    request.setMethod("ACL");
//    request.setURI(getLocalpart() + uri);
//    request.setContentType("text/xml");
//    request.setBody("<acl xmlns='DAV:' xmlns:sap='" + Consts.NS_SAP + "'>"
//                    + "<ace><principal><href>" + self2URI + "</href></principal>"
//                    + "  <grant><privilege><sap:read/></privilege>"
//                    + "         <privilege><sap:write/></privilege></grant>"
//                    + "</ace>"
//                    + "<ace><principal><href>" + selfURI + "</href></principal>"
//                    + "  <grant><privilege><write-acl/></privilege></grant>"
//                    + "</ace>"
//                    + "</acl>");
//    IResponse response = requester.perform(request);
//    checkResponseCode(response, 200);
//    clear(request);
//  }

  
  private void setReadWrite(String uri) throws WcmException {
    setReadWrite(uri, false);
  }
  
  private void setReadWrite(String uri, boolean mayBeAbsent) throws WcmException {
    clear(request);
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'>"
                    + "<ace><principal><href>" + self2URI + "</href></principal>"
                    + "  <grant><privilege><read/></privilege><privilege><write/></privilege></grant>"
                    + "</ace>"
                    + "<ace><principal><href>" + selfURI + "</href></principal>"
                    + "  <grant><privilege><write-acl/></privilege></grant>"
                    + "</ace>"
                    + "</acl>");
    IResponse response = requester.perform(request);
    if (mayBeAbsent)
      checkResponseCode(response, 200, 404);
    else
      checkResponseCode(response, 200);
    clear(request);
  }

  private Element removeInherited(Element doc) {
    List l = DomElement.getChildren(doc, Consts.N_ace);
    assertTrue(l.size() == 1);
    
    Element ace = (Element)l.get(0);
    List l2 = DomElement.getChildren(ace, Consts.N_inherited);
    if (!l2.isEmpty()) {
      ace.removeChild((Element)l2.get(0));
    }    
    return doc;
  }
  
  private void setAllOnlyMyself(String uri) throws WcmException {
    clear(request);
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'>"
                    + "<ace><principal><href>" + selfURI + "</href></principal>"
                    + "  <grant><privilege><all/></privilege></grant>"
                    + "</ace>"
                    + "</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

//  private void setAllForAll(String uri) throws WcmException {
//    clear(request);
//    request.setMethod("ACL");
//    request.setURI(getLocalpart() + uri);
//    request.setContentType("text/xml");
//    request.setBody("<acl xmlns='DAV:'>"
//                    + "<ace><principal><all/></principal>"
//                    + "  <grant><privilege><all/></privilege></grant>"
//                    + "</ace>"
//                    + "<ace><principal><href>" + selfURI + "</href></principal>"
//                    + "  <grant><privilege><write-acl/></privilege></grant>"
//                    + "</ace>"
//                    + "</acl>");
//    IResponse response = requester.perform(request);
//    checkResponseCode(response, 200);
//    clear(request);
//  }

//  private void setWriteAclOnly(String uri) throws WcmException {
//    clear(request);
//    request.setMethod("ACL");
//    request.setURI(getLocalpart() + uri);
//    request.setContentType("text/xml");
//    request.setBody("<acl xmlns='DAV:'>"
//                    + "<ace><principal><href>" + selfURI + "</href></principal>"
//                    + "  <grant><privilege><write-acl/></privilege></grant>"
//                    + "</ace>"
//                    + "</acl>");
//    IResponse response = requester.perform(request);
//    checkResponseCode(response, 200);
//    clear(request);
//  }

//  private void setNone(String uri) throws WcmException {
//    clear(request);
//    request.setMethod("ACL");
//    request.setURI(getLocalpart() + uri);
//    request.setContentType("text/xml");
//    request.setBody("<acl xmlns='DAV:'></acl>");
//    IResponse response = requester.perform(request);
//    checkResponseCode(response, 400);
//    clear(request);
//  }
}
