package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.sapportals.wcm.WcmException;
import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DavConditions;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.DomSerializer;
import com.sapportals.wcm.protocol.webdav.dompath.DomPath;
import com.sapportals.wcm.util.http.HttpRequest;
import com.sapportals.wcm.util.http.IRequester;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for ACL method.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2005
 * @author stefan.eissing@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: AclTest.java,v 1.12 2005/04/18 13:48:38 jre Exp $
 */

public class AclTest extends TestMember
{

  private static final String[] desc = new String[]
  {
    "00 ACL Setting up ACL",
    "01 ACL PROPFIND supported-privilege-set",
    "02 ACL PROPFIND acl",
    "03 ACL PROPFIND acl-semantics",
    "04 ACL PROPFIND current-user-privilege-set",
    "05 ACL PROPFIND principal-collection-set",
    "06 ACL REPORT acl-principal-prop-set",
    "07 ACL REPORT principal-match self, no props",
    "08 ACL REPORT principal-match self, prop displayname",
    "09 ACL REPORT principal-match principal-property owner",
    "10 ACL MKCOL new collection",
    "11 ACL new collection, read user",
    "12 ACL DELETE unwriteable collection",
    "13 ACL read user/write user",
    "14 ACL read/write user",
    "15 ACL PUT new file in acl controlled collection",
    "16 ACL PROPFIND supported-privilege-set on file",
    "17 ACL new file, read only",
    "18 ACL DELETE unwriteable file",
    "19 ACL write, no delete file, PROPFIND",
    "20 ACL PROPFIND on file, not listable",
    "21 ACL all for all on file",
    "22 ACL write-acl only on file",
    "23 ACL nothing only on file",
    "24 ACL all for only myself on file",
    "25 ACL check resourcetype of owner href",
    "26 ACL check supported-report-set",
    "27 ACL check principal-search-properties",
    "28 ACL principal search",
    "29 ACL principal search (non searchable prop)",
    "30 ACL (unused)",
    "31 ACL (unused)",
    "32 ACL precondition failure DAV:recognized-principal",
    "33 ACL precondition failure DAV:supported-privilege",
    "34 ACL precondition failure DAV:no-abstract",
    "35 ACL precondition failure DAV:grant-only",
    "36 ACL (unused)",
    "37 ACL readonly collection, RENAME child",
    "38 ACL readonly collection, MOVE child",
    "39 ACL readonly collection, PUT child",
    "40 ACL readonly collection, COPY child",
    "41 ACL readonly collection, DELETE child",
    "42 ACL readonly collection, MKCOL child",
    "43 ACL Inheritance, child inherits DAV:acl",
    "44 ACL Inheritance, child inherits again after acl removal",
    "45 ACL Inheritance, child keeps own after parent acl removal",
    "46 ACL Inheritance, parent without acl remove deep",
    "47 ACL Inheritance, parent with own acl remove deep",
    "48 ACL give read permission to DAV:authenticated",
    "49 ACL give read permission to DAV:unauthenticated",
    "50 ACL search test",
    "51 ACL GET on restricted resource",
    "52 ACL HEAD/OPTIONS/PROPFIND on restricted resource",
    "53 ACL readwrite collection, readonly child, DELETE collection",
    "   ACL cleanup",
  };

  public static final int NrOfTests = desc.length;

  private static final DomPath DP_hrefs = new DomPath(
    "/D:multistatus/D:response/D:href", DomPath.DEFAULT_NAMESPACE_BINDINGS);

  private static final DomPath DP_acl_report_pps = new DomPath(
    "/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:supported-report-set/D:supported-report/D:report/D:principal-property-search",
    DomPath.DEFAULT_NAMESPACE_BINDINGS);

  private static final DomPath DP_acl_report_psps = new DomPath(
    "/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:supported-report-set/D:supported-report/D:report/D:principal-search-property-set",
    DomPath.DEFAULT_NAMESPACE_BINDINGS);

  private static final DomPath DP_acl_report_psps_response = new DomPath(
    "/D:principal-search-property-set/D:principal-search-property/D:prop/D:displayname",
    DomPath.DEFAULT_NAMESPACE_BINDINGS);

  private static final DomPath DP_group_membership_href = new DomPath(
    "//D:group-membership/D:href",
    DomPath.DEFAULT_NAMESPACE_BINDINGS);

  private static final String COLLECTION = "/aclfolder";
  private static final String FILE = "/aclfolder/aclfile";
  private static final String FILE2 = "/aclfolder/aclfile.bak";
  private static final String COLLECTION2 = "/aclfolder/inherit";
  private static final String FILE3 = "/aclfolder/inherit/inheritedAcl";
  
  private static final String FILECONTENT = "this is a text file.";
  
  private static final List principalCollections = new ArrayList(10);
  private static String selfCollection;
  private static String selfURI;
  private static String self2URI;

  private static DomPath DP_owner_href = null;
  private static DomPath DP_resourcetype_principal = null;

  private static IRequester requester2 = null;
  private static HttpRequest request2 = new HttpRequest();

  static {
    DP_owner_href = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:owner/D:href",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
    DP_resourcetype_principal = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:resourcetype/D:principal",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
  }

  public static String getSelfURI() {
    return AclTest.selfURI; 
  }
  
  public AclTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("ACL");

    try {
      if (requester2 == null) {
        requester2 = getSecondRequester();
      }

      String respfile = "ACL-" + TestMember.THREEDIGITS.format(nr) + ".txt";
      IResponse response;

      clear(request);
      clear(request2);

      switch (nr)
      {
        case 0:
          break;

        case 1:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/");
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><supported-privilege-set/></prop></propfind>",
            respfile);
          break;

        case 2:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/");
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 3: {
            Element acl_semantics = getProperty(getLocalpart() + "/", Consts.N_acl_semantics);
            Element acl_restrictions = getProperty(getLocalpart() + "/", Consts.N_acl_restrictions);
            
            if (acl_semantics != null) logComment("supportOldAcl");
            
            String s;
            String expected;
            
            if (acl_semantics != null) {
              s = DomSerializer.toString(acl_semantics, false);
              expected = "<D:acl-semantics pf:hidden=\"true\" xmlns:pf=\"http://sapportals.com/xmlns/cm/webdav\" pf:protected=\"true\" xmlns:D=\"DAV:\">"
                + "<D:ace-combination><D:first-match/></D:ace-combination><D:ace-ordering/><D:allowed-ace><D:grant-only/><D:no-invert/></D:allowed-ace></D:acl-semantics>";
              
              assertTrue("acl-semantics property is present, but doesn't have expected value\n" +
                    "expected: " + expected + "\ngot: " + s, s.equals(expected));
                 
            }
            
            s = DomSerializer.toString(acl_restrictions, false);
            expected = "<D:acl-restrictions pf:hidden=\"true\" xmlns:pf=\"http://sapportals.com/xmlns/cm/webdav\" pf:protected=\"true\" xmlns:D=\"DAV:\"><D:grant-only/><D:no-invert/></D:acl-restrictions>";
                
            assertTrue("acl-restrictions property is present, but doesn't have expected value\n" +
                  "expected: " + expected + "\ngot: " + s, s.equals(expected));
          }
          break;

        case 4:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/");
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><owner/><current-user-privilege-set/></prop></propfind>",
            respfile);
          break;

        case 5: {
          Element pcs = getProperty(getLocalpart() + "/", Consts.N_principal_collection_set);
          assertTrue("principal-collection-set not found", pcs != null);

          List hrefs = DomElement.getChildren(pcs, Consts.N_href);
            
          for (Iterator it = hrefs.iterator(); it.hasNext(); ) {
            String href = DomElement.getText((Node)it.next());
            if (href != null && href.length() > 0) {
              principalCollections.add(href);
            }
          }
          break;
        }

        case 6:
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + "/");
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<acl-principal-prop-set xmlns='DAV:'><prop><displayname/></prop></acl-principal-prop-set>",
            respfile);
          break;

        case 7:
          List principals = new ArrayList();

          for (int i = 0; i < principalCollections.size(); ++i) {
            String url = (String)principalCollections.get(i);
            String m_base = this.getBase();
            if (url.startsWith(m_base)) url = url.substring(m_base.length());
            clear(request);
            request.setMethod("REPORT");
            request.setURI(url);
            request.setHeader("Depth", "0");
            request.setBody("<principal-match xmlns='DAV:'><self/></principal-match>");
            request.setContentType("text/xml");
            response = requester.perform(request);
            checkResponseCode(response, 207);
            Document doc = response.getDocument();
            assertTrue ("document missing", doc != null);
            Node[] hrefs = DP_hrefs.matchNodes(doc);
            for (int j = 0; j < hrefs.length; ++j) {
              selfCollection = url;
              principals.add(DomElement.getText(hrefs[j]));
            }
          }
          assertTrue ("unexpected number of matching principals: "+principals,
                 principals.size() == 1);
          selfURI = (String)principals.get(0);

          principals.clear();
          for (int i = 0; i < principalCollections.size(); ++i) {
            String url = (String)principalCollections.get(i);
            String m_base = this.getBase();
            if (url.startsWith(m_base)) url = url.substring(m_base.length());
            clear(request);
            request2.setMethod("REPORT");
            request2.setURI(url);
            request2.setHeader("Depth", "0");
            request2.setBody("<principal-match xmlns='DAV:'><self/></principal-match>");
            request2.setContentType("text/xml");
            response = requester2.perform(request2);
            checkResponseCode(response, 207);
            Document doc = response.getDocument();
            assertTrue ("document missing", doc != null);
            Node[] hrefs = DP_hrefs.matchNodes(doc);
            for (int j = 0; j < hrefs.length; ++j) {
              selfCollection = url;
              principals.add(DomElement.getText(hrefs[j]));
            }
          }
          assertTrue ("unexpected number of matching principals for 2nd user: "+principals,
                 principals.size() == 1);
          self2URI = (String)principals.get(0);
          break;

        case 8:
          assertTrue ("collection of self URI not known", selfCollection != null);
          request.setMethod("REPORT");
          request.setURI(selfCollection);
          this.checkMultistatus(
            "<principal-match xmlns='DAV:'><self/><prop><displayname/></prop></principal-match>",
            respfile);
          break;

        case 9:
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + "/");
          this.checkMultistatus(
            "<principal-match xmlns='DAV:'><principal-property><owner/></principal-property></principal-match>",
            respfile);
          break;

        case 10:
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + COLLECTION);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 11:
          setReadOnly(COLLECTION);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 12:
          request2.setMethod("DELETE");
          request2.setURI(getLocalpart() + COLLECTION);
          request2.setHeader("Depth", "Infinity");
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 13:
          request.setMethod("ACL");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          request.setContentType("text/xml");
          request.setBody("<acl xmlns='DAV:'>"
                            +"<ace><principal><href>"+selfURI+"</href></principal>"
                            +"  <grant><privilege><read/></privilege></grant>"
                            +"</ace>"
                            +"<ace><principal><href>"+selfURI+"</href></principal>"
                            +"  <grant><privilege><write/></privilege></grant>"
                            +"</ace>"
                            +"<ace><principal><href>"+selfURI+"</href></principal>"
                            +"  <grant><privilege><write-acl/></privilege></grant>"
                            +"</ace>"
                            +"</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 14:
          setReadWrite(COLLECTION);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 15:
          request.setMethod("PUT");
          request.setURI(getLocalpart() + FILE);
          request.setContentType("text/plain");
          request.setBody(FILECONTENT);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 16:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><supported-privilege-set/></prop></propfind>",
            respfile);
          break;

        case 17:
          setReadOnly(FILE);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 18:
          request2.setMethod("DELETE");
          request2.setURI(getLocalpart() + FILE);
          request2.setHeader("Depth", "Infinity");
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 19:
          setWriteNoDelete(FILE);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "1");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          request2.setMethod("DELETE");
          request2.setURI(getLocalpart() + FILE);
          request2.setHeader("Depth", "Infinity");
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          clear(request2);
          request2.setMethod("PUT");
          request2.setURI(getLocalpart() + FILE);
          request2.setHeader("Content-Type", "text/plain");
          request2.setBody("this is a little test.");
          response = requester2.perform(request2);
          checkResponseCode(response, 200);
          break;

        case 20:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "1");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 21:
          setAllForAll(FILE, this);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 22:
          setWriteAclOnly(FILE);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 23:
          setAllForAll(FILE, this);
          removeAcl(FILE);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/><owner/></prop></propfind>",
            respfile);
          break;

        case 24:
          setAllOnlyMyself(FILE, this);
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<expand-property xmlns='DAV:'><property name='acl'><property name='group-membership'/><property name='principal-URL'/><property name='alternate-URI-set'/></property><property name='owner'/></expand-property>",
            respfile, DP_group_membership_href);
          break;

        case 25:
          // PROPFIND the owner property, then PROPFIND the owner and check the
          // resourcetype
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<propfind xmlns='DAV:'><prop><owner/></prop></propfind>");
          request.setHeader("Depth", "0");
          response = requester.perform( request );
          checkResponseCode(response, 207);
          Element href = (Element) DP_owner_href.matchOrDie (response.getDocument(), 1, 1, "owner/href not found").get(0);
          String principalUri = DomElement.getText(href);
          request.clear();
          request.setMethod("PROPFIND");
          request.setURI(principalUri);
          request.setBody("<propfind xmlns='DAV:'><prop><resourcetype/></prop></propfind>");
          request.setHeader("Depth", "0");
          response = requester.perform( request );
          checkResponseCode(response, 207);
          DP_resourcetype_principal.matchOrDie (response.getDocument(), 1, 1, "resourcetype/principal not found");
          break;

        case 26:
          // CHECK presence of principal property search reports
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<propfind xmlns='DAV:'><prop><supported-report-set/></prop></propfind>");
          request.setHeader("Depth", "0");
          response = requester.perform( request );
          checkResponseCode(response, 207);
          DP_acl_report_pps.matchOrDie (response.getDocument(), 1, 1, "principal-property-search not found");
          DP_acl_report_psps.matchOrDie (response.getDocument(), 1, 1, "principal-search-property-set not found");
          break;

        case 27:
          // CHECK principal search properties
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<principal-search-property-set xmlns='DAV:'/>");
          request.expectsResponseDocument(true);
          response = requester.perform( request );
          checkResponseCode(response, 200);
          assertTrue (response.getDocument() != null);
          DP_acl_report_psps_response.matchOrDie(response.getDocument(), 1, 1, "DAV:displayname must be searchable");
          break;

        case 28:
          // search by displayname
          clear(request);
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + FILE);
          this.checkMultistatus("<principal-property-search xmlns='DAV:'><apply-to-principal-collection-set/><property-search><prop><displayname/></prop>" +
            "<match>"+m_userid2+"</match></property-search><prop><displayname/></prop></principal-property-search>",
            respfile);

          // search by displayname without apply-to-principal-collection-set
          clear(request);
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<principal-property-search xmlns='DAV:'><property-search><prop><displayname/></prop>" +
            "<match>eissing</match></property-search><prop><displayname/></prop></principal-property-search>");
          response = requester.perform(request);
          super.checkResponseCode(response, 207);
          // there shouldn't be any child elements in it...
          assertTrue(response.getDocument().getDocumentElement().getElementsByTagNameNS("DAV:","response").getLength() == 0);
          break;

        case 29:
          // search by x-y-z
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<principal-property-search xmlns='DAV:'><property-search><prop><x-y-z/></prop>"
              + "<match>user</match></property-search></principal-property-search>");
          request.expectsResponseDocument(true);
          response = requester.perform(request);
          checkResponseCode(response, 207);
          assertTrue("should be empty multistatus response, but has " + getHrefsFromMultistatus(response.getDocument()).size() + " entries.",
            getHrefsFromMultistatus(response.getDocument()).size() == 0);
          break;

        case 30:
          break;

        case 31:
          break;

        case 32:
          request.setMethod("ACL");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<acl xmlns='DAV:'>"
                        +"<ace><principal><href>"+self2URI+"doesnotexist"+"</href></principal>"
                        +"  <grant><privilege><read/></privilege></grant>"
                        +"</ace>"
                        +"</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          checkErrorElement(response, DavConditions.N_recognized_principal);
          break;

        case 33:
          request.setMethod("ACL");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<acl xmlns='DAV:'>"
                        +"<ace><principal><href>"+self2URI+"</href></principal>"
                        +"  <grant><privilege><read-does-not-exist/></privilege></grant>"
                        +"</ace>"
                        +"</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          checkErrorElement(response, DavConditions.N_supported_privilege);
          break;

        case 34:
          request.setMethod("ACL");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<acl xmlns='DAV:'>"
                        +"<ace><principal><href>"+selfURI+"</href></principal>"
                        +"  <grant><privilege><all/></privilege></grant>"
                        +"</ace>"
                        +"<ace><principal><href>"+selfURI+"</href></principal>"
                        +"  <grant><privilege><read-acl/></privilege></grant>"
                        +"</ace>"
                        +"</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          checkErrorElement(response, DavConditions.N_no_abstract);

          clear(request);
          request.setMethod("ACL");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<acl xmlns='DAV:'>"
                        +"<ace><principal><href>"+selfURI+"</href></principal>"
                        +"  <grant><privilege><read-acl/></privilege></grant>"
                        +"</ace>"
                        +"</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          checkErrorElement(response, DavConditions.N_no_abstract);
          break;

        case 35:
          request.setMethod("ACL");
          request.setURI(getLocalpart() + FILE);
          request.setBody("<acl xmlns='DAV:'>"
                        +"<ace><principal><href>"+selfURI+"</href></principal>"
                        +"  <deny><privilege><write/></privilege></deny>"
                        +"</ace>"
                        +"</acl>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          checkErrorElement(response, DavConditions.N_grant_only);
          break;

        case 36:
          break;

        case 37:
          setReadOnly(COLLECTION);
          setReadWrite(FILE);
          request2.setMethod("MOVE");
          request2.setURI(getLocalpart() + FILE);
          request2.setHeader("Destination", getLocalpart() + FILE2);
          request2.setHeader("Overwrite", "F");
          response = requester2.perform(request2);
          if (response.getStatus() == 201) {
            clear(request2);
            request2.setMethod("MOVE");
            request2.setURI(getLocalpart() + FILE2);
            request2.setHeader("Destination", getLocalpart() + FILE);
            request2.setHeader("Overwrite", "F");
            requester2.perform(request2);
          }
          checkResponseCode(response, 403);
          break;

        case 38:
          setReadOnly(COLLECTION);
          setReadWrite(FILE);
          request2.setMethod("MOVE");
          request2.setURI(getLocalpart() + FILE);
          request2.setHeader("Destination", getLocalpart() + FILE2);
          request2.setHeader("Overwrite", "T");
          response = requester2.perform(request2);
          if (response.getStatus() == 201) {
            clear(request2);
            request2.setMethod("MOVE");
            request2.setURI(getLocalpart() + FILE2);
            request2.setHeader("Destination", getLocalpart() + FILE);
            request2.setHeader("Overwrite", "T");
            requester2.perform(request2);
          }
          checkResponseCode(response, 403);
          break;

        case 39:
          request2.setMethod("PUT");
          request2.setURI(getLocalpart() + FILE2);
          request2.setBody("this is a test");
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 40:
          request2.setMethod("COPY");
          request2.setURI(getLocalpart() + FILE);
          request2.setHeader("Destination", getLocalpart() + FILE2);
          request2.setHeader("Overwrite", "T");
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 41:
          request2.setMethod("DELETE");
          request2.setURI(getLocalpart() + FILE);
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 42:
          request2.setMethod("MKCOL");
          request2.setURI(getLocalpart() + COLLECTION + "/coll3");
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 43:
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + COLLECTION2);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + FILE3);
          request.setBody("this is a test resource");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE3);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 44:
          setReadWrite(FILE3);
          removeAcl(FILE3);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE3);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 45:
          setWriteNoDelete(FILE3);
          removeAcl(COLLECTION2);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE3);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 46:
          removeAclDeep(COLLECTION2);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE3);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 47:
          setWriteNoDelete(FILE3);
          setReadOnly(COLLECTION2);
          removeAclDeep(COLLECTION2);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE3);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 48:
          setReadAuthenticated(FILE);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 49:
          setReadUnauthenticated(FILE);
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + FILE);
          request.setHeader("Depth", "0");
          this.checkMultistatus(
            "<propfind xmlns='DAV:'><prop><acl/></prop></propfind>",
            respfile);
          break;

        case 50:
          setAllOnlyMyself(FILE, this);
          
          String body = "<searchrequest xmlns='DAV:'><basicsearch><select><prop><resourcetype/></prop></select><from><scope><href>" + getLocalpart() + COLLECTION + "</href><depth>infinity</depth></scope></from><where><nsx:not xmlns:nsx='DAV:'><nsx:eq xmlns:nsx='DAV:'><nsx:prop xmlns:nsx='DAV:'><getcontentlength xmlns='DAV:'/></nsx:prop><nsx:literal xmlns:nsx='DAV:'>1234567</nsx:literal></nsx:eq></nsx:not></where></basicsearch></searchrequest>";          
          
          clear(request2);
          request2.setMethod("SEARCH");
          request2.setURI(getLocalpart() + COLLECTION);
          request2.setBody(body);
          response = requester2.perform(request2);
          checkResponseCode(response, 207);
          
          List hrefs = getHrefsFromMultistatus(response.getDocument());
          assertTrue("should NOT find resource " + FILE + " in results " + hrefs, hrefs.toString().indexOf(FILE) == -1);
          break;
          
        case 51:
          //setAllOnlyMyself(FILE, this);
          
          clear(request2);
          request2.setMethod("GET");
          request2.setURI(getLocalpart() + FILE);
          request2.expectsResponseStream(true);
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          //InputStream is = response.getStream();
          //ContentType ct = ContentType.valueOf(response.getHeader("Content-Type"));
          //checkContent(is, ct, FILECONTENT);
          break;
          
        case 52:
          //setAllOnlyMyself(FILE, this);
          
          clear(request2);
          request2.setMethod("HEAD");
          request2.setURI(getLocalpart() + FILE);
          response = requester2.perform(request2);
          checkResponseCode(response, 403);

          clear(request2);
          request2.setMethod("OPTIONS");
          request2.setURI(getLocalpart() + FILE);
          response = requester2.perform(request2);
          checkResponseCode(response, 403);

          clear(request2);
          request2.setMethod("PROPFIND");
          request2.setHeader("Depth", "0");
          request2.setURI(getLocalpart() + FILE);
          response = requester2.perform(request2);
          checkResponseCode(response, 403);
          break;

        case 53: {
            // Incomplete delete operations on collections were handled such that the
            // ACL for the non-deleted member was removed anyway. This test ensures
            // that a subsequent DELETE attempt indeed fails as it should.
          
            setAllForAll(COLLECTION, this);
            setReadOnly(FILE);
            
            request2.setMethod("DELETE");
            request2.setHeader("Depth", "Infinity");
            request2.setURI(getLocalpart() + COLLECTION);
            response = requester2.perform(request2);
            
            request2.setMethod("DELETE");
            request2.setHeader("Depth", "Infinity");
            request2.setURI(getLocalpart() + COLLECTION);
            response = requester2.perform(request2);
            checkResponseCode(response, 207);
            checkResponseDocument(response, respfile, null);
          }
          break;

        
      default:
          try {
            setReadWrite(COLLECTION);
            setReadWrite(FILE);
          }
          catch (Throwable ex) {
            // ignore failures, collection may already have been deleted
          }
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + COLLECTION);
          request.setHeader("Depth", "Infinity");
          response = requester.perform(request);
          checkResponseCode(response, 204, 404);
          break;
      }

    }
    catch (WcmException ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      ex.getContainedException().printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

  private void setReadOnly(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'>"
                      +"<ace><principal><href>"+self2URI+"</href></principal>"
                      +"  <grant><privilege><read/></privilege></grant>"
                      +"</ace>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  private void setWriteNoDelete(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:' xmlns:sap='"+Consts.NS_SAP+"'>"
                      +"<ace><principal><href>"+self2URI+"</href></principal>"
                      +"  <grant><privilege><read/></privilege>"
                      +"         <privilege><write/></privilege></grant>"
                      +"</ace>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  private void setReadWrite(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'>"
                      +"<ace><principal><href>"+self2URI+"</href></principal>"
                      +"  <grant><privilege><read/></privilege><privilege><write/></privilege></grant>"
                      +"</ace>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  public static void setAllOnlyMyself(String uri, TestMember tester) throws WcmException {
    tester.request.setMethod("ACL");
    tester.request.setURI(tester.getLocalpart() + uri);
    tester.request.setContentType("text/xml");
    tester.request.setBody("<acl xmlns='DAV:'>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><all/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = tester.requester.perform(tester.request);
    tester.checkResponseCode(response, 200);
    tester.clear(tester.request);
  }

  public static void setAllForAll(String uri, TestMember tester) throws WcmException {
    tester.request.setMethod("ACL");
    tester.request.setURI(tester.getLocalpart() + uri);
    tester.request.setContentType("text/xml");
    tester.request.setBody("<acl xmlns='DAV:'>"
                      +"<ace><principal><all/></principal>"
                      +"  <grant><privilege><all/></privilege></grant>"
                      +"</ace>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = tester.requester.perform(tester.request);
    tester.checkResponseCode(response, 200);
    tester.clear(tester.request);
  }

  private void setWriteAclOnly(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  private void setReadUnauthenticated(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:' xmlns:sap='"+Consts.NS_SAP+"'>"
                      +"<ace><principal><unauthenticated/></principal>"
                      +"  <grant><privilege><read/></privilege>"
                      +"  </grant>"
                      +"</ace>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  private void setReadAuthenticated(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:' xmlns:sap='"+Consts.NS_SAP+"'>"
                      +"<ace><principal><authenticated/></principal>"
                      +"  <grant><privilege><read/></privilege>"
                      +"  </grant>"
                      +"</ace>"
                      +"<ace><principal><href>"+selfURI+"</href></principal>"
                      +"  <grant><privilege><write-acl/></privilege></grant>"
                      +"</ace>"
                      +"</acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  private void removeAcl(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:'></acl>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }

  private void removeAclDeep(String uri) throws WcmException {
    request.setMethod("ACL");
    request.setURI(getLocalpart() + uri);
    request.setContentType("text/xml");
    request.setBody("<acl xmlns='DAV:' xmlns:s=\""+Consts.NS_SAP+"\" s:scope=\"deep\" />");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 200);
    clear(request);
  }
  
//  private void checkContent(InputStream is, ContentType ct, String reference) throws IOException {
//    try {
//      ByteArrayOutputStream bos = new ByteArrayOutputStream();
//      int read = -1;
//      byte[] buffer = new byte[1024];
//      while ((read = is.read(buffer)) >= 0) {
//        bos.write(buffer, 0, read);
//      }
//      String content = null;
//      if (ct.getEncoding() != null) {
//        content = bos.toString(ct.getEncoding());
//      }
//      else {
//        content = bos.toString(ct.getEncoding());
//      }
//
//      assertTrue("unexpected resource content: " + content + ", expected " + reference, reference.equals(content));
//    }
//    finally {
//      is.close();
//    }
//  }
}
