package com.sapportals.wcm.protocol.webdav.server.tests;

import java.io.BufferedInputStream;
import java.io.InputStream;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Tests for PUT/GET content.
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: ContentTest.java,v 1.3 2004/03/29 11:52:07 jre Exp $
 */

public class ContentTest extends TestMember
{
  private static final String[] desc = new String[]
  {
    "00 CT create test resource pattern 0xff / length 1MB",
    "01 CT verify test resource pattern 0xff / length 1MB",
    "02 CT create test resource pattern 0xe5 / length 1234567",
    "03 CT verify test resource pattern 0xe5 / length 1234567",
    "04 CT create test resource pattern 0x5e / length 10MB",
    "05 CT verify test resource pattern 0x5e / length 10MB",
    "   CT cleanup",
  };

  public static int NrOfTests = desc.length;

  public ContentTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("CT");

    // String respfile = "CONTENT-" + m_threeDigits.format (nr) + ".txt";
    String testdoc = "/content-test.bin";
    int bufferSize = 64 * 1024;
    int length;

    try {
      IResponse response;
      clear(request);

      switch (nr) {

        case 00:
          length = 1024 * 1024;
          clear(request);
          request.setMethod("PUT");
          setRelativeURI(testdoc);
          request.setContentLength(length);
          request.setBody(new BufferedInputStream(new DataGenerator(0xff, length), bufferSize));
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 01:
          length = 1024 * 1024;
          clear(request);
          request.setMethod("GET");
          setRelativeURI(testdoc);
          request.expectsResponseStream(true);
          response = requester.perform(request);
          checkResponseCode(response, 200);
          compare(response.getStream(), new DataGenerator(0xff, 1024*1024));
          break;

        case 02:
          length = 1234567;
          clear(request);
          request.setMethod("PUT");
          setRelativeURI(testdoc);
          request.setContentLength(length);
          request.setBody(new BufferedInputStream(new DataGenerator(0xe5, length), bufferSize));
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 03:
          length = 1234567;
          clear(request);
          request.setMethod("GET");
          setRelativeURI(testdoc);
          request.expectsResponseStream(true);
          response = requester.perform(request);
          checkResponseCode(response, 200);
          compare(response.getStream(), new DataGenerator(0xE5, length));
          break;

        case 04:
          length = 1024*1024*10;
          clear(request);
          request.setMethod("PUT");
          setRelativeURI(testdoc);
          request.setContentLength(length);
          request.setBody(new BufferedInputStream(new DataGenerator(0xe5, length), bufferSize));
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 05:
          length = 1024*1024*10;
          clear(request);
          request.setMethod("GET");
          setRelativeURI(testdoc);
          request.expectsResponseStream(true);
          response = requester.perform(request);
          checkResponseCode(response, 200);
          compare(response.getStream(), new DataGenerator(0xe5, length));
          break;

        default:
          clear(request);
          request.setMethod("DELETE");
          setRelativeURI(testdoc);
          requester.perform(request);
          break;
      }
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

  private void compare(InputStream x1, InputStream x2) throws java.io.IOException {

    InputStream s1 = new BufferedInputStream(x1);
    InputStream s2 = new BufferedInputStream(x2);

    int pos = 0;
    boolean done = false;

    while (!done) {
      try {
        int c1 = s1.read();
        int c2 = s2.read();
        assertTrue("mismatch at position " + pos, c1 == c2);
        done = c1 == -1 || c2 == -1;
        pos++;
      }
      catch (java.io.IOException ex) {
        System.out.println(ex.getMessage() + " + at position " + pos);
        throw ex;
      }
    }
  }

  private class DataGenerator extends InputStream {

    private int pattern;
    private int length;
    private int position;

    public DataGenerator (int pattern, int length) {
      this.pattern = pattern;
      this.length = length;
      this.position = 0;
    }

    public int read() {
      if (this.position++ < this.length) {
        return pattern;
      }
      else {
        return -1;
      }
    }
  }
}
