package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Element;

import com.sapportals.wcm.WcmException;
import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.Name;
import com.sapportals.wcm.util.http.IResponse;

/**
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author martin.boettcher@greenbytes.de
 * @version $Id: EnhancedCollectionTest.java,v 1.8 2005/05/11 12:19:48 mbo Exp $
 */

public class EnhancedCollectionTest extends TestMember {

  private static final String[] desc = new String[]
  {
    "00 EC setup (create container collection)",
    "01 EC create child resource",
    "02 EC create child collection",
    "03 EC make sure container not versioned",
    "04 EC make sure child resource not versioned",
    "05 EC make sure child collection not versioned",
    "06 EC set enhanced flag",
    "07 EC make sure container and collection member still non-versioned",
    "08 EC make sure child resource is versioned",
    "09 EC add new child",
    "10 EC make sure new child is versioned",
    "11 EC add new child collection",
    "12 EC make sure new child collection is not versioned",
    "13 EC copy into enhanced collection",
    "14 EC make sure new child is versioned",
    "15 EC copy collection into enhanced collection",
    "16 EC make sure new child collection is not versioned",
    "17 EC move into enhanced collection",
    "18 EC make sure new child is versioned",
    "19 EC move collection into enhanced collection",
    "20 EC make sure new child collection is not versioned",
    "21 EC lock/put/proppatch/unlock, then check # of versions",
    "22 EC SEARCH for enhanced collections",
    "23 EC set enhanced flag to false",
    "24 EC SEARCH for enhanced collections",    
    "   EC cleanup",
  };

  public static int NrOfTests = desc.length;
  public static boolean ecSupport;
  public static boolean notUnset;
  public static String locktoken;
  public static final String CONTENT1 = "initial content";
  public static final String CONTENT2 = "this is also an initial content";
  public static Set vhrs = new HashSet();

  public EnhancedCollectionTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }
  
  static String orgCreationDate, orgModificationDate, modification1;
  
  protected void runTest() {
    if (nr == 0) logIdentifier("EC");

    Name pec = new Name("http://sapportals.com/xmlns/cm", "enhancedcollection");

    try {
      IResponse response;

      switch (nr) {

        case 0:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/enhancedcol");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/sourcecontainer");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + "/sourcecontainer/plainsrc");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/sourcecontainer/colsrc");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          Element prop = getProperty(getLocalpart() + "/enhancedcol", pec);
          ecSupport = (prop != null);
          logComment("enhancedcollections:" + ecSupport);
          break;

        case 1:
          if (ecSupport) {
            clear(request);
            request.setMethod("PUT");
            request.setURI(getLocalpart() + "/enhancedcol/novers.txt");
            request.setBody(CONTENT1);
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 2:
          if (ecSupport) {
            clear(request);
            request.setMethod("MKCOL");
            request.setURI(getLocalpart() + "/enhancedcol/novers");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 3:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol", Consts.N_checked_in));
          }
          break;

        case 4:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_checked_in));
            
            orgCreationDate = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_creationdate).getChildNodes().item(0).toString();
            orgModificationDate = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_getlastmodified).getChildNodes().item(0).toString();            
          }
          break;

        case 5:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol/novers", Consts.N_checked_in));
          }
          break;

        case 6:
          if (ecSupport) {
            //wait two seconds so that the lastmod-date of the resources will actually change!
            try{
              Thread.sleep(2000);
            }
            catch(InterruptedException ignore) {
              //
            }
            setProperty(getLocalpart() + "/enhancedcol", pec, Boolean.TRUE, true);

            prop = getProperty(getLocalpart() + "/enhancedcol", pec);
            assertTrue(DomElement.getText(prop).equals("true"));
          }
          break;

        case 7:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol", Consts.N_checked_in));
            assertNull(getProperty(getLocalpart() + "/enhancedcol/novers", Consts.N_checked_in));
          }
          break;

        case 8:
          if (ecSupport) {
            Element p = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_checked_in);
            assertNotNull(p);
            String versionURI = getRelativeUriFromPropertyHrefs(p);
            
            clear(request);
            request.setMethod("HEAD");
            request.setURI(versionURI);
            response = requester.perform(request);
            checkResponseCode(response, 200);
            
            assertTrue("contentlength of version (" + response.getContentLength() + ") has not expected length " + CONTENT1.length(), response.getContentLength()==CONTENT1.length());

            // remember VHR URI for later
            Element vhr = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_version_history);
            vhrs.add(getRelativeUriFromPropertyHrefs(vhr));
            
            String creation = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_creationdate).getChildNodes().item(0).toString();
            modification1 = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_getlastmodified).getChildNodes().item(0).toString();
            assertEquals("creation date of VCR may not change", orgCreationDate, creation);
            assertFalse("modification date of VCR has to change but didn't", orgModificationDate.equals(modification1));            
          }
          break;

        case 9:
          if (ecSupport) {
            clear(request);
            request.setMethod("PUT");
            request.setURI(getLocalpart() + "/enhancedcol/vers.txt");
            request.setBody(CONTENT2);
            response = requester.perform(request);
            checkResponseCode(response, 201);
            
            //wait two seconds so that the lastmodied date changes!
            try{
              Thread.sleep(2000);
            }
            catch(InterruptedException ignore) {
              //
            }
            
            clear(request);
            request.setMethod("PUT");
            request.setURI(getLocalpart() + "/enhancedcol/novers.txt");
            request.setBody(CONTENT2);
            response = requester.perform(request);
            checkResponseCode(response, 200);
            
            String creation = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_creationdate).getChildNodes().item(0).toString();
            String modification = getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_getlastmodified).getChildNodes().item(0).toString();
            assertEquals("creation date of VCR may not change", orgCreationDate, creation);
            assertFalse("modification date of VCR has to be different from orginal one but isn't", orgModificationDate.equals(modification));
            assertFalse("modification date of VCR has to change but didn't", modification1.equals(modification));
          }
          break;

        case 10:
          if (ecSupport) {
            Element p = getProperty(getLocalpart() + "/enhancedcol/vers.txt", Consts.N_checked_in);
            assertNotNull(p);
            
            String versionURI = getRelativeUriFromPropertyHrefs(p);
            
            clear(request);
            request.setMethod("HEAD");
            request.setURI(versionURI);
            response = requester.perform(request);
            checkResponseCode(response, 200);
            
            assertTrue("contentlength of version (" + response.getContentLength() + ") has not expected length " + CONTENT2.length(), response.getContentLength()==CONTENT2.length());

            // remember VHR URI for later
            Element vhr = getProperty(getLocalpart() + "/enhancedcol/vers.txt", Consts.N_version_history);
            vhrs.add(getRelativeUriFromPropertyHrefs(vhr));
          }
          break;

        case 11:
          if (ecSupport) {
            clear(request);
            request.setMethod("MKCOL");
            request.setURI(getLocalpart() + "/enhancedcol/col");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 12:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol/col", Consts.N_checked_in));
          }
          break;

        case 13:
          if (ecSupport) {
            clear(request);
            request.setMethod("COPY");
            request.setURI(getLocalpart() + "/sourcecontainer/plainsrc");
            request.setHeader("Destination", this.getBaseAndLocalpart() + "/enhancedcol/plaindest");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 14:
          if (ecSupport) {
            assertNotNull(getProperty(getLocalpart() + "/enhancedcol/plaindest", Consts.N_checked_in));

            // remember VHR URI for later
            Element vhr = getProperty(getLocalpart() + "/enhancedcol/plaindest", Consts.N_version_history);
            vhrs.add(getRelativeUriFromPropertyHrefs(vhr));
          }
          break;

        case 15:
          if (ecSupport) {
            clear(request);
            request.setMethod("COPY");
            request.setURI(getLocalpart() + "/sourcecontainer/colsrc");
            request.setHeader("Destination", this.getBaseAndLocalpart() + "/enhancedcol/coldest");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 16:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol/coldest", Consts.N_checked_in));
          }
          break;

        case 17:
          if (ecSupport) {
            clear(request);
            request.setMethod("MOVE");
            request.setURI(getLocalpart() + "/sourcecontainer/plainsrc");
            request.setHeader("Destination", this.getBaseAndLocalpart() + "/enhancedcol/plaindestmv");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 18:
          if (ecSupport) {
            assertNotNull(getProperty(getLocalpart() + "/enhancedcol/plaindestmv", Consts.N_checked_in));

            // remember VHR URI for later
            Element vhr = getProperty(getLocalpart() + "/enhancedcol/plaindestmv", Consts.N_version_history);
            vhrs.add(getRelativeUriFromPropertyHrefs(vhr));
          }
          break;

        case 19:
          if (ecSupport) {
            clear(request);
            request.setMethod("MOVE");
            request.setURI(getLocalpart() + "/sourcecontainer/colsrc");
            request.setHeader("Destination", this.getBaseAndLocalpart() + "/enhancedcol/coldestmv");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 20:
          if (ecSupport) {
            assertNull(getProperty(getLocalpart() + "/enhancedcol/coldestmv", Consts.N_checked_in));
          }
          break;

        case 21:
          if (ecSupport) {
            clear(request);
            request.setMethod("LOCK");
            request.setURI(getLocalpart() + "/enhancedcol/lockput");
            request.setBody("<lockinfo xmlns='DAV:'><lockscope><exclusive/></lockscope><locktype><write/></locktype></lockinfo>");
            response = requester.perform(request);
            checkResponseCode(response, 201);
            locktoken = getLockToken(response);
            assertTrue(locktoken != null);
            
            clear(request);
            request.setMethod("PUT");
            request.setURI(getLocalpart() + "/enhancedcol/lockput");
            request.setHeader("if", "(<" + locktoken + ">)");
            request.setBody("foobar");
            response = requester.perform(request);
            checkResponseCode(response, 200, 201);
            
            clear(request);
            request.setMethod("PROPPATCH");
            request.setURI(getLocalpart() + "/enhancedcol/lockput");
            request.setHeader("if", "(<" + locktoken + ">)");
            request.setBody("<propertyupdate xmlns='DAV:'><set><prop><foobar/></prop></set></propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 207);
            
            clear(request);
            request.setMethod("UNLOCK");
            request.setURI(getLocalpart() + "/enhancedcol/lockput");
            request.setHeader("lock-token", "<" + locktoken + ">");
            response = requester.perform(request);
            checkResponseCode(response, 204);
            locktoken = null;

            assertTrue("resource not under version control", null != getProperty(getLocalpart() + "/enhancedcol/lockput", Consts.N_checked_in));
            
            Element vhr = getProperty(getLocalpart() + "/enhancedcol/lockput", Consts.N_version_history);
            if (vhr != null) {
              String href = getRelativeUriFromPropertyHrefs(vhr);
              Element vs = getProperty(href, Consts.N_version_set);
              assertTrue("DAV:version-set present on VHR", vs != null);
              int cnt = DomElement.getChildren(vs, Consts.N_href).size();
              assertTrue("should have one version after UNLOCK, but has " + cnt, cnt == 1); 
              
              vhrs.add(href);
            }
          }
          break;

        case 22:
          assertTrue(getEnhancedCollections(getLocalpart()).size() == 1);
          break;

        case 23:
          if (ecSupport) {
            setProperty(getLocalpart() + "/enhancedcol", pec, Boolean.FALSE, true);

            prop = getProperty(getLocalpart() + "/enhancedcol", pec);
            assertTrue(DomElement.getText(prop).equals("false"));

            assertNull(getProperty(getLocalpart() + "/enhancedcol", Consts.N_checked_in));
            assertNull("child still version-controlled", getProperty(getLocalpart() + "/enhancedcol/novers.txt", Consts.N_checked_in));
          }
          break;
          
        case 24:
          assertTrue(getEnhancedCollections(getLocalpart()).size() == 0);
          break;
          
        default: {

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + "/enhancedcol");
          if (locktoken != null) {
            request.setHeader("If", "<" + getLocalpart() + "/enhancedcol/lockput"
                                    + ">(<" + locktoken + ">)");
          }
          response = requester.perform(request);
          checkResponseCode(response, 204, 200);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + "/sourcecontainer");
          response = requester.perform(request);
          checkResponseCode(response, 204, 200);
          
          // cleanup VHRs
          for (Iterator it = vhrs.iterator(); it.hasNext(); ) {
            String uri = it.next().toString();
            clear(request);
            request.setMethod("DELETE");
            request.setURI(uri);
            response = requester.perform(request);
            checkResponseCode(response, vhrPreserved() ? 204 : 404);
          }
          
          break;
        }
      }
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
  
  private List getEnhancedCollections(String base) throws WcmException {
      
    clear(request);
    request.setMethod("SEARCH");
    request.setURI(base);
    request.expectsResponseDocument(true);
    request.setContentType("application/xml");
    request.setBody("<searchrequest xmlns='DAV:'><basicsearch>" + "<select><prop><resourcetype/></prop></select>"
      + "<from><scope><href>" + base + "</href><depth>infinity</depth></scope></from>" + "<where>" + "<and>" + "<is-collection/>"
      + "<eq><prop><enhancedcollection xmlns='http://sapportals.com/xmlns/cm'/></prop><literal>1</literal></eq>" + "</and>"
      + "</where>" + "</basicsearch></searchrequest>");
    IResponse response = requester.perform(request);
    checkResponseCode(response, 207);
    return getHrefsFromMultistatus(response.getDocument());
  }
  
}
