package com.sapportals.wcm.protocol.webdav.server.tests;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;

import junit.framework.TestSuite;

import com.sapportals.wcm.util.http.*;
import com.sapportals.wcm.util.uri.RID;

/**
 * Generates various tests for a certain class.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2005
 * @author martin.boettcher@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: FSTestSuite.java,v 1.8 2005/10/11 16:03:01 mko Exp $
 */

public class FSTestSuite {
  private String m_host = null;

  // base collection for tests
  private String m_base = null;

  // base path of servlet (for servlet actions tests)
  private String servletPath = null;

  private String m_result = null; 
  
  private String m_user;
  private String m_password;

  private IRequesterFactory m_factory = null;
  private IRequester m_requester = null;
  private IContext m_context = null;

  public FSTestSuite() {
    init();
  }

  public void setUp(boolean createFiles) throws Exception {
    if (createFiles) {
      WDHelper wd = new WDHelper(this.getRequester(), this.getHost(), this.m_base);
      wd.setup();
    }
  }

  private void init() {

    this.m_host = System.getProperty("host", "http://localhost:8080");
    this.m_base = System.getProperty("base", "/wcm/docs/test");
    this.m_result = System.getProperty("result");
    this.m_user = System.getProperty("user", "user");
    this.m_password = System.getProperty("password", "user");
    
    this.servletPath = System.getProperty("servletPath", null);
    if (this.servletPath == null && this.m_base != null) {
    	this.servletPath = RID.getRID(this.m_base).parent().parent().toString();
    }
  }

  public TestSuite suite(Class memberClass, String name) {
    if (!TestMember.class.isAssignableFrom(memberClass))
      throw new IllegalArgumentException("wrong class: " + memberClass);

    TestSuite suite = new TestSuite(name);
    //suite.addTest(new FSTestSuite("Setting up..."));

    try {
      Class[] consArgs = new Class[]{Integer.class};
      Constructor cons = memberClass.getConstructor(consArgs);
      Field field = memberClass.getField("NrOfTests");
      int maxTests = field.getInt(null);
      for (int i = 0; i < maxTests; i++) {
        Object[] args = new Object[]{new Integer(i)};
        TestMember instance = (TestMember) cons.newInstance(args);
        this.init(instance);
        suite.addTest(instance);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
    }
    return suite;
  }

  public void init(TestMember test) {
    test.init(this.getFactory(), this.getRequester(), this.getHost(), 
        this.getBaseURI(), this.servletPath, this.m_user);
  }

  public IRequester getRequester() {
    if (this.m_requester == null) {
      try {
        this.m_requester = this.getFactory().getRequester(this.getContext());
      }
      catch (Exception ex) {
        java.io.StringWriter sw = new java.io.StringWriter();
        ex.printStackTrace(new java.io.PrintWriter(sw));
        throw new RuntimeException(sw.toString());
      }
    }
    return this.m_requester;
  }

  public String getHost() {
    return this.m_host;
  }

  public String getBaseURI() {
    return this.m_base;
  }
  
  private IRequesterFactory getFactory() {
    if (this.m_factory == null) {
      try {
        this.m_factory = RequesterFactory.newInstance();
        this.m_factory.setBaseURL(this.getHost());
      }
      catch (Exception ex) {
        java.io.StringWriter sw = new java.io.StringWriter();
        ex.printStackTrace(new java.io.PrintWriter(sw));
        throw new RuntimeException(sw.toString());
      }
    }
    return this.m_factory;
  }

  private IContext getContext() {
    if (this.m_context == null) {
      this.m_context = this.m_factory.getContext("user");
      if (this.m_context.isNew()) {
        this.m_context.setUserInfo(UserInfo.valueOf(this.m_user, this.m_password));
      }
    }
    return this.m_context;
  }
  
  private String extractResult(String identifier, int nr) {
    if (this.m_result == null) 
      return null;
    String r = this.m_result;
    
    int index = r.indexOf("[" + identifier + "]");
    if (index < 1 ) return "-";
    r = r.substring(index -1);
    
    index = r.indexOf("[", 2);
    if (index > -1) {
      r = r.substring(0, index-1);
    }
    
    for (int i = 0; i < nr; i++) {
      index = r.indexOf(".", 1);
      if (index > -1) {
        r = r.substring(index);
      }
      else {
        return "-";
      }
    }
    
    index = r.indexOf(".", 1);
    if (index > -1) {
      return r.substring(0, index);
    }
    else {
      return r;
    }
  }
  
}
