package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for HEAD method.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author martin.boettcher@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: HeadTest.java,v 1.4 2005/03/02 16:32:48 sei Exp $
 */

public class HeadTest extends TestMember {

  private static final String[] desc =
    new String[] {
      "00 HEAD with improper URI",
      "01 HEAD with collection",
      "02 HEAD with unknown file",
      "03 HEAD for Worddatei.doc",
      "04 HEAD for Worddatei.doc with trailing blanks",
      "05 HEAD for Worddatei.doc, check Content-Length",
      };

  public static int NrOfTests = desc.length;

  public HeadTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest() {

    if (nr == 0)
      super.logIdentifier("HEAD");

    try {
      IResponse response;
      request.setMethod("HEAD");

      switch (nr) {

        case 0:
          request.setURI(getLocalpart());
          response = requester.perform(request);
          checkResponseCode(response, 403, 200);
          break;

        case 1:
          request.setURI(getLocalpart() + "/voll");
          response = requester.perform(request);
          checkResponseCode(response, 403, 200);
          break;

        case 2:
          request.setURI(getLocalpart() + "/abc.xyz");
          response = requester.perform(request);
          checkResponseCode(response, 404, 200);
          break;

        case 3:
          request.setURI(getLocalpart() + "/Worddatei.doc");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 4:
          String uri = getLocalpart() + "/Worddatei.doc";

          request.setURI(uri + "%20");
          response = requester.perform(request);
          checkResponseCode(response, 404, 200);

          if (response.getStatus() == 200) {
            String contLocation = response.getHeader("Content-Location");
            assertTrue("Content-Location header is missing", contLocation != null);
            assertTrue(
              "Content-Location header has wrong value: " + contLocation + ", should be: " + this.getBase() + uri,
              contLocation.equals(this.getBase() + uri));
          }

          break;

        case 5:
          request.setURI(getLocalpart() + "/Worddatei.doc");
          request.setHeader("Translate", "F");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          String cl = response.getHeader("Content-Length");
          assertTrue("Content-Length header is missing", cl != null);
          long len = Long.parseLong(cl);
          assertTrue("expected positive length, got: " + cl, len > 0);
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

}
