package com.sapportals.wcm.protocol.webdav.server.tests;

import org.w3c.dom.Element;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.dompath.DomPath;
import com.sapportals.wcm.util.http.IResponse;
import com.sapportals.wcm.util.uri.HttpUrl;
import com.sapportals.wcm.util.uri.IUri;
import com.sapportals.wcm.util.uri.UriReference;



/**
 * Various tests for SAPs internal links.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: InternalLinksTest.java,v 1.5 2005/04/08 14:33:59 mba Exp $
 */

public class InternalLinksTest extends TestMember {

  public static final String ELEMENT_internal_link = "<" + Consts.N_SAP_internal_link.getLocalName() +
      " xmlns='" + Consts.N_SAP_internal_link.getNamespaceUri() + "'/>";
  public static String ELEMENT_flex_link = "<" + Consts.N_SAP_internal_link.getLocalName() +
      " xmlns='" + Consts.N_SAP_internal_link.getNamespaceUri() + "'><" +
      Consts.N_SAP_dynamic.getLocalName() + "/></" + Consts.N_SAP_internal_link.getLocalName() + ">";

  private static final String[] desc = new String[]
  {
    "00 IL create collection",
    "01 IL MKRESOURCE internal link",
    "02 IL REPORT/expand-property internal link",
    "03 IL make container ordered",
    "04 IL MKRESOURCE another internal link",
    "05 IL PROPFIND internal link (after making parent ordered)",
    "06 IL relative link to parent collection",
    "07 IL relative link to relative link to parent collection",
    "08 IL relative link in subcollection to relative link to relative link to parent collection",
    "09 IL SEARCH collection with possibly infinite link",
    "10 IL create link loop direct",
    "11 IL create link loop by rename",
    "12 IL COPY collection with possibly infinite link",
    "13 IL DELETE collection with possibly infinite link",
    "14 IL link to external resource",
    "15 IL link to root resource",
    "16 IL link to parent of root resource",
    "17 IL link to etc repository",
    "18 IL create flex link",
    "19 IL PROPFIND flex link",
    "20 IL MOVE flex link target",
    "21 IL PROPFIND flex link",
    "22 IL DELETE flex link target",
    "23 IL HEAD flex link",
    "24 IL create internal link a pointing to a/a, then delete it",
    "25 IL MOVE collection with possibly infinite link",
    "   IL cleanup",
  };

  public static int NrOfTests = desc.length;

  public InternalLinksTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  private static boolean supportsInternalLinks = true;
  private static boolean supportsFlexLinks = false;

  protected void runTest() {
    if (nr == 0) super.logIdentifier("IL");

    String respfile = "INTERNALLINK-" + TestMember.THREEDIGITS.format(nr) + ".txt";
    String container =  "/ilcontainer";
    String container2 = "/ilcontainer2";

    try {
      IResponse response;
      clear(request);

      switch (nr) {
        case 0:
          request.setMethod("MKCOL");
          setRelativeURI(container);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 1:
          request.setMethod("PUT");
          setRelativeURI(container + "/target");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          request.clear();
          request.setMethod("MKRESOURCE");
          setRelativeURI(container + "/il");
          request.setBody("<propertyupdate xmlns='DAV:'>" +
                          "<set>" +
                          "<prop>" +
                          // nit: add whitespace below resourcetype to test proper request parsing
                          "<resourcetype> " + ELEMENT_internal_link + "</resourcetype>" +
                          "<reftarget><href>" + getLocalpart()+ container + "/target" + "</href></reftarget>" +
                          "</prop>" +
                          "</set>" +
                          "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 201, 501);
          if (response.getStatus() == 501) {
            supportsInternalLinks = false;
          }
          super.logComment("internal-links:" + supportsInternalLinks);
          
          if (supportsInternalLinks) {
            Element etag = getProperty(getLocalpart() + container + "/il", Consts.N_getetag);
            assertTrue("DAV:getetag should not be present on internal link: " + etag, etag == null);
          }
          
          
          break;

        case 2:
          if (supportsInternalLinks) {
            request.setMethod("REPORT");
            setRelativeURI(container + "/il");
            request.setHeader("Depth", "0");
            request.setHeader("Apply-To-Redirect-Ref", "T");
            super.checkMultistatus("<expand-property xmlns='DAV:'><property name='reftarget'><property name='resourcetype'/></property></expand-property>",
                                   respfile);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/il");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            assertTrue(response.getHeader("Location") != null);
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;

        case 3:
          if (supportsInternalLinks) {
            request.setMethod("ORDERPATCH");
            setRelativeURI(container);
            request.setBody("<orderpatch xmlns='DAV:'>" +
                            "<ordering-type>" +
                            "<href>mailto:julian.reschke@greenbytes.de?subject=ordering tests</href>" +
                            "</ordering-type>" +
                            "</orderpatch>");
            response = requester.perform(request);
            checkResponseCode(response, 200);
          }
          break;

        case 4:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            request.setHeader("Position", "last");
            setRelativeURI(container + "/il2");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>" + getLocalpart() + container + "/target" + "</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 5:
          if (supportsInternalLinks) {

            request.setMethod("PROPFIND");
            setRelativeURI(container + "/il2");
            request.setHeader("Depth", "0");
            request.setHeader("Apply-to-redirect-ref", "T");
            super.checkMultistatus("<propfind xmlns='DAV:'><prop><reftarget /></prop></propfind>",
                                   respfile);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/il2");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            String location = response.getHeader("Location");
            HttpUrl base = m_factory.getBase();
            IUri target = base.resolve(new UriReference(getLocalpart() + container + "/target", null, null));
            assertTrue("wrong location: " + location + " expected " + target,
                       this.sameURI(location, target.toString()));
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;


        case 6:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            request.setHeader("Position", "last");
            setRelativeURI(container + "/il3");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>..</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            {
              clear(request);
              request.setMethod("PROPFIND");
              setRelativeURI(container + "/il3");
              request.setHeader("Depth", "0");
              response = requester.perform(request);
              checkResponseCode(response, 302);
              String location = response.getHeader("Location");

              HttpUrl base = m_factory.getBase();
              IUri target = base.resolve(new UriReference(getLocalpart() + "/", null, null));
              assertTrue("wrong location: " + location + " expected " + target,
                         this.sameURI(location, target.toString()));
              assertTrue(response.getHeader("Redirect-Ref") != null);
            }
            
            // same with trailing / in request URI
            {
              clear(request);
              request.setMethod("PROPFIND");
              setRelativeURI(container + "/il3/");
              request.setHeader("Depth", "0");
              response = requester.perform(request);
              checkResponseCode(response, 302);
              String location = response.getHeader("Location");

              HttpUrl base = m_factory.getBase();
              IUri target = base.resolve(new UriReference(getLocalpart() + "/", null, null));
              assertTrue("wrong location: " + location + " expected " + target,
                         this.sameURI(location, target.toString()));
              assertTrue(response.getHeader("Redirect-Ref") != null);
            }
          }
          break;

        case 7:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            request.setHeader("Position", "last");
            setRelativeURI(container + "/il4");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>./il3</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/il4");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            String location = response.getHeader("Location");
            HttpUrl base = m_factory.getBase();
            IUri target = base.resolve(new UriReference(getLocalpart() + container + "/il3", null, null));
            assertTrue("wrong location: " + location + " expected " + target,
                       this.sameURI(location, target.toString()));
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;

        case 8:
          if (supportsInternalLinks) {
            request.setMethod("MKCOL");
            setRelativeURI(container + "/collection");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/collection/il5");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>../il4</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/collection/il5");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            String location = response.getHeader("Location");
            HttpUrl base = m_factory.getBase();
            IUri target = base.resolve(new UriReference(getLocalpart() + container + "/il4", null, null));
            assertTrue("wrong location: " + location + " expected " + target,
                       this.sameURI(location, target.toString()));
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;

        case 9:
          if (supportsInternalLinks) {
            request.setMethod("SEARCH");
            setRelativeURI(container + "/collection/il5");
            checkMultistatus(
                "<searchrequest xmlns='DAV:'>"
                + "  <basicsearch><select><prop><resourcetype/></prop></select>"
                + "    <from><scope><href>" + getLocalpart() + container + "</href>"
                + "          <depth>infinity</depth></scope>"
                + "    </from>"
                + "  </basicsearch>"
                + "</searchrequest>", respfile);
//            response = requester.perform (request);
//            checkResponseCode (response, 207);
          }
          break;

        case 10:
          if (supportsInternalLinks) {
            request.setMethod("DELETE");
            setRelativeURI(container + "/il4");
            request.setHeader("Apply-to-redirect-ref", "T");
            response = requester.perform(request);
            checkResponseCode(response, 204);

            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/il4");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>./collection/il5</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 403, 201);
          }
          break;

        case 11:
          if (supportsInternalLinks) {
            request.setMethod("DELETE");
            setRelativeURI(container + "/il4");
            request.setHeader("Apply-to-redirect-ref", "T");
            response = requester.perform(request);
            checkResponseCode(response, 204, 404);

            clear(request);
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/il4");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>./collection/il6</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            clear(request);
            request.setMethod("MOVE");
            request.setHeader("Apply-to-redirect-ref", "T");
            setRelativeURI(container + "/collection/il5");
            request.setHeader("Destination", this.getBase() + getLocalpart() + container + "/collection/il6");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/collection/il6");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            String location = response.getHeader("Location");
            HttpUrl base = m_factory.getBase();
            IUri target = base.resolve(new UriReference(getLocalpart() + container + "/il4", null, null));
            assertTrue("wrong location: " + location + " expected " + target,
                       this.sameURI(location, target.toString()));
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;

        case 12:
          if (supportsInternalLinks) {
            DomPath locations = new DomPath("//D:location", DomPath.DEFAULT_NAMESPACE_BINDINGS);

            clear(request);
            request.setMethod("COPY");
            setRelativeURI(container);
            request.setHeader("Destination", this.getBaseAndLocalpart() + container2);
            response = requester.perform(request);
            checkResponseCode(response, 207);
            checkResponseDocument(response, respfile, locations);

            clear(request);
            request.setMethod("COPY");
            setRelativeURI(container);
            request.setHeader("Apply-To-Redirect-Ref", "T");
            request.setHeader("Destination", this.getBaseAndLocalpart() + container2);
            response = requester.perform(request);
            checkResponseCode(response, 204);
          }
          break;

        case 13:
          if (supportsInternalLinks) {
            request.setMethod("DELETE");
            setRelativeURI(container2);
            response = requester.perform(request);
            checkResponseCode(response, 204, 404);
          }
          break;

        case 14:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/ilnowork");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>http://greenbytes.de</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 502);
          }
          break;

        case 15:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/ilroot");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>../../..</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 16:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/ilnowork2");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>../../../..</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 502);
          }
          break;

        case 17:
          if (supportsInternalLinks) {
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/ilnowork3");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>../../../etc</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 18:
          if (supportsInternalLinks) {
            request.setMethod("PUT");
            setRelativeURI(container + "/flexlinktarget");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            request.clear();
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/flexlink");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_flex_link + "</resourcetype>" +
                            "<reftarget><href>" + getLocalpart() + container + "/flexlinktarget" + "</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201, 501);
            if (response.getStatus() == 201) {
              supportsFlexLinks = true;
            }
            super.logComment("flexlinks:" + supportsFlexLinks);
          }
          break;

        case 19:
          if (supportsFlexLinks) {
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/flexlink");
            request.setHeader("Depth", "0");
            request.setHeader("Apply-To-Redirect-Ref", "T");
            super.checkMultistatus("<propfind xmlns='DAV:'><prop><reftarget /><resourcetype /></prop></propfind>",
                                   respfile);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/flexlink");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            assertTrue(response.getHeader("Location").endsWith("/flexlinktarget"));
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;

        case 20:
          if (supportsFlexLinks) {
            request.setMethod("MOVE");
            setRelativeURI(container + "/flexlinktarget");
            request.setHeader("Destination", this.getBase() + getLocalpart() + container + "/newflexlinktarget");
            response = requester.perform(request);
            checkResponseCode(response, 201);
          }
          break;

        case 21:
          if (supportsFlexLinks) {
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/flexlink");
            request.setHeader("Depth", "0");
            request.setHeader("Apply-To-Redirect-Ref", "T");
            super.checkMultistatus("<propfind xmlns='DAV:'><prop><reftarget /></prop></propfind>",
                                   respfile);

            clear(request);
            request.setMethod("PROPFIND");
            setRelativeURI(container + "/flexlink");
            request.setHeader("Depth", "0");
            response = requester.perform(request);
            checkResponseCode(response, 302);
            assertTrue(response.getHeader("Location").endsWith("/newflexlinktarget"));
            assertTrue(response.getHeader("Redirect-Ref") != null);
          }
          break;

        case 22:
          if (supportsFlexLinks) {
            request.setMethod("DELETE");
            setRelativeURI(container + "/newflexlinktarget");
            response = requester.perform(request);
            checkResponseCode(response, 204);
          }
          break;

        case 23:
          if (supportsFlexLinks) {
            request.setMethod("HEAD");
            setRelativeURI(container + "/flexlink");
            response = requester.perform(request);
            checkResponseCode(response, 404);
          }
          break;

        case 24:
          if (supportsInternalLinks) {
            clear(request);
            request.setMethod("MKRESOURCE");
            setRelativeURI(container + "/selfchildlink");
            request.setBody("<propertyupdate xmlns='DAV:'>" +
                            "<set>" +
                            "<prop>" +
                            "<resourcetype>" + ELEMENT_internal_link + "</resourcetype>" +
                            "<reftarget><href>" + getLocalpart() + container + "/selfchildlink/x" + "</href></reftarget>" +
                            "</prop>" +
                            "</set>" +
                            "</propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 201);

            clear(request);
            request.setMethod("DELETE");
            setRelativeURI(container + "/selfchildlink");
            response = requester.perform(request);
            checkResponseCode(response, 302);

            clear(request);
            request.setMethod("DELETE");
            setRelativeURI(container + "/selfchildlink");
            request.setHeader("Apply-To-Redirect-Ref", "T");
            response = requester.perform(request);
            checkResponseCode(response, 204);
          }
          break;

        case 25:
          if (supportsInternalLinks) {
            clear(request);
            request.setMethod("MOVE");
            setRelativeURI(container);
            request.setHeader("Destination", getBaseAndLocalpart() + container2);
            response = requester.perform(request);
            checkResponseCode(response, 201);

            clear(request);
            request.setMethod("DELETE");
            setRelativeURI(container2);
            response = requester.perform(request);
            checkResponseCode(response, 204, 404);
          }
          break;

        default:
          request.setMethod("DELETE");
          setRelativeURI(container);
          response = requester.perform(request);
          checkResponseCode(response, 204, 404);
          break;
      }

    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
