package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.protocol.webdav.DavConditions;
import com.sapportals.wcm.util.http.IResponse;
import com.sapportals.wcm.util.uri.URICodec;

/**
 * Various tests for MKCOL method.
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author martin.boettcher@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: MkColTest.java,v 1.3 2004/03/29 11:52:07 jre Exp $
 */

public class MkColTest extends TestMember {
  
  private static final String[] desc = new String[]{
    "00 MC MKCOL with improper URI",
    "01 MC MKCOL test",
    "02 MC MKCOL test/two",
    "03 MC MKCOL anothertest",
    "04 MC MKCOL test 2",
    "05 MC MKCOL with umlaute",
    "06 MC MKCOL with Euro sign",
    "07 MC MKCOL with single quote",
    "08 MC MKCOL with character that is not supported by some versions of java.io.File",
    "   MC MKCOL cleanup",
  };

  public static int NrOfTests = desc.length;

  public MkColTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest() {
    if (nr == 0) super.logIdentifier("MC");

    try {
      IResponse response;
      clear(request);

      switch (nr) {
        case 0:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "");
          response = requester.perform(request);
          checkResponseCode(response, 405);
          break;

        case 1:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/test");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 2:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/test/two");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 3:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/anothertest");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 4:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/test");
          response = requester.perform(request);
          checkResponseCode(response, 405);
          break;

        case 5:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + URICodec.Encode("/anothertest/\u00c4\u00d6\u00dc\u00e4\u00f6\u00fc-\u00df-folder"));
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + URICodec.Encode("/anothertest/\u00c4\u00d6\u00dc\u00e4\u00f6\u00fc-\u00df-folder"));
          response = requester.perform(request);
          checkResponseCode(response, 204);
          break;

        case 6:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + URICodec.Encode("/anothertest/Euro-\u20ac-folder"));
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + URICodec.Encode("/anothertest/Euro-\u20ac-folder"));
          response = requester.perform(request);
          checkResponseCode(response, 204);
          break;

        case 7:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + URICodec.Encode("/anothertest/Stefan's folder"));
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + URICodec.Encode("/anothertest/Stefan's folder"));
          response = requester.perform(request);
          checkResponseCode(response, 204);
          break;

        case 8: {
            String dn = URICodec.Encode("/anothertest/aleph-\ufb30");

            clear(request);
            request.setMethod("MKCOL");
            request.setURI(getLocalpart() + dn);
            response = requester.perform(request);
            checkResponseCode(response, 201, 403);

            if (response.getStatus() == 403) {
              checkErrorElement(response, DavConditions.N_name_allowed);
            }
            else {
              clear(request);
              request.setMethod("HEAD");
              request.setURI(getLocalpart() + dn);
              response = requester.perform(request);
              checkResponseCode(response, 200);

              clear(request);
              request.setMethod("DELETE");
              request.setURI(getLocalpart() + dn);
              response = requester.perform(request);
              checkResponseCode(response, 204);
            }
          }
          break;

        default:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + "/test");
          request.setHeader("Depth", "Infinity");
          response = requester.perform(request);
          checkResponseCode(response, 200, 204);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + "/anothertest");
          request.setHeader("Depth", "Infinity");
          response = requester.perform(request);
          checkResponseCode(response, 200, 204);
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
