package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for OPTIONS method.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author julian.reschke@greenbytes.de
 * @version $Id: OptionsTest.java,v 1.3 2004/03/29 11:52:07 jre Exp $
 */

public class OptionsTest extends TestMember {

  private static final String[] desc=new String[]
  {
    "00 Options",
    "01 Options on null-resource, with parent",
    "02 Options on null-resource, without parent",
    "03 unknown method name",
    "04 OPTIONS on repository root",
  };

  public static int NrOfTests = desc.length;

  public OptionsTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("OP");

    try
    {
      IResponse response;
      clear(request);
      request.setMethod("OPTIONS");

      switch (nr) {
        case 0:
          request.setURI(getLocalpart());
          response = requester.perform(request);
          checkResponseCode(response, 200);

          String msAuthorVia = response.getHeader("MS-Author-Via");
          assertTrue(msAuthorVia != null);
          assertTrue(msAuthorVia.indexOf("DAV") >= 0);
          break;

        case 1:
          request.setURI(getLocalpart() + "/doesnotexist");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 2:
          request.setURI(getLocalpart() + "/doesnotexist2/another");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 3:
          request.setMethod("PROPFOUND");
          request.setURI(getLocalpart() + "/");
          response = requester.perform(request);
          checkResponseCode(response, 501);
          break;

        case 4: {
          int index = getLocalpart().lastIndexOf('/');
          assertTrue("strange uri: " + getLocalpart(),  index > 0);
          String repositoryRoot = getLocalpart().substring(0, index);
          request.setURI(repositoryRoot);
          response = requester.perform(request);
          checkResponseCode(response, 200);
          String allow = response.getHeader("Allow");
          assertTrue("DELETE should not be supported: "+allow,
                     allow != null && allow.indexOf("DELETE") < 0);
          break;
        }

      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }

  }

}