package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for POST method.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author martin.boettcher@greenbytes.de
 * @version $Id: PostTest.java,v 1.3 2004/03/29 11:52:07 jre Exp $
 */

public class PostTest extends TestMember {
  private static final String[] desc =
    new String[] {
      "00 POST with improper URI",
      "01 POST with collection",
      "02 POST with unknown file",
      "03 POST for Worddatei.doc",
      };

  public static int NrOfTests = desc.length;

  public PostTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest() {
    if (nr == 0)
      super.logIdentifier("POST");

    try {
      IResponse response;
      clear(request);
      request.setMethod("POST");
      
      // @TODO: don't accept 400 starting with 60SP2

      switch (nr) {
        case 0 :
          request.setURI(getLocalpart());
          response = requester.perform(request);
          checkResponseCode(response, 400, 415);
          break;

        case 1 :
          request.setURI(getLocalpart() + "/voll");
          response = requester.perform(request);
          checkResponseCode(response, 400, 415);
          break;

        case 2 :
          request.setURI(getLocalpart() + "/abc.xyz");
          response = requester.perform(request);
          checkResponseCode(response, 400, 404, 415);
          break;

        case 3 :
          request.setURI(getLocalpart() + "/Worddatei.doc");
          response = requester.perform(request);
          checkResponseCode(response, 400, 415);
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
