package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.Map;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.sapportals.wcm.IWcmConst;
import com.sapportals.wcm.protocol.webdav.*;
import com.sapportals.wcm.protocol.webdav.dompath.DomPath;
import com.sapportals.wcm.repository.PropertyName;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for PROPPATCH method.
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author martin.boettcher@greenbytes.de
 * @author julian.reschke@greenbytes.de
 * @version $Id: ProppatchTest.java,v 1.6 2005/04/01 13:16:53 sei Exp $
 */

public class ProppatchTest extends TestMember
{
  private static DomPath DP_prop_nons = null;

  static {
    DP_prop_nons = new DomPath ("/D:multistatus/D:response/D:propstat/D:prop/no-ns",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
  }

  private static final String[] desc = new String[] {
    "00 PP (unused)",
    "01 PP PROPPATCH DAV:getcontentlength",
    "02 PP PROPPATCH DAV:getcontentlength und xyz:...",
    "03 PP PROPPATCH set a new one",
    "04 PP PROPFIND the new one",
    "05 PP PROPPATCH set system property with wrong datatype",
    "06 PP PROPPATCH set with xyz:whoAmI",
    "07 PP PROPPATCH set with DAV:whoAmI",
    "08 PP PROPPATCH set three new",
    "09 PP PROPFIND the new props",
    "10 PP PROPPATCH invalid xml",
    "11 PP PROPFIND the new props (2)",
    "12 PP PROPPATCH remove one",
    "13 PP PROPFIND the new props (3)",
    "14 PP PROPPATCH remove internal and added",
    "15 PP PROPPATCH remove last added",
    "16 PP PROPFIND the new props (4)",
    "17 PP PROPPATCH remove DAV:getcontentlength",
    "18 PP PROPPATCH remove xyz:getcontentlength",
    "19 PP PROPPATCH remove xyz:getcontentlength (2)",
    "20 PP PROPPATCH textfile",
    "21 PP PROPFIND patched textfile",
    "22 PP PROPPATCH with inherited xml:lang",
    "23 PP PROPFIND with inherited xml:lang",
    "24 PP PROPPATCH with inherited xml:base",
    "25 PP PROPFIND with inherited xml:base",
    "26 PP PROPPATCH with xsd:boolean",
    "27 PP PROPFIND with xsd:boolean",
    "28 PP PROPPATCH with xsd:int",
    "29 PP PROPFIND with xsd:int",
    "30 PP PROPPATCH with xsd:long",
    "31 PP PROPFIND with xsd:long",
    "32 PP PROPPATCH with xsd:date",
    "33 PP PROPFIND with xsd:date",
    "34 PP PROPPATCH with unparseable xsd:long",
    "35 PP PROPPATCH with multiple xsd:long values",
    "36 PP PROPFIND with multiple xsd:long values",
    "37 PP PROPPATCH with multiple xsd:dateTime values",
    "38 PP PROPFIND with multiple xsd:dateTime values",
    "39 PP PROPPATCH with property name character outside ASCII",
    "40 PP PROPFIND with property name character outside ASCII",
    "41 PP PROPPATCH with unsupported property type",
    "42 PP PROPPATCH with empty multivalued property",
    "43 PP PROPFIND with empty multivalued property",
    "44 PP PROPPATCH with multiple SAP-xmlmarkup values",
    "45 PP PROPFIND with multiple SAP-xmlmarkup values",
    "46 PP PROPPATCH with property in no namespace",
    "47 PP PROPFIND with property in no namespace",
    "48 PP cleanup of property in no namespace",
    "49 PP PROPPATCH (resource) with umlaute in property value",
    "50 PP PROPFIND (resource) with umlaute in property value",
    "51 PP PROPPATCH (collection) with umlaute in property value",
    "52 PP PROPFIND (collection) with umlaute in property value",
    "53 PP PROPPATCH (collection) with empty namespace",
    "54 PP PROPFIND (collection) with empty namespace",
    "55 PP PROPPATCH case-sensitivity test",
    "56 PP PROPFIND case-sensitivity test",
    "57 PP PROPPATCH with unresolvable datatype",
    "58 PP PROPFIND after patch with unresolvable datatype",
    "59 PP PROPPATCH with unresolvable datatype (multivalued)",
    "60 PP PROPFIND after patch with unresolvable datatype (multivalued)",
    "61 PP PROPPATCH with unparseable xsd:boolean",
    "62 PP sap:description",
    "63 PP test DAV:displayname functionality",
    "64 PP PROPPATCH a dead property with DAV:href content",
    "65 PP REPORT expand-property",
    "66 PP PROPPATCH DAV:ishidden",
    "67 PP PROPPATCH dead property with high-end Unicode character value",
    "68 PP PROPPATCH/FIND with flags",
    "69 PP PROPPATCH large propertyname",
    "70 PP PROPPATCH large property value for sap:description",
    "71 PP invalid request",
    "72 PP PROPFIND by prop incl dead-props",
    "73 PP PROPPATCH with property value ending with whitespace",
    "74 PP PROPPATCH with multiple SAP-expanded-name values",
    "75 PP PROPFIND with multiple SAP-expanded-name values",
    "76 PP PROPPATCH with single SAP-expanded-name value",
    "77 PP PROPFIND with single SAP-expanded-name value",
    "78 PP PROPPATCH with single SAP-mapped-rid value",
    "79 PP PROPFIND with single SAP-mapped-rid value",
    "80 PP PROPPATCH with single SAP-mapped-rid-list value",
    "81 PP PROPFIND with single SAP-mapped-rid-list value",
    "82 PP PROPPATCH with attribute",
    "83 PP PROPPATCH with empty attribute value",	
    "84 PP PROPPATCH with empty sap:description",
  };

  public static int NrOfTests = desc.length;
  
  public ProppatchTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  private String propertyupdate(String s) {
    return "<propertyupdate xmlns='DAV:'>" + s + "</propertyupdate>";
  }

  private String remove(String s) {
    return "<remove xmlns='DAV:'>" + s + "</remove>";
  }

  private String setWithBase(String s) {
    return "<set xmlns='DAV:' xml:base='http://greenbytes.de'>" + s + "</set>";
  }

  private String setWithLang(String s) {
    return "<set xmlns='DAV:' xml:lang='fr'>" + s + "</set>";
  }

  private String set(String s) {
    return "<set xmlns='DAV:'>" + s + "</set>";
  }

  private String prop(String s) {
    return "<prop xmlns='DAV:'>" + s + "</prop>";
  }


  private static boolean supportsTypes = false;
  private static boolean supportsAttributes = false;

  protected void runTest() {
    
    if (nr == 0) super.logIdentifier("PP");

    String respfile = "PROPPATCH-" + TestMember.THREEDIGITS.format(nr) + ".txt";

    final String TEST_UND_TEST2 = "<propfind xmlns='DAV:'><prop><test "
      + "xmlns='xyz:'/><test2 xmlns='xyz:'/><foo xmlns='bar'/></prop></propfind>";

    try {
      
      request.setMethod("PROPPATCH");
      request.setURI(getLocalpart() + "/Worddatei.doc");
      request.removeHeader("Depth");
      IResponse response;

      switch (nr)
      {
        case 0:
          break;

        case 1:
          super.checkMultistatus(propertyupdate(set(prop("<getcontentlength>Not allowed</getcontentlength>"))), respfile);
          break;

        case 2:
          super.checkMultistatus (propertyupdate (set (prop (
            "<getcontentlength>Not allowed</getcontentlength><darfnichtsein xmlns='xyz:'>This is not allowed</darfnichtsein>"))),
            respfile);
          break;

        case 3:
          super.checkMultistatus (propertyupdate (set (prop ("<test xmlns='xyz:'>Dies ist der erste Test</test>"))),
            respfile);
          break;

        case 4:
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus("<propfind xmlns='DAV:'><prop><test xmlns='xyz:'/></prop></propfind>", respfile);
          break;

        case 5:
          {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPPATCH");
            request.setBody(propertyupdate (set (prop
              ("<hidden xmlns='http://sapportals.com/xmlns/cm'>foobar</hidden>"))));
            response = requester.perform(request);
            checkResponseCode(response, 207);

            // jre: will succeed with r-moddav and r-iis. Now what?
            //DomPath DP_ok_props = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/*", DomPath.DEFAULT_NAMESPACE_BINDINGS);
            //boolean isok = DP_ok_props.match(response.getDocument()).size() > 0;
            //assertTrue(! isok);
          }
          break;

        case 6:
          super.checkMultistatus (propertyupdate (set (prop
            ("<test xmlns='xyz:'>Dies ist der erste Test</test>")) + "<whoAmi  xmlns='xyz:'/>"),
            "PROPPATCH-003.txt");
          break;

        case 7:
          super.checkMultistatus (propertyupdate (set (prop
            ("<test xmlns='xyz:'>Dies ist der erste erneut modifizierte Test</test>")) + "<whoAmi/>"),
            "PROPPATCH-003.txt");
          break;

        case 8:
          super.checkMultistatus (propertyupdate (set (prop (
            "<foo xmlns='bar' xml:lang='de'>Versuch</foo>" +
            "<test2 xmlns='xyz:'>" +
            "<name>Boettcher<abc:name xmlns:abc='abc:' ZZZ:dasAttribut='AttributWert' xmlns:ZZZ='def:'>" +
            "Martin</abc:name></name>Dies ist der dritte Test!" +
            "</test2>" ))), respfile);
          break;

        case 9:
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus(TEST_UND_TEST2, respfile);
          break;

        case 10:
          request.setBody(propertyupdate(set(prop(
            "<test2 xmlns='xyz:'>" +
            "<name>Boettcher<name xmlns='abc:' def:dasAttribut='AttributWert'>Willi</name>" +
            "</name>Dies ist der vierte Test!" +
            "</test2>"))));
          response = requester.perform(request);
          checkStatusAndErrorElement(response, 400, DavConditions.N_SAP_request_body_wellformed);
          break;

        case 11:
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus(TEST_UND_TEST2, "PROPPATCH-009.txt");
          break;

        case 12:
          super.checkMultistatus (propertyupdate (remove (prop ("<test xmlns='xyz:'/>"))),
            respfile);
          break;

        case 13:
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus(TEST_UND_TEST2, respfile);
          break;

        case 14:
          super.checkMultistatus (propertyupdate (remove (prop ("<test2 xmlns='xyz:'/><getcontentlength/>"))),
            respfile);
          break;

        case 15:
          super.checkMultistatus (propertyupdate (remove (prop ("<test2 xmlns='xyz:'/>"))),
            respfile);
          break;

        case 16:
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus(TEST_UND_TEST2, respfile);
          break;

        case 17:
          super.checkMultistatus (propertyupdate (remove (prop ("<getcontentlength/>"))),
            respfile);
          break;

        case 18:
          request.setMethod("PROPPATCH");
          request.setBody (propertyupdate (set (prop (
            "<getcontentlength xmlns='xyz:'>123</getcontentlength>"))));
          response = requester.perform( request );
          assertTrue (response.getStatus()==207);

          // fall through

        case 19:
          // first should be ok, second one should fail
          super.checkMultistatus (propertyupdate (remove (prop ("<getcontentlength xmlns='xyz:'/>"))),
            respfile);
          break;

        case 20:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (set (prop ("<test xmlns='xyz:'>Dies ist der erste Test</test>"))),
            respfile);
          break;

        case 21:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus("<propfind xmlns='DAV:'><prop><test xmlns='xyz:'/></prop></propfind>", respfile);
          break;

        case 22:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (setWithLang (prop ("<julian xmlns='withlang'>Versuch</julian>"))),
            respfile);
          break;

        case 23:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='withlang'/></prop></propfind>",
            respfile);
          break;

        case 24:
          // overwrite "withlang" property without lang, set xml:base
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (set (prop("<julian xmlns='withlang'>lang removed</julian>")) + setWithBase (prop ("<julian xmlns='withbase'>Versuch 2</julian>"))),
            respfile);
          break;

        case 25:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='withlang'/><julian xmlns='withbase'/></prop></propfind>",
            respfile);
          break;

        case 26:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns='testboolean' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xsi:type='xsd:boolean'>true</julian>"))),
            respfile);
          supportsTypes = true;
          super.logComment("typed-properties:yes");
          break;

        case 27:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testboolean'/></prop></propfind>",
              respfile);
          }
          break;

        case 28:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns='testint' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xsi:type='xsd:int'>012345</julian>"))),
              respfile);
          }
          break;

        case 29:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testint'/></prop></propfind>",
              respfile);
          }
          break;

        case 30:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns='testlong' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xs='http://www.w3.org/2001/XMLSchema' xsi:type='xs:long'>-0987654321987654</julian>"))),
              respfile);
          }
          break;

        case 31:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testlong'/></prop></propfind>",
              respfile);
          }
          break;

        case 32:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns='testdate' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xsi:type='xsd:dateTime'>2001-07-28T10:00:00+01:00</julian>"))),
              respfile);
          }
          break;

        case 33:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testdate'/></prop></propfind>",
              respfile);
          }
          break;

        case 34:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns='testlongfail' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xs='http://www.w3.org/2001/XMLSchema' xsi:type='xs:long'>x-0987654321987654</julian>"))),
              respfile);
          }
          break;

        case 35:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xsi:type='soap-enc:Array' xmlns:soap-enc='http://schemas.xmlsoap.org/soap/encoding/' soap-enc:arrayType='xsd:long[2]' xmlns='testlongs'><xsd:long>-42</xsd:long><xsd:long>17</xsd:long></julian>"))),
              respfile);
          }
          break;

        case 36:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testlongs'/></prop></propfind>",
              respfile);
          }
          break;

        case 37:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xmlns:soap-enc='http://schemas.xmlsoap.org/soap/encoding/' soap-enc:arrayType='xsd:dateTime[3]' xsi:type='soap-enc:Array' xmlns='testdates'> <!-- intentional whitespace for test--><dateTime>2001-01-01T12:34:56Z</dateTime><xsd:dateTime>1965-12-23T00:00:00Z</xsd:dateTime><xsd:dateTime>2001-07-01T12:34:56Z</xsd:dateTime></julian>"))),
              respfile);
          }
          break;

        case 38:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testdates'/></prop></propfind>",
              respfile);
          }
          break;

        case 39:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (set (prop ("<julian-\u00df xmlns='charset'></julian-\u00df>"))),
            respfile);
          break;

        case 40:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian-\u00df xmlns='charset'/></prop></propfind>",
            respfile);
          break;

        case 41:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus ((propertyupdate (set (prop ("<julian xmlns='testunknown' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:myxsd='mailto:reschke@muenster.de' xsi:type='myxsd:unknown'>true</julian>")))),
              respfile);
          }
          break;

        case 42:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xmlns:soap-enc='http://schemas.xmlsoap.org/soap/encoding/' soap-enc:arrayType='xsd:dateTime[0]' xsi:type='soap-enc:Array' xmlns='testdates-empty'></julian>"))),
              respfile);
          }
          break;

        case 43:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testdates-empty'/></prop></propfind>",
              respfile);
          }
          break;

        case 44:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:sd='"+Consts.NS_SAP_DATATYPES+"' xmlns:soap-enc='http://schemas.xmlsoap.org/soap/encoding/' soap-enc:arrayType='sd:markup[2]' xsi:type='soap-enc:Array' xmlns='testmarkups'><sd:markup><green attr='foobar'/>bytes</sd:markup><sd:markup>FOOBAR</sd:markup></julian>"))),
              respfile);
          }
          break;

        case 45:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testmarkups'/></prop></propfind>",
              respfile);
          }
          break;

        case 46:
          request.setMethod("PUT");
          request.setURI (getLocalpart() + "/no-ns.TXT");
          response = requester.perform( request );
          checkResponseCode(response, 201);

          request.clear ();
          request.setURI (getLocalpart() + "/no-ns.TXT");
          request.setMethod ("PROPPATCH");
          request.setBody (propertyupdate (set (prop ("<no-ns xmlns=''>Versuch</no-ns>"))));
          request.expectsResponseDocument ();
          response = requester.perform (request);
          checkResponseCode (response, 207);
          break;

        case 47:
          request.setURI(getLocalpart() + "/no-ns.TXT");
          request.setMethod ("PROPFIND");
          request.setHeader("Depth", "0");
          request.setBody ("<propfind xmlns='DAV:'><prop><no-ns xmlns=''/></prop></propfind>");
          request.expectsResponseDocument();
          response = requester.perform (request);
          checkResponseCode (response, 207);
          Element prop = (Element) DP_prop_nons.matchOrDie (response.getDocument(),
            1, 1, "property no-ns not found").get(0);
          assertTrue ("invalid content: " + DomElement.getText(prop),
            "Versuch".equals (DomElement.getText(prop)));
          break;

        case 48:
          request.setMethod("DELETE");
          request.setURI (getLocalpart() + "/no-ns.TXT");
          response = requester.perform( request );
          checkResponseCode(response, 204);
          break;

        case 49:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (set (prop ("<stefan xmlns='http://greenbytes.de/test'>D\u00f6ser Test ben\u00fctzt \u00e4inige \u00dcml\u00e4\u00f6ute mit \u00df.</stefan>"))),
            respfile);
          break;

        case 50:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><stefan xmlns='http://greenbytes.de/test'/></prop></propfind>",
            respfile);
          break;

        case 51:
          request.setURI(getLocalpart() + "/voll");
          super.checkMultistatus (propertyupdate (set (prop ("<stefan xmlns='http://greenbytes.de/test'>D\u00f6ser Test ben\u00fctzt \u00e4inige \u00dcml\u00e4\u00f6ute mit \u00df.</stefan>"))),
            respfile);
          break;

        case 52:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/voll");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><stefan xmlns='http://greenbytes.de/test'/></prop></propfind>",
            respfile);
          break;

        case 53:
          request.setURI(getLocalpart() + "/voll");
          super.checkMultistatus (propertyupdate (set (prop ("<stefan xmlns=''>test</stefan>"))),
            respfile);
          break;

        case 54:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/voll");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><stefan xmlns=''/></prop></propfind>",
            respfile);
          break;

        case 55:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus (propertyupdate (set (prop ("<a xmlns='cs-test'>lowercase property name</a>") +
            prop("<A xmlns='cs-test'>uppercase property name</A>") +
            prop("<a xmlns='CS-TEST'>uppercase property namespace</a>"))),
            respfile);
          break;

        case 56:
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setHeader("Depth", "0");
          super.checkMultistatus ("<propfind xmlns='DAV:'><prop><a xmlns='cs-test'/><A xmlns='cs-test'/><a xmlns='CS-TEST'/></prop></propfind>",
            respfile);
          break;

        case 57:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus ((propertyupdate (set (prop ("<julian xmlns='unresolvable-datatype' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:type='myxsd:unknown'>true</julian>")))),
              respfile);
          }
          break;

        case 58:
          if (supportsTypes) {
            request.setMethod("PROPFIND");
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='unresolvable-datatype'/></prop></propfind>",
              respfile);
          }
          break;

        case 59:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus ((propertyupdate (set (prop ("<julian xmlns='unresolvable-datatype-mv' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:soap-enc='http://schemas.xmlsoap.org/soap/encoding/' soap-enc:arrayType='myxsd:foo[0]' xsi:type='soap-enc:Array'><x>true</x></julian>")))),
                          respfile);
          }
          break;

        case 60:
          if (supportsTypes) {
            request.setMethod("PROPFIND");
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='unresolvable-datatype-mv'/></prop></propfind>",
              respfile);
          }
          break;

        case 61:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns='testboolfail' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xs='http://www.w3.org/2001/XMLSchema' xsi:type='xs:boolean'></julian>"))),
              respfile);
          }
          break;

        case 62:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus ((propertyupdate (set (prop ("<sap:description xmlns:sap='http://sapportals.com/xmlns/cm'>this is a small description</sap:description>")))),
            respfile);
          break;

        case 63:
          {
            Element displayname = super.getProperty(getLocalpart() + "/UTF8.TXT", Consts.N_displayname);
            if (displayname != null) {
              super.logComment("displayname:fixed");
            }
            else {
              clear(request);
              request.setMethod("PROPPATCH");
              request.setURI(getLocalpart() + "/UTF8.TXT");
              super.checkMultistatus ((propertyupdate (set (prop ("<displayname>Something else</displayname>")))),
                respfile);

              displayname = super.getProperty(getLocalpart() + "/UTF8.TXT", Consts.N_displayname);
              assertTrue(displayname != null);
              assertTrue(DomElement.getText(displayname).equals("Something else"));

              super.logComment("displayname:settable");
            }
          }
          break;

        case 64:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          super.checkMultistatus ((propertyupdate (set (prop (
            "<hreftest xmlns='' xmlns:d='DAV:'><d:href>http://gibts.nicht/abc</d:href><d:href>"+
              getLocalpart() + "/Worddatei.doc" +
              "</d:href><something-else />text content</hreftest>"
            )))),
            respfile);
          break;

        case 65:
          request.setURI(getLocalpart() + "/UTF8.TXT");
          request.setMethod("REPORT");
          super.checkMultistatus ("<expand-property xmlns='DAV:'><property namespace='' name='hreftest'><property name='getcontentlength'/><property name='getcontenttype'/></property></expand-property>",
            respfile);
          break;

        case 66:
          if (supportsTypes) {

            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPPATCH");
            request.setBody("<propertyupdate xmlns='DAV:'><set><prop><ishidden xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' xsi:type='xsd:boolean'>true</ishidden></prop></set></propertyupdate>");
            response = requester.perform(request);
            checkResponseCode(response, 207);

            DomPath DP_ok_props = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/*", DomPath.DEFAULT_NAMESPACE_BINDINGS);
            boolean isok = DP_ok_props.match(response.getDocument()).size() == 1;

            if (isok) {
              Element ishidden = getProperty(getLocalpart() + "/UTF8.TXT", Consts.N_ishidden);
              String val = DomElement.getText(ishidden);
              assertTrue(val.equals("true") || val.equals("1"));
            }
          }

          break;

        case 67:
          {
            clear(request);
            request.setMethod("PROPPATCH");
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setBody(propertyupdate (set (prop ("<unicodetest xmlns=''>&#x10000;</unicodetest>"))));
            response = requester.perform(request);
            checkResponseCode(response, 207);

            Element utest = getProperty(getLocalpart() + "/UTF8.TXT", new Name(null, "unicodetest"));

            // get rid of the property to avoid problems in other test cases

            clear(request);
            request.setMethod("PROPPATCH");
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setBody(propertyupdate (remove (prop ("<unicodetest xmlns=''/>"))));
            response = requester.perform(request);
            checkResponseCode(response, 207);

            // check whether it has been set (before it was removed again)
            assertTrue(utest != null);
            String val = DomElement.getText(utest);
            assertTrue(val.length() == 2);
            assertTrue(com.sapportals.wcm.util.xml.SimpleSerializer.escape(val, false).equalsIgnoreCase("&#65536;"));
          }
          break;

        case 68:
          {
            clear(request);
            request.setMethod("PROPPATCH");
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setBody(propertyupdate (set (prop ("<hidden xmlns='' sap:hidden='true' xmlns:sap='"+Consts.NS_SAP_PROPERTYFLAGS+"'/>"))));
            response = requester.perform(request);
            checkResponseCode(response, 207);

            Element utest = getProperty(getLocalpart() + "/UTF8.TXT", new Name(null, "hidden"));
            assertTrue(utest != null);
            assertTrue(utest.getAttributeNS(Consts.NS_SAP_PROPERTYFLAGS, "hidden").equals("true"));
          }
          break;

        case 69:
          {
            // property names are restricted to 450 chars in rmcm
            StringBuffer ln = new StringBuffer("x");
            for (int i = 0; i < 50; i++) ln.append("0123456789");
            Name pn = new Name(null, ln.toString());

            DavStatus ds = setProperty(getLocalpart() + "/UTF8.TXT", pn, "abc", false);
            assertTrue(ds.getStatus() == 200 ||
              (ds.getStatus() == 403 && ds.getCondition().equals(DavConditions.N_SAP_property_name_valid)));

            if (ds.getStatus() == 200) {
              assertTrue(null != getProperty(getLocalpart() + "/UTF8.TXT", pn));
            }
          }
          break;

        case 70:
          {
            // sap:description is limited to 1024 characters in rmcm
            StringBuffer ln = new StringBuffer("x");
            for (int i = 0; i < 120; i++) ln.append("0123456789");
            Name pn = new Name(PropertyName.createDescription().getNamespaceOrNull(),
              PropertyName.createDescription().getName());

            DavStatus ds = setProperty(getLocalpart() + "/Worddatei.doc", pn,
              ln.toString(), false);
            assertTrue(ds.getStatus() == 200 ||
              (ds.getStatus() == 403 && ds.getCondition().equals(DavConditions.N_SAP_property_value_fits)));

            if (ds.getStatus() == 200) {
              assertTrue(null != getProperty(getLocalpart() + "/Worddatei.doc", pn));
            }
          }
          break;

        case 71: {
            clear(request);
            request.setMethod("PROPPATCH");
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setBody(propertyupdate (remove ("<foobar xmlns=''/>")));
            response = requester.perform(request);
            checkResponseCode(response, 400);
          }
          break;
          
        case 72: {
            // set a dead property
            Name pn = new Name("xyz:", "test");
            setProperty(getLocalpart() + "/Worddatei.doc", pn, "test", false);

            clear(request);
            request.setMethod("PROPFIND");
            request.setURI(getLocalpart() + "/Worddatei.doc");
            request.setHeader("depth", "0");
            request.setBody("<propfind xmlns='DAV:'><prop><resourcetype/></prop><dead-props/></propfind>");
            response = requester.perform(request);
            checkResponseCode(response, 207);

            DomPath dp_props = new DomPath("/D:multistatus/D:response/D:propstat/D:prop/*", DomPath.DEFAULT_NAMESPACE_BINDINGS);
            Map elems = dp_props.matchFirstNames(response.getDocument());

            assertTrue("result must contain resourcetype", elems.containsKey(Consts.N_resourcetype));
            assertTrue("result must contain " + pn + " (dead property)", elems.containsKey(pn));
          }
          break;

        case 73:
          {
            String valSet = "test ";
            Name pn = new Name("xyz:", "trailingblank");
            setProperty(getLocalpart() + "/Worddatei.doc", pn, valSet, false);

            Element property = getProperty(getLocalpart() + "/Worddatei.doc", pn);
            assertTrue(property != null);
            String valGet = property.getFirstChild().getNodeValue();
            assertTrue("wrong property value: '" + valGet + "', should be '" + valSet + "'", valSet.equals(valGet));
          }
          break;

        case 74:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:sd='"+Consts.NS_SAP_DATATYPES+"' xmlns:soap-enc='http://schemas.xmlsoap.org/soap/encoding/' soap-enc:arrayType='sd:expanded-name[2]' xsi:type='soap-enc:Array' xmlns='testexpnames'><sd:expanded-name>{green}bytes</sd:expanded-name><sd:expanded-name>{}bytes</sd:expanded-name></julian>"))),
              respfile);
          }
          break;

        case 75:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testexpnames'/></prop></propfind>",
              respfile);
          }
          break;

        case 76:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:sd='"+Consts.NS_SAP_DATATYPES+"' xsi:type='sd:expanded-name' xmlns='testexpname'>{green}bytes</julian>"))),
              respfile);
          }
          break;

        case 77:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testexpname'/></prop></propfind>",
              respfile);
          }
          break;
          
        case 78:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:sd='"+Consts.NS_SAP_DATATYPES+"' xsi:type='sd:mapped-rid' xmlns='testmappedrid'><href xmlns='DAV:'>rel</href></julian>"))),
              respfile);
          }
          break;

        case 79:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testmappedrid'/></prop></propfind>",
              respfile);
          }
          break;
          
        case 80:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            super.checkMultistatus (propertyupdate (set (prop ("<julian xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:sd='"+Consts.NS_SAP_DATATYPES+"' xsi:type='sd:mapped-rid-list' xmlns='testmappedridlist'>" +
                "<href xmlns='DAV:'>rel</href>" +
                "<href xmlns='DAV:'>rel?foo=bar</href>" +
                "<href xmlns='DAV:'>" + getLocalpart() + "/absolut%65"  + "</href>" + // use hex-escaping to test URI round-tripping
                "<href xmlns='DAV:'>" + getLocalpart() + "/absolute?foo=bar&amp;foo2=bar2"  + "</href>" + // use hex-escaping to test URI round-tripping
                "</julian>"))),
              respfile);
          }
          break;

        case 81:
          if (supportsTypes) {
            request.setURI(getLocalpart() + "/UTF8.TXT");
            request.setMethod("PROPFIND");
            request.setHeader("Depth", "0");
            super.checkMultistatus ("<propfind xmlns='DAV:'><prop><julian xmlns='testmappedridlist'/></prop></propfind>",
              respfile);
          }
          break;
          case 82:
          	{
          		clear(request);
          		request.setMethod("PROPPATCH");
          		request.setURI(getLocalpart() + "/UTF8.TXT");
          		request.setBody(propertyupdate(set(prop("<theprop xmlns='foo' anotherattr='bar' >foobar</theprop>"))));            
          		response = requester.perform(request);
          		checkResponseCode(response, 207);
          		
          		Element property = getProperty(getLocalpart() + "/UTF8.TXT", new Name("foo", "theprop"));
          		assertTrue(property != null);
          		NamedNodeMap nnm = property.getAttributes();
          		if (nnm!=null) {
          			Node n = nnm.getNamedItem("anotherattr");
          			if (n != null) {
          				super.logComment("property-attributes:yes");
          				supportsAttributes = true;
          				assertTrue("attribute value should be 'bar' but is " + n.getNodeValue(), n.getNodeValue().equals("bar"));
          			}              
          		}
          	}
          	break;
          	
          case 83: 
          	if (supportsAttributes) {
          		clear(request);
          		request.setMethod("PROPPATCH");
          		request.setURI(getLocalpart() + "/UTF8.TXT");
          		request.setBody(propertyupdate(set(prop("<theprop xmlns='foo' emptyattr='' anotherattr='bar' blankattr=' ' leadblank=' lead' trailblank='trail '>foobar</theprop>"))));            
          		response = requester.perform(request);
          		checkResponseCode(response, 207);
          		
          		request.setMethod("PROPFIND");
          		request.setHeader("Depth", "0");
          		super.checkMultistatus ("<propfind xmlns='DAV:'><prop><theprop xmlns='foo'/></prop></propfind>", respfile);
          	}
          	break;
          	
          case 84: {
              String valSet = "";
              Name pn = new Name(IWcmConst.SAP_WCM_NAMESPACE, IWcmConst.PROP_DESCRIPTION);
              setProperty(getLocalpart() + "/Worddatei.doc", pn, valSet, false);
    
              Element property = getProperty(getLocalpart() + "/Worddatei.doc", pn);
              if (property == null) {
                // this is ok, the persistence doesn't distinguish between null and ""
              }
              else {
                assertTrue("property value should be '" + valSet + "' but is '" + DomElement.getText(property) + "'", DomElement.getText(property).equals(valSet));
              }
            }
          
          	break;
          }
    }
    catch(Exception ex)
    {
      ex.printStackTrace(System.err);
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

}
