package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.protocol.webdav.DavConditions;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for <a href="http://greenbytes.de/tech/webdav/draft-ietf-webdav-redirectref-protocol-latest.html">RedirectRef</a> resources.
 * <p>
 * Copyright (c) SAP AG 2001-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: RedirectrefTest.java,v 1.8 2004/07/22 14:36:13 jre Exp $
 */

public class RedirectrefTest extends TestMember {

  private static final String[] desc = new String[]
  {
    "00 RF MKCOL for container",
    "01 RF check OPTIONS",
    "02 RF (unused)",
    "03 RF (unused)",
    "04 RF (unused)",
    "05 RF (unused)",
    "06 RF (unused)",
    "07 RF MKREDIRECTREF redirectref resource",
    "08 RF PROPFIND on redirectref resource (standard)",
    "09 RF PROPFIND on redirectref resource (Apply-To-Redirect-Ref)",
    "10 RF HEAD on redirectref (standard)",
    "11 RF HEAD on redirectref (Apply-To-Redirect-Ref)",
    "12 RF PUT on redirectref (standard)",
    "13 RF PUT on redirectref (Apply-To-Redirect-Ref)",
    "14 RF PROPPATCH on redirectref (standard)",
    "15 RF PROPPATCH on redirectref (Apply-To-Redirect-Ref)",
    "16 RF PROPFIND redirectref resource",
    "17 RF DELETE redirectref resource",
    "18 RF COPY collection with redirectref",
    "19 RF MKRESOURCE redirect ref to external collection",
    "20 RF HEAD on child of ref to external collection",
    "21 RF MKREDIRECTREF / MKRESOURCE try overwrite",
    "22 RF DELETE redirectref resource (Apply-To-Redirect-Ref)",
    "23 RF (unused)",
    "   RF DELETE container resource(s)",
  };

  public static int NrOfTests = desc.length;

  public RedirectrefTest (Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest() {
    if (nr == 0) logIdentifier("RF");

    String respfile = "REDIRECTREF-" + TestMember.THREEDIGITS.format(nr) + ".txt";
    String redirectTo = "http://www.greenbytes.de/tech/webdav/draft-ietf-webdav-redirectref-protocol-02.html";
    String container = "/redirectref%20container";

    try {
      IResponse response;
      clear(request);

      switch (nr) {

        case 0:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + container);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 1:
          assertMethodAllowed(getLocalpart() + container + "xyz", "MKREDIRECTREF");
          break;

        case 2:
          break;

        case 3:
          break;

        case 4:
          break;

        case 5:
          break;

        case 6:
          break;

        case 7:
          clear(request);
          request.setMethod("MKREDIRECTREF");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setBody("<mkredirectref xmlns='DAV:'>" + "<reftarget>" + "<href>" + redirectTo + "</href>" + "</reftarget>"
              + "</mkredirectref>");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 8:
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Depth", "0");
          request.setBody("<propfind xmlns='DAV:'><allprop /></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Redirect-Ref").equals(redirectTo));
          assertTrue(response.getHeader("Location") != null);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Depth", "0");
          request.setHeader("Apply-To-Redirect-Ref", "F");
          request.setBody("<propfind xmlns='DAV:'><allprop /></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Redirect-Ref").equals(redirectTo));
          assertTrue(response.getHeader("Location") != null);
          break;

        case 9:
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Depth", "0");
          request.setHeader("Apply-To-Redirect-Ref", "T");
          checkMultistatus("<propfind xmlns='DAV:'><prop><resourcetype/><reftarget/><redirect-lifetime/></prop></propfind>",
              respfile);
          break;

        case 10:
          clear(request);
          request.setMethod("HEAD");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue("wrong Location header returned", response.getHeader("Location").equals(redirectTo));
          assertTrue("wrong Redirect-Ref header returned", response.getHeader("Redirect-Ref").equals(redirectTo));
          break;

        case 11:
          clear(request);
          request.setMethod("HEAD");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Apply-To-Redirect-Ref", "T");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          assertTrue("Location header returned", response.getHeader("Location") == null);
          assertTrue("Redirect-Ref header returned", response.getHeader("Redirect-Ref") == null);
          break;

        case 12:
          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setContentType("foo/bar");
          request.setBody("unfug");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue("wrong Location header returned", response.getHeader("Location").equals(redirectTo));
          assertTrue("wrong Redirect-Ref header returned", response.getHeader("Redirect-Ref") != null);
          break;

        case 13:
          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setContentType("foo/bar");
          request.setBody("unfug");
          request.setHeader("Apply-To-Redirect-Ref", "T");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          break;

        case 14:
          clear(request);
          request.setMethod("PROPPATCH");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setBody("<propertyupdate xmlns='DAV:'>" + "<set>" + "<prop><another xmlns='foobar'>xyz</another></prop>"
              + "</set>" + "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue("wrong Location header returned", response.getHeader("Location").equals(redirectTo));
          assertTrue("wrong Redirect-Ref header returned", response.getHeader("Redirect-Ref") != null);
          break;

        case 15:
          clear(request);
          request.setMethod("PROPPATCH");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Apply-To-Redirect-Ref", "T");
          checkMultistatus("<propertyupdate xmlns='DAV:'>" + "<set>" + "<prop><another xmlns='foobar'>xyz</another></prop>"
              + "</set>" + "</propertyupdate>", respfile);
          break;

        case 16:
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Depth", "0");
          request.setHeader("Apply-To-Redirect-Ref", "T");
          checkMultistatus("<propfind xmlns='DAV:'><prop><resourcetype/><reftarget/><another xmlns='foobar'/></prop></propfind>",
              respfile);
          break;

        case 17:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue("wrong Location header returned", response.getHeader("Location").equals(redirectTo));
          assertTrue("wrong Redirect-Ref header returned", response.getHeader("Redirect-Ref") != null);
          break;

        case 18:
          clear(request);
          request.setMethod("COPY");
          request.setURI(getLocalpart() + container);
          request.setHeader("Destination", this.getBaseAndLocalpart() + "/copy");
          checkMultistatus(null, respfile);
          break;

        case 19:
          clear(request);
          request.setMethod("MKRESOURCE");
          request.setURI(getLocalpart() + container + "/redirectref%20to%20collection");
          request.setBody("<propertyupdate xmlns='DAV:'>" + "<set>" + "<prop>" + "<resourcetype><redirectref /></resourcetype>"
              + "<reftarget><href>" + "http://greenbytes.de/tech/webdav/" + "</href></reftarget>" + "</prop>" + "</set>"
              + "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 20:
          String expect = "http://greenbytes.de/tech/webdav/rfc2518.html";

          clear(request);
          request.setMethod("HEAD");
          request.setURI(getLocalpart() + container + "/redirectref%20to%20collection" + "/rfc2518.html");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue("wrong Location header returned, expected <" + expect + ">, got <" + response.getHeader("Location") + ">",
              expect.equals(response.getHeader("Location")));
          assertTrue("wrong Redirect-Ref header returned", response.getHeader("Redirect-Ref") != null);
          break;

        case 21:
          clear(request);
          request.setMethod("MKREDIRECTREF");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setBody("<mkredirectref xmlns='DAV:'>" + "<reftarget>" + "<href>" + redirectTo + "</href>" + "</reftarget>"
              + "</mkredirectref>");
          response = requester.perform(request);
          checkResponseCode(response, 409);
          checkErrorElement(response, DavConditions.N_resource_must_be_null);

          clear(request);
          request.setMethod("MKRESOURCE");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setBody("<propertyupdate xmlns='DAV:'>" + "<set>" + "<prop>" + "<resourcetype><redirectref /></resourcetype>"
              + "<reftarget><href>" + redirectTo + "</href></reftarget>" + "<another xmlns='foo'>property</another>" + "</prop>"
              + "</set>" + "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 409);
          checkErrorElement(response, DavConditions.N_resource_must_be_null);
          break;

        case 22:
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container + "/a%20redirectref%20resource");
          request.setHeader("Apply-To-Redirect-Ref", "T");
          response = requester.perform(request);
          checkResponseCode(response, 200, 204);
          break;

        case 23:
          break;

        default:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container);
          response = requester.perform(request);
          checkResponseCode(response, 200, 204);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + "/copy/");
          response = requester.perform(request);
          checkResponseCode(response, 200, 204);
          break;
      }
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

}
