package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Element;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.Name;

/**
 * Various tests for the status of the repository.
 * <p>
 * Copyright (c) SAP AG 2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: RepositoryManagerStatusTest.java,v 1.1 2004/09/02 09:29:35 jre Exp $
 */

public class RepositoryManagerStatusTest extends TestMember {

  private static final String[] desc = new String[]{
      "00 RS setup",
      "01 RS check repository manager",
      "02 RS check ACL manager",
      "03 RS fsmount status",
      "04 RS RM specific settings",
      "   RS cleanup",
     };

  public static int NrOfTests = desc.length;

  public RepositoryManagerStatusTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }
  

  public static final String NS_CRT = "http://sapportals.com/xmlns/cm/crt";
  public static final Name PN_CRT_cfgClass = new Name(NS_CRT, "config-class");
  public static final Name PN_CRT_cfgSecurityManagerRef = new Name(NS_CRT, "config-securitymgr.ref");
  
  private static String CONFIGURABLEURI;
  private static String RM;
  
  private static Map SHORTNAMES;
  
  static {
    Map tmp = new HashMap();
    
    tmp.put("com.sapportals.wcm.repository.manager.reporting.RPRepositoryManager", "RP");
    tmp.put("com.sapportals.wcm.repository.manager.webdav.WDRepositoryManager", "WD");
    tmp.put("com.sapportals.wcm.repository.manager.sfs.FSRepositoryManager", "SFS");
    tmp.put("com.sapportals.wcm.repository.manager.cm.CmRepositoryManager", "CM");
    tmp.put("com.sapportals.wcm.repository.util.file.StdFileImpl", "Java File I/O"); // temp for SP2CP
    tmp.put("com.sapportals.wcm.repository.util.file.SmbFileImpl", "JCifs"); // temp for SP2CP
    tmp.put("false", "f");
    tmp.put("true", "t");
    
    SHORTNAMES = Collections.unmodifiableMap(tmp);
  }
  
  private static String getShortName(String n) {
    String mapped = (String)SHORTNAMES.get(n);
    return mapped == null ? n : mapped;
  }
  

  protected void runTest() {
    if (nr == 0) {
      logIdentifier("RS");
    }


    try {
      switch (nr) {

        case 0: {
            Element rep = getProperty(getLocalpart(), Consts.N_SAP_repository);
            assertTrue("live property sap:repository missing", rep != null);
            CONFIGURABLEURI = getRelativeUriFromPropertyHrefs(rep);
          }
          break;
          
        case 1: {
            Element rm = getProperty(CONFIGURABLEURI, PN_CRT_cfgClass);
            assertTrue("live property crt:config-class missing", rm != null);
            RM = getShortName(DomElement.getText(rm));
            logComment(RM);
          }
          break;

        case 2: {
            Element sm = getProperty(CONFIGURABLEURI, PN_CRT_cfgSecurityManagerRef);
            if (sm != null) {
              logComment(getShortName(DomElement.getText(sm)));
            }
          }
          break;
        
        case 3: {
            if ("SFS".equals(RM)) {
              String mi = CONFIGURABLEURI + "/mountinfo";
              
              Element ty = getProperty(mi, new Name(NS_CRT, "type"));
              assertTrue("live property crt:type missing on mountinfo entry", ty != null);
              logComment(getShortName(DomElement.getText(ty)));
            }
          }
          break;

        case 4: {
            if ("SFS".equals(RM)) {
              // format {lookupmode}

              String comment = "";

              Element lm = getProperty(CONFIGURABLEURI, new Name(NS_CRT, "config-lookupmode"));
              assertTrue("live property crt:config-lookupmode missing", lm != null);
              comment += getShortName(DomElement.getText(lm));

              logComment(comment);
            }
            else if ("CM".equals(RM)) {
              // format {persistence.mode keep-vhr}
              
              String comment = "";
              
              Element pm = getProperty(CONFIGURABLEURI, new Name(NS_CRT, "config-persistence.mode"));
              assertTrue("live property crt:config-persistence.mode missing", pm != null);
              comment += getShortName(DomElement.getText(pm));

              Element kv = getProperty(CONFIGURABLEURI, new Name(NS_CRT, "config-keepVersionHistory"));
              assertTrue("live property crt:config-keepVersionHistory missing", kv != null);
              comment += " " + getShortName(DomElement.getText(kv));
              
              logComment(comment);
            }
            
          }
          break;
          
        default:
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}