package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for the servlet engine.
 * <p>
 * Copyright (c) SAP AG 2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: ServletEngineTest.java,v 1.4 2004/12/17 12:54:57 jre Exp $
 */

public class ServletEngineTest extends TestMember {
  
  private static final String[] desc = new String[] {
    "00 SE setup",
    "01 SE MKCOL/PROPFIND/HEAD/PROPFIND",
    "02 SE Weird UA string",
    "03 SE PUT result content-type",
    "04 SE ';' in Request-URI",
    "   SE cleanup",
  };

  public static int NrOfTests = desc.length;

  public ServletEngineTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }
  
  protected void runTest() {
    if (nr == 0) super.logIdentifier("SE");
    
    IResponse response;
    
    try {
      switch (nr) {

        case 0:
          break;
          
        case 1:
          // make sure that a PROPFIND following a HEAD does work (this may break when intermediates
          // such as the J2EE have trouble processing HEAD responses (content length issue).

          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + "/childcollection");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/childcollection");
          request.setHeader("Depth", "0");
          request.setHeader("cache-control", "no-cache");
          request.setBody("<propfind xmlns='DAV:'><allprop/></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);

          request.setMethod("HEAD");
          request.setURI(getLocalpart() + "/childcollection");
          request.setHeader("Translate", "f");
          request.setHeader("cache-control", "no-cache");
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/childcollection");
          request.setHeader("Depth", "0");
          request.setHeader("cache-control", "no-cache");
          request.setBody("<propfind xmlns='DAV:'><allprop/></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          break;
          
        case 2:
          clear(request);
          request.setMethod("PROPFIND");
          request.setURI(getLocalpart() + "/ANSI.TXT");
          request.setHeader("User-Agent", "MSIE");
          request.setHeader("Depth", "0");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          break;

        case 3:
          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + "/childcollection/puttest");
          request.setHeader("Content-Type", "text/plain; charset=UTF8");
          request.setBody("ServletEngineTest 3");
          response = requester.perform(request);
          assertTrue("content type header must not be present on response", null == response.getHeader("content-type"));
          break;

        case 4:
          clear(request);
          request.setMethod("HEAD");
          request.setURI(getLocalpart() + ";x/ANSI.TXT");
          response = requester.perform(request);
          checkResponseCode(response, 404);
          break;

        default:
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + "/childcollection");
          response = requester.perform(request);
          checkResponseCode(response, 204);
          
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
