package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.HashSet;
import java.util.Set;

import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for TRACE method.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author julian.reschke@greenbytes.de
 * @version $Id: TraceTest.java,v 1.3 2004/03/29 11:52:07 jre Exp $
 */

public class TraceTest extends TestMember {

  private static final String[] desc = new String[] {
    "00 TR Setup",
    "01 TR empty header",
    "02 TR multiline header",
    "03 TR long header",
    "04 TR cleanup",
  };

  public static int NrOfTests = desc.length;

  private static final String MA =
      "A-12345678901234567890123456789012345678901234567890123456789012345678901234567890";
  private static final String MB =
      "B-12345678901234567890123456789012345678901234567890123456789012345678901234567890";

  public TraceTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  private static String emptyHeader = null;
  private static String longHeader = null;
  private static Set multHeaders = new HashSet();
  private static String longtemplate;

  static {
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < 250; i++) sb.append("0123456789");
    longtemplate = sb.toString();
  }

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("TR");

    try
    {
      IResponse response;

      switch (nr) {
        case 0: {
            clear(request);
            request.setMethod("TRACE");
            request.setURI(getLocalpart());
            request.setHeader("empty-header", "");
            request.addHeader("mult-header", MA);
            request.addHeader("mult-header", MB);
            request.addHeader("long-header", longtemplate);
            request.expectsResponseStream(true);
            response = requester.perform(request);
            checkResponseCode(response, 200);

            java.io.LineNumberReader reader = new java.io.LineNumberReader
              (new java.io.InputStreamReader(response.getStream()));

            String s;
            while ((s = reader.readLine()) != null) {
              if (s.toLowerCase().startsWith("empty-header:")) {
                emptyHeader = s.substring("empty-header:".length()).trim();
              }
              if (s.toLowerCase().startsWith("mult-header:")) {
                multHeaders.add(s.substring("mult-header:".length()).trim());
              }
              if (s.toLowerCase().startsWith("long-header:")) {
                longHeader = s.substring("long-header:".length()).trim();
              }
            }
          }
          break;

        case 1:
          assertTrue(emptyHeader != null && emptyHeader.length() == 0);
          break;

        case 2:
//          assertTrue("header "+MA+" missing in: "+multHeaders, multHeaders.contains(MA));
//          assertTrue("header "+MB+" missing in: "+multHeaders, multHeaders.contains(MB));
          break;

        case 3:
          assertTrue(longHeader != null && longHeader.equals(longtemplate));
          break;

        default:
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }

  }

}