package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.DomSerializer;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for UM repository manager.
 * <p>
 * Copyright (c) SAP AG 2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: UmRepTest.java,v 1.4 2004/04/20 16:56:12 jre Exp $
 */

public class UmRepTest extends TestMember {
  
  private static final String[] desc = new String[] {
    "00 UM setup",
    "01 UM get principal collection set",
    "02 UM find self URI",
    "03 UM check group-member-ship",
    "04 UM check group containment",
    "05 UM check parent-set property",
    "   UM cleanup",
  };

  public static int NrOfTests = desc.length;

  public UmRepTest(Integer nr) {
    super(nr, desc[nr.intValue()]);
  }

  private static Set principalCollections = null;
  // private static String selfCollection = null;
  private static String self = null;
  private static Set selfGroups = null;
  
  protected void runTest() {
    if (nr == 0) super.logIdentifier("UM");
    
    IResponse response;
    
    try {
      switch (nr) {

        case 0:
          break;
          
        case 1: {
            Element pcs = getProperty(getLocalpart() + "/", Consts.N_principal_collection_set);
            assertTrue("principal-collection-set not found", pcs != null);

            principalCollections = DomElement.getHrefs(pcs);
            assertTrue("there should be at least one principal collection href in DAV:principal-collection-set", !principalCollections.isEmpty());
          }
          break;

        case 2: {
            List principals = new ArrayList();
  
            for (Iterator it = principalCollections.iterator(); it.hasNext(); ) {
              String url = it.next().toString();
              
              clear(request);
              request.setMethod("REPORT");
              request.setURI(url);
              request.setHeader("Depth", "0");
              request.setBody("<principal-match xmlns='DAV:'><self/></principal-match>");
              request.setContentType("text/xml");
              response = requester.perform(request);
              checkResponseCode(response, 207);
              Document doc = response.getDocument();
              assertTrue("document missing", doc != null);
              List hrefs = getHrefsFromMultistatus(doc);
              for (Iterator it2 = hrefs.iterator(); it2.hasNext(); ) {
                // selfCollection = url;
                principals.add(it2.next());
              }
            }
            
            assertTrue ("unexpected number of matching principals: " + principals,
                   principals.size() == 1);
            self = (String)principals.get(0);  
          }
          break;  
        
        case 3: {
            Element gms = getProperty(self, Consts.N_group_membership);
            assertTrue("DAV:group-membership must be present on " + self, gms != null);
            selfGroups = DomElement.getHrefs(gms);
            assertTrue("current user is supposed to be member of at least one group/role, response was: " + DomSerializer.toString(gms, true), ! selfGroups.isEmpty());
          }
          break;  

        case 4: {
            // for each group we're member of, do the reverse lookup
            for (Iterator it = selfGroups.iterator(); it.hasNext();) {
              String url = it.next().toString();
              Element gm = getProperty(url, Consts.N_group_member_set);
              assertTrue("DAV:group-member-set must be present on " + url, gm != null);
              Set members = DomElement.getHrefs(gm);
              assertTrue("current user should be listed as member of DAV:group-member-set of " + url + ", response was: "
                  + DomSerializer.toString(gm, true), members.contains(self));
              
              // check DAV:parent-set for the group
              Element ps = getProperty(url, Consts.N_parent_set);
              assertTrue("DAV:parent-set must be present on " + url, ps != null);
            }
          }
          break;

        case 5: {
            Element ps = getProperty(self, Consts.N_parent_set);
            assertTrue("DAV:parent-set must be present on " + self, ps != null);
          }
          break;

        default:
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
