package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.List;

import org.w3c.dom.Element;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.dompath.DomPath;
import com.sapportals.wcm.util.http.IResponse;



/**
 * Various tests for UPDATE feature.
 *
 * <p>
 * Copyright (c) SAP AG 2002-2005
 * @author julian.reschke@greenbytes.de
 * @version $Id: UpdateTest.java,v 1.5 2005/01/28 11:50:47 jre Exp $
 */
public class UpdateTest extends TestMember
{
  private static final String[] desc = new String[]
  {
    "00 UP Setting up UPDATE feature",
    "01 UP Create version 2",
    "02 UP Create version 3",
    "03 UP Check version tree",
    "04 UP UPDATE from version 2",
    "05 UP Check DAV:checked-in",
    "06 UP Recheck version tree",
    "07 UP Create version 4 and recheck version tree" ,
    "08 UP Create version 5, delete version 4, recheck tree",
    "09 UP attempt delete of root version",
    "10 UP Delete version 2, recheck tree",
    "   UP cleanup",
  };

  public static final int NrOfTests = desc.length;

  public UpdateTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  private static String v1Uri;
  private static String v2Uri;
  // private static String v3Uri;
  private static String v4Uri;
  private static String v5Uri;

  protected void runTest()
  {
    String reqUri;

    if (nr == 0) super.logIdentifier("UP");

    String respfile = "UPDATE-" + TestMember.TWODIGITS.format(nr) + ".txt";
    String collname = "/update/";
    String filename = "/update/test%20of%20update.xls";

    try
    {
      IResponse response;
      List hrefs;

      clear(request);

      switch (nr)
      {
        case 0:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + collname);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + filename);
          request.setBody("1");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("VERSION-CONTROL");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          v1Uri = getRelativeUriFromLocationHeader(response);
          break;

        case 1:
          clear(request);
          request.setMethod("CHECKOUT");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + filename);
          request.setBody("22");
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("CHECKIN");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          v2Uri = getRelativeUriFromLocationHeader(response);
          break;

        case 2:
          clear(request);
          request.setMethod("CHECKOUT");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + filename);
          request.setBody("333");
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("CHECKIN");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          // v3Uri = getRelativeUriFromLocationHeader(response);
          break;

        case 3:
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + filename);
          super.checkMultistatus("<version-tree xmlns='DAV:'>"
            +"<prop>"
            +"<getcontentlength/><successor-set/><predecessor-set/>"
            +"</prop>"
            +"</version-tree>", respfile);
          break;

        case 4:
          clear(request);
          request.setMethod("UPDATE");
          request.setURI(getLocalpart() + filename);
          request.setBody("<update xmlns='DAV:'><version><href>"+
                          v2Uri + "</href></version>"
            +"<prop><getcontentlength/></prop>"
            +"</update>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 5:
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + filename);
          super.checkMultistatus("<expand-property xmlns='DAV:'>"
            + "<property name='getcontentlength'/>"
            + "<property name='checked-in'>"
            +   "<property name='getcontentlength'/>"
            + "</property>"
            + "</expand-property>", respfile, new DomPath("//D:href", DomPath.DEFAULT_NAMESPACE_BINDINGS));
          break;

        case 6:
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + filename);
          super.checkMultistatus("<version-tree xmlns='DAV:'>"
            +"<prop>"
            +"<getcontentlength/><successor-set/><predecessor-set/>"
            +"</prop>"
            +"</version-tree>", respfile);
          break;

        case 7: {
            Element checked_in = getProperty(getLocalpart() + filename, Consts.N_checked_in);
            String version = getRelativeUriFromPropertyHrefs(checked_in);

            Element checkout_fork = getProperty(version, Consts.N_checkout_fork);
            assertTrue("checkout-fork property should be present on version", checkout_fork != null);
            assertTrue("checkout-fork should say discouraged",
              DomElement.hasChild(checkout_fork,
                com.sapportals.wcm.repository.IVersionResource.FORKBEHAVIOUR_DISCOURAGED));

            clear(request);
            request.setMethod("CHECKOUT");
            request.setURI(getLocalpart() + filename);
            response = requester.perform(request);
            checkResponseCode(response, 403);
            // todo: check correct precondition

            clear(request);
            request.setMethod("CHECKOUT");
            request.setURI(getLocalpart() + filename);
            request.setBody("<checkout xmlns='DAV:'><fork-ok/></checkout>");
            response = requester.perform(request);
            checkResponseCode(response, 200);

            clear(request);
            request.setMethod("PUT");
            request.setURI(getLocalpart() + filename);
            request.setBody("4444");
            response = requester.perform(request);
            checkResponseCode(response, 200);

            Element checkin_fork = getProperty(getLocalpart() + filename, Consts.N_checkin_fork);
            assertTrue("checkin-fork property should be present", checkin_fork != null);
            assertTrue("checkin-fork should say discouraged",
              DomElement.hasChild(checkin_fork,
                com.sapportals.wcm.repository.IVersionResource.FORKBEHAVIOUR_DISCOURAGED));

            clear(request);
            request.setMethod("CHECKIN");
            request.setURI(getLocalpart() + filename);
            response = requester.perform(request);
            checkResponseCode(response, 403);
            // todo: check correct precondition

            clear(request);
            request.setMethod("CHECKIN");
            request.setURI(getLocalpart() + filename);
            request.setBody("<checkin xmlns='DAV:'><fork-ok/></checkin>");
            response = requester.perform(request);
            checkResponseCode(response, 201);
            v4Uri = response.getHeader("Location");

            reqUri = removeProtocolAndBase(v2Uri);

            Element succ = getProperty(reqUri, Consts.N_successor_set);
            hrefs = succ == null ? null : DomElement.getChildren(succ, Consts.N_href);
            assertTrue(hrefs != null && hrefs.size() == 2);
            // should also check that the successors are the correct ones
          }
          break;

        case 8:
          clear(request);
          request.setMethod("CHECKOUT");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + filename);
          request.setBody("55555");
          response = requester.perform(request);
          checkResponseCode(response, 200);

          clear(request);
          request.setMethod("CHECKIN");
          request.setURI(getLocalpart() + filename);
          response = requester.perform(request);
          checkResponseCode(response, 201);
          v5Uri = response.getHeader("Location");

          reqUri = removeProtocolAndBase(v4Uri);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(reqUri);
          response = requester.perform(request);
          checkResponseCode(response, 204);

          reqUri = removeProtocolAndBase(v5Uri);

          // check that the newly created resource has only one predecessor,
          // and that is has the contentlength 2
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(reqUri);
          checkMultistatus("<expand-property xmlns='DAV:'>"
            + "<property name='predecessor-set'>"
            +   "<property name='getcontentlength'/>"
            + "</property>"
            + "</expand-property>", respfile, new DomPath("//D:href", DomPath.DEFAULT_NAMESPACE_BINDINGS));
          break;

        case 9:
          reqUri = removeProtocolAndBase(v1Uri);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(reqUri);
          response = requester.perform(request);
          checkResponseCode(response, 204);

          reqUri = removeProtocolAndBase(v2Uri);
          // System.out.println(reqUri);

          // make sure the predecessor-set of V2 is now empty
          Element pred = getProperty(reqUri, Consts.N_predecessor_set);
          hrefs = pred == null ? null : DomElement.getChildren(pred, Consts.N_href);
          assertTrue(hrefs == null || hrefs.size() == 0);
          break;

        case 10:
          reqUri = removeProtocolAndBase(v2Uri);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(reqUri);
          response = requester.perform(request);
          assertTrue(response.getStatus() >= 400);
          break;

        default:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + collname);
          response = requester.perform(request);
          checkResponseCode(response, 204, 404);
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

}
