package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.Iterator;
import java.util.Set;

import org.w3c.dom.Element;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.protocol.webdav.DavConditions;
import com.sapportals.wcm.protocol.webdav.DomElement;
import com.sapportals.wcm.protocol.webdav.dompath.DomPath;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for VHR resources.
 * <p>
 * Copyright (c) SAP AG 2002-2005
 * @author julian.reschke@greenbytes.de
 * @version $Id: VersionHistoryResourceTest.java,v 1.8 2005/02/18 12:46:59 sei Exp $
 */
public class VersionHistoryResourceTest extends TestMember
{

  private static final String[] desc = new String[]
  {
    "00 VH Setting up VHR",
    "01 VH Getting VHR URI",
    "02 VH check resourcetype and live properties of VHR",
    "03 VH PROPPATCH / PROPFIND on VHR",
    "04 VH try locate-by-history report (wrong request URI)",
    "05 VH try locate-by-history report (wrong VHR reference)",
    "06 VH try locate-by-history report (successful)",
    "07 VH try locate-by-history report on parent resource (successful)",
    "08 VH try locate-by-history report on wrong collection (-> empty)",
    "09 VH copy VHR",
    "10 VH move VHR",
    "11 VH try expand-property report",
    "12 VH try rename of VCR",
    "13 VH test for presence of VHR",
    "14 VH delete VHR",
    "15 VH make sure it's not versioned anymore",
    "16 VH check DAV:version-history-collection-set",
    "17 VH OPTIONS/DAV:version-history-collection-set",
    "   VH cleanup",
  };

  public static final int NrOfTests = desc.length;


  private static final String CONTENT_1 = "this is the content of the first version";

  private static DomPath DP_vhr_href = null;
  private static DomPath DP_vhr_href2 = null;
  private static DomPath DP_resourcetype_vhr = null;
  private static DomPath DP_prop_root_version = null;
  private static DomPath DP_prop_version_set = null;
  private static DomPath DP_props = null;

  static {
    DP_vhr_href = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:version-history/D:href",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
    DP_vhr_href2 = new DomPath ("/D:multistatus/D:response/D:propstat/D:prop/D:checked-in/D:response/D:propstat/D:prop/D:version-history/D:href",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
    DP_resourcetype_vhr = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:resourcetype/D:version-history",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
    DP_prop_root_version = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:root-version/D:href",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
    DP_prop_version_set = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/D:version-set/D:href",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
    DP_props = new DomPath ("/D:multistatus/D:response/D:propstat[contains(D:status,'200')]/D:prop/*",
      DomPath.DEFAULT_NAMESPACE_BINDINGS);
  }

  public VersionHistoryResourceTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  private static String vhruri = null;
  private static String testColl = "/vhr";
  private static String testVcr = testColl + "/resource";

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("VHR");

    String respfile = "VHR-" + TestMember.TWODIGITS.format(nr) + ".txt";

    try
    {
      IResponse response;
      Element href;

      clear(request);

      switch (nr)
      {
        case 0:
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + testColl);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + testVcr);
          request.setBody(CONTENT_1);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          assertMethodAllowed(getLocalpart() + testVcr, "VERSION-CONTROL");

          clear(request);
          request.setMethod("VERSION-CONTROL");
          request.setURI(getLocalpart() + testVcr);
          response = requester.perform(request);
          checkResponseCode(response, 200, 201);

          clear(request);
          request.setMethod("VERSION-CONTROL");
          request.setURI(getLocalpart() + testVcr);
          response = requester.perform(request);
          checkResponseCode(response, 200, 201);

          break;

        case 1:
          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + testVcr);
          request.setBody("<propfind xmlns='DAV:'><prop><version-history/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          href = (Element) DP_vhr_href.matchOrDie (response.getDocument(), 1, 1, "version-history/href not found").get(0);
          vhruri = DomElement.getText(href);

          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + testVcr);
          request.setBody("<expand-property xmlns='DAV:'><property name='checked-in'><property name='version-history'/></property></expand-property>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          href = (Element) DP_vhr_href2.matchOrDie (response.getDocument(), 1, 1, "version-history/href not found").get(0);
          String vhruri2 = DomElement.getText(href);
          assertTrue("VCR and version references to VHR should match", vhruri.equals(vhruri2));
          break;

        case 2:
          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(vhruri);
          request.setBody("<propfind xmlns='DAV:'><prop><resourcetype/><version-set/><root-version/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          //System.out.println(com.sapportals.wcm.protocol.webdav.DomSerializer.toString(response.getDocument(),true));
          DP_resourcetype_vhr.matchOrDie (response.getDocument(), 1, 1, "resourcetype/version-history not found");
          DP_prop_root_version.matchOrDie (response.getDocument(), 1, 1, "root-version/href not found");
          DP_prop_version_set.matchOrDie (response.getDocument(), 1, 999999999, "version-set/href not found");
          break;

        case 3:
          clear(request);
          request.setMethod("PROPPATCH");
          request.setURI(vhruri);
          request.setBody("<propertyupdate xmlns='DAV:'><set><prop><foo xmlns='bar'>xzy</foo></prop></set></propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 207);

          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(vhruri);
          request.setBody("<propfind xmlns='DAV:'><prop><foo xmlns='bar'/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          assertTrue(DP_props.matchNodes(response.getDocument()).length == 1);
          break;

        case 4:
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + testVcr);
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop><resourcetype/></prop></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 403);

          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "1");
          request.setURI(getLocalpart() + testVcr);
          checkMultistatus("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop><resourcetype/></prop></locate-by-history>",
            respfile);

          break;

        case 5:
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "1");
          request.setURI(getLocalpart() + testColl);
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+getLocalpart()+"/vhr/resource</href></version-history-set></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          break;

        case 6:
          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + testColl);
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop><resourcetype/></prop></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);

          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "1");
          request.setURI(getLocalpart() + testColl);
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop><resourcetype/></prop></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, "VHR-" + TestMember.TWODIGITS.format(nr) + "-B.txt", null);

          //clear(request);
          //request.setMethod("REPORT");
          //request.setHeader("Depth", "infinity");
          //request.setURI(getLocalpart() + testColl);
          //request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop><resourcetype/></prop></locate-by-history>");
          //response = requester.perform(request);
          //checkResponseCode(response, 403);
          //checkErrorElement(response, DavConditions.N_SAP_depth_infinity_allowed);
          break;

        case 7:
          clear(request);
          request.setMethod("REPORT");
          request.setURI(getLocalpart());
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop/></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 8:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + testColl + "2");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + testColl + "2");
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop/></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);

          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "1");
          request.setURI(getLocalpart() + testColl + "2");
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>"+vhruri+"</href></version-history-set><prop/></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, "VHR-" + TestMember.TWODIGITS.format(nr) + "-B.txt", null);

          clear(request);
          request.setMethod("REPORT");
          request.setHeader("Depth", "infinity");
          request.setURI(getLocalpart() + testColl + "2");
          request.setBody("<locate-by-history xmlns='DAV:'><version-history-set><href>" + vhruri
              + "</href></version-history-set><prop/></locate-by-history>");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          checkErrorElement(response, DavConditions.N_SAP_depth_infinity_allowed);
          break;

        case 9:
          clear(request);
          request.setMethod("COPY");
          request.setURI(vhruri);
          request.setHeader("Destination", getBaseAndLocalpart() + "/vhr/resource-copy");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          super.checkErrorElement(response, DavConditions.N_cannot_copy_history);
          break;

        case 10:
          clear(request);
          request.setMethod("MOVE");
          request.setURI(vhruri);
          request.setHeader("Destination", getBaseAndLocalpart() + "/vhr/resource-move");
          response = requester.perform(request);
          checkResponseCode(response, 403);
          super.checkErrorElement(response, DavConditions.N_cannot_rename_history);
          break;

        case 11:
          clear(request);
          request.setMethod("REPORT");
          request.setURI(getLocalpart() + testColl);
          request.setHeader("depth", "1");
          request.setBody("<expand-property xmlns='DAV:'><property name='version-history'><property namespace='DAV:' name='version-set'><property name='version-name'/><property name='predecessor-set'/><property name='successor-set'/><property name='gibtsnicht' namespace='xyz'/></property></property></expand-property>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 12:
          clear(request);
          request.setMethod("MOVE");
          request.setURI(getLocalpart() + testVcr);
          request.setHeader("Destination", getBaseAndLocalpart() + testVcr + "2");
          response = requester.perform(request);
          if (response.getStatus() == 201) {
            // renamed
            testVcr += "2";
          }
          else {
            checkStatusAndErrorElement(response, 409, DavConditions.N_preserve_versioning_properties);
          }
          break;

        case 13:
          clear(request);
          request.setMethod("HEAD");
          request.setURI(vhruri);
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 14:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(vhruri);
          response = requester.perform(request);
          checkResponseCode(response, 204);
          break;

        case 15:
          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + testVcr);
          request.setBody("<propfind xmlns='DAV:'><prop><version-history/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          java.util.List foo = DP_vhr_href.match (response.getDocument());
          assertTrue("version-history property still set", foo.size() == 0);
          break;

        case 16: {
            Element vhcs = getProperty(getLocalpart() + testVcr, Consts.N_version_history_collection_set);
            assertTrue(vhcs != null);
            Set hrefs = DomElement.getHrefs(vhcs);
            assertTrue(hrefs.size() != 0);

            boolean foundParent = false;
            for (Iterator it = hrefs.iterator(); it.hasNext(); /**/) {
              String hr = (String)it.next();
              foundParent |= vhruri.startsWith(hr);
            }
          }
          break;

        case 17: {
            clear(request);
            request.setMethod("OPTIONS");
            request.setURI(getLocalpart() + testVcr);
            request.expectsResponseDocument(true);
            request.setBody("<options xmlns='DAV:'><version-history-collection-set/></options>");
            response = requester.perform(request);
            checkResponseCode(response, 200);
            assertTrue(response.getDocument() != null);
            assertTrue(DomElement.hasChild(response.getDocument().getDocumentElement(), Consts.N_version_history_collection_set));
          }
          break;

        default:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + testColl);
          response = requester.perform(request);
          checkResponseCode(response, 204);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + testColl + "2");
          response = requester.perform(request);
          checkResponseCode(response, 204);

          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

}
