package com.sapportals.wcm.protocol.webdav.server.tests;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.w3c.dom.Element;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for Versioned Links.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2005
 * @author manfred.baedke@greenbytes.de
 */
public class VersionedLinksTest extends TestMember {

  private static final String[] desc = new String[]
  {
      "00 VL Setting up versioned links test",
      "01 VS create version controlled flexlink",
      "02 VS delete target of version controlled flexlink",
      "   VL cleanup",
  };

  public static final int NrOfTests = desc.length;

  private static boolean supportsFlexLinks;
  
  private static String testCollection = "/versionedlinks"; 
  private static String linkTarget = "/versionedlinks/linktarget"; 
  private static String flexlinkResource = "/versionedlinks/linkresource"; 
  
  private static Set vhrs = new HashSet();

  public VersionedLinksTest(Integer nr) {

    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("VL");

    String respfile = "VERSIONEDLINKS-" + TestMember.TWODIGITS.format(nr) + ".txt";

    try
    {
      IResponse response;
      Element href;
      String content;

      clear(request);
      switch (nr)
      {

        case 0:
          
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + testCollection);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + linkTarget);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          break;

        case 1:

          clear(request);
          request.setMethod("MKRESOURCE");
          request.setURI(getLocalpart() + flexlinkResource);
          request.setBody("<propertyupdate xmlns='DAV:'>" +
                          "<set>" +
                          "<prop>" +
                          "<resourcetype>" + InternalLinksTest.ELEMENT_flex_link + "</resourcetype>" +
                          "<reftarget><href>" + getLocalpart() + linkTarget + "</href></reftarget>" +
                          "</prop>" +
                          "</set>" +
                          "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 201, 501);

          if (response.getStatus() == 201) {

            supportsFlexLinks = true;
          }

          if (supportsFlexLinks) {
            
            clear(request);
            request.setMethod("VERSION-CONTROL");
            request.setHeader("Apply-To-Redirect-Ref", "T");
            request.setURI(getLocalpart() + flexlinkResource);
            response = requester.perform(request);
            checkResponseCode(response, 201);

            String rootVersion = getRelativeUriFromLocationHeader(response);
            Element vhr = getProperty(getLocalpart() + flexlinkResource, Consts.N_version_history);
            if (vhr != null) {
              vhrs.add(getRelativeUriFromPropertyHrefs(vhr));
            }
          }
          
          break;

        case 2:
          
          if (supportsFlexLinks) {

            clear(request);
            request.setMethod("DELETE");
            request.setURI(getLocalpart() + linkTarget);
            response = requester.perform(request);
            checkResponseCode(response, 204);
          }

          break;

        default:

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + flexlinkResource);
          request.setHeader("Apply-To-Redirect-Ref", "T");
          response = requester.perform(request);
          checkResponseCode(response, 204, 404);
  
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + linkTarget);
          response = requester.perform(request);
          checkResponseCode(response, 204, 404);

          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + testCollection);
          response = requester.perform(request);
          checkResponseCode(response, 204);

          Iterator vhrIter = vhrs.iterator();
          while (vhrIter.hasNext()) {

            String vhrUri = (String) vhrIter.next();
            clear(request);
            request.setMethod("DELETE");
            request.setURI(vhrUri);
            response = requester.perform(request);
            checkResponseCode(response, 204, 404);
          }

          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
