package com.sapportals.wcm.protocol.webdav.server.tests;

import com.sapportals.wcm.protocol.webdav.Consts;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for MS webfolder client.
 *
 * <p>
 * Copyright (c) SAP AG 2002-2004
 * @author julian.reschke@greenbytes.de
 * @version $Id: WebFolderTest.java,v 1.4 2004/05/25 16:29:22 jre Exp $
 */

public class WebFolderTest extends TestMember
{
  private static final String[] desc = new String[]
  {
    "00 WF setup (plain resources)",
    "01 WF setup (internal links)",
    "02 WF test namespace",
    "03 WF test namespace (webfolder user agent header)",
    "04 WF copy link",
    "05 WF copy link (apply-to-redirect-ref)",
    "06 WF copy link (webfolder user agent header)",
    "07 WF move link",
    "08 WF move link (apply-to-redirect-ref)",
    "09 WF move link (webfolder user agent header)",
    "10 WF delete link",
    "11 WF delete link (apply-to-redirect-ref)",
    "12 WF delete link (webfolder user agent header)",
    "   WF cleanup",
  };

  private static String userAgent = "Microsoft Data Access Internet Publishing Provider DAV";

  private static String ELEMENT_internal_link = "<" + Consts.N_SAP_internal_link.getLocalName() +
    " xmlns='" + Consts.N_SAP_internal_link.getNamespaceUri() + "'/>";

  public static int NrOfTests = desc.length;

  public WebFolderTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  // private static String locktoken = null;

  protected void runTest()
  {
    if (nr == 0) super.logIdentifier("WF");

    String respfile = "WEBFOLDER-" + TestMember.THREEDIGITS.format(nr) + ".txt";

    try
    {
      IResponse response;
      String container = "/ms-webfolder-tests";

      switch (nr)
      {
        case 0:
          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + container);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("MKCOL");
          request.setURI(getLocalpart() + container + "/folder");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PUT");
          request.setURI(getLocalpart() + container + "/file");
          request.setBody("test content");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 1:
          clear(request);
          request.setMethod("MKRESOURCE");
          request.setURI(getLocalpart() + container + "/collection-link");
          request.setBody ("<propertyupdate xmlns='DAV:'>" +
              "<set>" +
                "<prop>" +
                  "<resourcetype>"+ELEMENT_internal_link+"</resourcetype>" +
                  "<reftarget><href>" + getLocalpart() + container + "/folder" + "/file" + "</href></reftarget>" +
                "</prop>" +
              "</set>" +
            "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("MKRESOURCE");
          request.setURI(getLocalpart() + container + "/file-link");
          request.setBody ("<propertyupdate xmlns='DAV:'>" +
              "<set>" +
                "<prop>" +
                  "<resourcetype>"+ELEMENT_internal_link+"</resourcetype>" +
                  "<reftarget><href>" + getLocalpart() + container + "/file" + "</href></reftarget>" +
                "</prop>" +
              "</set>" +
            "</propertyupdate>");
          response = requester.perform(request);
          checkResponseCode(response, 201);
          break;

        case 2:
          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "1");
          request.setURI(getLocalpart() + container);
          request.setBody("<propfind xmlns='DAV:'><prop><getcontentlength/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 3:
          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "1");
          request.setHeader("User-Agent", userAgent);
          request.setURI(getLocalpart() + container);
          request.setBody("<propfind xmlns='DAV:'><prop><getcontentlength/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 4:
          clear(request);
          request.setMethod("COPY");
          request.setURI(getLocalpart() + container + "/file-link");
          request.setHeader("Destination", getLocalpart() + container + "/copy-of-file-link");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Location") != null);
          break;

        case 5:
          clear(request);
          request.setMethod("COPY");
          request.setURI(getLocalpart() + container + "/file-link");
          request.setHeader("Destination", getLocalpart() + container + "/copy-of-file-link");
          request.setHeader("Apply-To-Redirect-Ref", "");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + container + "/copy-of-file-link");
          request.setBody("<propfind xmlns='DAV:'><prop><getcontentlength/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Location") != null);
          break;

        case 6:
          clear(request);
          request.setMethod("COPY");
          request.setURI(getLocalpart() + container + "/file-link");
          request.setHeader("Destination", getLocalpart() + container + "/copy-of-file-link2");
          request.setHeader("User-Agent", userAgent);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + container + "/copy-of-file-link2");
          request.setHeader("User-Agent", userAgent);
          request.setBody("<propfind xmlns='DAV:'><prop><getcontentlength/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 7:
          clear(request);
          request.setMethod("MOVE");
          request.setURI(getLocalpart() + container + "/file-link");
          request.setHeader("Destination", getLocalpart() + container + "/move-of-file-link");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Location") != null);
          break;

        case 8:
          clear(request);
          request.setMethod("MOVE");
          request.setURI(getLocalpart() + container + "/copy-of-file-link");
          request.setHeader("Destination", getLocalpart() + container + "/move-of-file-link");
          request.setHeader("Apply-To-Redirect-Ref", "");
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + container + "/move-of-file-link");
          request.setBody("<propfind xmlns='DAV:'><prop><getcontentlength/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Location") != null);
          break;

        case 9:
          clear(request);
          request.setMethod("MOVE");
          request.setURI(getLocalpart() + container + "/copy-of-file-link2");
          request.setHeader("Destination", getLocalpart() + container + "/move-of-file-link2");
          request.setHeader("User-Agent", userAgent);
          response = requester.perform(request);
          checkResponseCode(response, 201);

          clear(request);
          request.setMethod("PROPFIND");
          request.setHeader("Depth", "0");
          request.setURI(getLocalpart() + container + "/move-of-file-link2");
          request.setHeader("User-Agent", userAgent);
          request.setBody("<propfind xmlns='DAV:'><prop><getcontentlength/></prop></propfind>");
          response = requester.perform(request);
          checkResponseCode(response, 207);
          checkResponseDocument(response, respfile, null);
          break;

        case 10:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container + "/file-link");
          response = requester.perform(request);
          checkResponseCode(response, 302);
          assertTrue(response.getHeader("Location") != null);
          break;

        case 11:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container + "/move-of-file-link");
          request.setHeader("Apply-To-Redirect-Ref", "");
          response = requester.perform(request);
          checkResponseCode(response, 204);

          // make sure target is still there
          clear(request);
          request.setMethod("HEAD");
          request.setURI(getLocalpart() + container + "/file");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        case 12:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container + "/move-of-file-link2");
          request.setHeader("User-Agent", userAgent);
          response = requester.perform(request);
          checkResponseCode(response, 204);

          // make sure target is still there
          clear(request);
          request.setMethod("HEAD");
          request.setURI(getLocalpart() + container + "/file");
          response = requester.perform(request);
          checkResponseCode(response, 200);
          break;

        default:
          clear(request);
          request.setMethod("DELETE");
          request.setURI(getLocalpart() + container);
          request.setHeader("Apply-To-Redirect-Ref", "");
          response = requester.perform(request);
          checkResponseCode(response, 204, 404);
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }
}
