<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               version="1.0"
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
               xmlns:D="DAV:"
               xmlns:S="http://sapportals.com/xmlns/cm"
               xmlns:dt="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/"
               xmlns:rpt="http://sapportals.com/xmlns/cm/reporting"
               xmlns:wd="http://sapportals.com/xmlns/cm/webdav"
               xmlns:sdt="http://sapportals.com/xmlns/cm/datatypes"
               exclude-result-prefixes="D S xsi dt rpt wd sdt"
>

<xsl:param name="base" />
<xsl:param name="sortResponses" />
<xsl:param name="userid" />
<xsl:param name="userid2" />
<xsl:param name="userbase" />
<xsl:param name="groupbase" />

<xsl:output indent="yes" encoding="iso-8859-1" />

<!-- replace current user id -->
<xsl:template match="D:owner">
  <xsl:if test="text() = $userid">
    <owner xmlns="DAV:">user</owner>
  </xsl:if>
  <xsl:if test="not(text() = $userid)">
    <owner xmlns="DAV:">
      <xsl:apply-templates select="@*" />
      <xsl:apply-templates select="node()" />
    </owner>
  </xsl:if>
</xsl:template>

<xsl:template match="S:principal-URI">
  <xsl:choose>
    <xsl:when test="text() = concat($userbase, $userid)"><principal-URI xmlns="http://sapportals.com/xmlns/cm">/wcm/docs/um/users/user</principal-URI></xsl:when>
    <xsl:otherwise>
      <principal-URI xmlns="http://sapportals.com/xmlns/cm">
        <xsl:apply-templates select="@*" />
        <xsl:apply-templates select="node()" />
      </principal-URI>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- sort the children of the following elements -->
<xsl:template xmlns:wd="http://sapportals.com/xmlns/cm/webdav" match="D:prop|wd:results">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    <xsl:apply-templates select="@*" />
    <xsl:apply-templates select="node()">
      <xsl:sort select="namespace-uri()" />
      <xsl:sort select="local-name()" />
    </xsl:apply-templates>
  </xsl:element>
</xsl:template>

<!-- sort the children of the following elements -->
<xsl:template match="D:operators">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    <xsl:apply-templates select="@*" />
    <xsl:apply-templates select="node()">
      <xsl:sort select="namespace-uri(*[1])" />
      <xsl:sort select="local-name(*[1])" />
    </xsl:apply-templates>
  </xsl:element>
</xsl:template>

<!-- sort the children of the following elements -->
<xsl:template match="D:supported-privilege-set|D:supported-privilege">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    <xsl:apply-templates select="@*" />
    <xsl:apply-templates select="*[not(self::D:supported-privilege)]" />
    <xsl:apply-templates select="D:supported-privilege">
      <xsl:sort select="namespace-uri(D:privilege/*)" />
      <xsl:sort select="local-name(D:privilege/*)" />
    </xsl:apply-templates>
  </xsl:element>
</xsl:template>

<xsl:template match="D:response[D:href!='']">
  <xsl:variable name="duplicates" select="../D:response[D:href=current()/D:href]" />
  <xsl:choose>
    <xsl:when test="count($duplicates)!=1">
      DUPLICATE HREF <xsl:value-of select="D:href" />IN RESPONSE
    </xsl:when>
    <xsl:otherwise>
      <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
        <xsl:apply-templates select="@*" />
        <xsl:apply-templates select="node()">
          <xsl:sort select="namespace-uri()" />
          <xsl:sort select="local-name()" />
          <xsl:sort select="D:status" /> <!-- sort propstat elements by status -->
        </xsl:apply-templates>
      </xsl:element>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="D:response[not(D:href)]">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    <xsl:apply-templates select="@*" />
    <xsl:apply-templates select="node()">
      <xsl:sort select="namespace-uri()" />
      <xsl:sort select="local-name()" />
      <xsl:sort select="D:status" /> <!-- sort propstat elements by status -->
    </xsl:apply-templates>
  </xsl:element>
</xsl:template>

<xsl:template match="D:status">
  <status xmlns="DAV:" xsl:exclude-result-prefixes="D S"><xsl:apply-templates select="*" /><xsl:value-of select="substring(.,0,13)" /></status>
</xsl:template>

<!-- only use this rule if @dt:dt is not present, or it is present and has the correct value -->
<xsl:template match="D:creationdate[node()][not(@dt:dt) or @dt:dt='dateTime.tz']">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789','0000000000')" /></xsl:element>
</xsl:template>

<xsl:template match="D:displayname[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">XXX</xsl:element>
</xsl:template>

<xsl:template match="D:supported-privilege/D:description[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">XXX</xsl:element>
</xsl:template>

<xsl:template match="D:timeout[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789','')" /></xsl:element>
</xsl:template>

<!-- only use this rule if @dt:dt is not present, or it is present and has the correct value -->
<xsl:template match="D:getlastmodified[node()][not(@dt:dt) or @dt:dt='dateTime.rfc1123']">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz','0000000000AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA')" /></xsl:element>
</xsl:template>

<xsl:template match="D:locktoken[node()]|S:created[node()]|S:modified[node()]|D:getetag[node()]|D:getcontenttype[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template match="D:responsedescription">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">
  <xsl:apply-templates select="*" />
  <xsl:if test="text()">...</xsl:if>
</xsl:element>
</xsl:template>

<!-- do not care about etags on collections -->
<xsl:template match="D:prop[D:resourcetype/D:collection]/D:getetag" priority="9"/>

<!-- ignore SAP readonly property -->
<xsl:template match="D:prop/S:readonly" />

<!-- ignore moddav executable property -->
<xsl:template match="moddav:executable" xmlns:moddav="http://apache.org/dav/props/" />

<!-- ignore some SAP live properties -->
<xsl:template match="*[namespace-uri()='http://sapportals.com/xmlns/cm/rendering']" />

<!-- do not report the ability to do shared locks -->
<xsl:template match="D:lockentry[D:lockscope/D:shared]" />

<xsl:template match="D:href[not(local-name(..)='orderingtype') 
                            and not(local-name(../..)='version-history') 
                            and not(local-name(..)='successor-set') 
                            and not(local-name(..)='predecessor-set')]">
<xsl:choose>
  <xsl:when test="contains(text(),$base)"><href xmlns="DAV:">...<xsl:value-of select="substring-after(.,$base)" /></href></xsl:when>
  <xsl:when test="text() = concat($userbase,$userid)"><href xmlns="DAV:">/wcm/docs/um/users/user</href></xsl:when>
  <xsl:when test="text() = concat($userbase,$userid2)"><href xmlns="DAV:">/wcm/docs/um/users/user2</href></xsl:when>
  <xsl:when test="text() = concat($groupbase,'(Everyone)')"><href xmlns="DAV:">/wcm/docs/um/groups/(everybody)</href></xsl:when>
  <xsl:when test="text() = concat($groupbase,'(Everyone)/')"><href xmlns="DAV:">/wcm/docs/um/groups/(everybody)/</href></xsl:when>
  <xsl:when test="text() = concat($groupbase,'(everybody)')"><href xmlns="DAV:">/wcm/docs/um/groups/(everybody)</href></xsl:when>
  <xsl:when test="text() = concat($groupbase,'(everybody)/')"><href xmlns="DAV:">/wcm/docs/um/groups/(everybody)/</href></xsl:when>
  <xsl:otherwise><href xmlns="DAV:"><xsl:value-of select="." /></href></xsl:otherwise>
</xsl:choose>
</xsl:template>

<!-- normalization of version resource HREFs -->

<xsl:template name="generate-version-name">
  <xsl:param name="uri" />
  <xsl:variable name="pred" select="/D:multistatus/D:response[D:propstat/D:prop/D:successor-set/D:href=$uri]" />
  <xsl:choose>
    <xsl:when test="$pred/D:propstat/D:prop/D:successor-set/D:href[3]=$uri">
      <xsl:call-template name="generate-version-name">
        <xsl:with-param name="uri" select="$pred/D:href" />
      </xsl:call-template>
      <xsl:text>.3</xsl:text>
    </xsl:when>
    <xsl:when test="$pred/D:propstat/D:prop/D:successor-set/D:href[2]=$uri">
      <xsl:call-template name="generate-version-name">
        <xsl:with-param name="uri" select="$pred/D:href" />
      </xsl:call-template>
      <xsl:text>.2</xsl:text>
    </xsl:when>
    <xsl:when test="$pred">
      <xsl:call-template name="generate-version-name">
        <xsl:with-param name="uri" select="$pred/D:href" />
      </xsl:call-template>
      <xsl:text>.1</xsl:text>
    </xsl:when>
    <xsl:otherwise>1</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="D:href[../D:propstat[contains(D:status,'200')]/D:prop/D:successor-set]" priority="9">
  <href xmlns="DAV:"><xsl:call-template name="generate-version-name"><xsl:with-param name="uri" select="." /></xsl:call-template></href>
</xsl:template>

<xsl:template match="D:prop/D:successor-set/D:href|D:prop/D:predecessor-set/D:href">
  <href xmlns="DAV:"><xsl:call-template name="generate-version-name"><xsl:with-param name="uri" select="." /></xsl:call-template></href>
</xsl:template>

<!--<xsl:template match="D:version-set/D:response/D:href">
  <href xmlns="DAV:">...entry in version set...</href>
</xsl:template>-->

<xsl:template match="D:version-history/D:response/D:href">
  <href xmlns="DAV:">...name of VHR...</href>
</xsl:template>

<xsl:template match="D:prop/D:version-name">
  <xsl:variable name="uri" select="../../../D:href" />
  <xsl:variable name="vname" select="." />
  <xsl:for-each select="/D:multistatus/D:response[D:href!=$uri and D:propstat[contains(D:status,'200')]/D:prop/D:version-name=$vname]">
    <xsl:comment>DUPLICATE version name <xsl:value-of select="$vname" />, uri: <xsl:value-of select="D:href" /></xsl:comment>
  </xsl:for-each>
  <version-name xmlns="DAV:">name of <xsl:call-template name="generate-version-name"><xsl:with-param name="uri" select="$uri" /></xsl:call-template></version-name>
</xsl:template>



<xsl:template xmlns:ex="http://schemas.microsoft.com/exchange/" match="ex:foldersize|ex:defaultrevision|ex:mid|ex:ntsecuritydescriptor|ex:permanenturl">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template xmlns:ex="http://schemas.microsoft.com/exchange/" match="ex:outlookmessageclass|ex:replicationstyle" />

<xsl:template xmlns:ot="urn:x-opentext-com:ll:properties:" match="ot:*" />

<xsl:template xmlns:ps="publishing:security:" match="ps:*" />

<xsl:template xmlns:rat="http://sapportals.com/xmlns/cm/repository.service.rating.RatingManager"
	xmlns:acc="http://sapportals.com/xmlns/cm/acc_stat"
	xmlns:col="http://sapportals.com/xmlns/cm/collaboration" match="rat:*|acc:*|col:*" />

<xsl:template xmlns:pub="urn:schemas-microsoft-com:publishing:" match="pub:ShadowFolderUrl|pub:IsInternalData|pub:checkedoutby|pub:owner|pub:RolesChangeTime|pub:IsCheckedOut" />

<xsl:template xmlns:mapi="http://schemas.microsoft.com/mapi/proptag/" match="mapi:*" />

<xsl:template xmlns:repl="http://schemas.microsoft.com/repl/" match="repl:contenttag|repl:repl-uid|repl:resourcetag">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template match="D:id[node()]|D:revisionid[node()]|D:revisionlabel[node()]|D:revisionuri[node()]|D:uid[node()]|D:vresource-id[node()]">
	<xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<!-- suppress certain SAP properties -->
<xsl:template match="S:readonly/@xsi:type" />
<xsl:template match="S:enhancedcollection" />
<xsl:template match="S:createdby" />
<xsl:template match="S:modifiedby" />
<xsl:template match="S:description" />
<xsl:template match="S:resourcetype" />
<!--<xsl:template match="D:prop/S:versioning" />
<xsl:template match="D:prop/S:versioned" /> -->

<!-- do not suppress certain SAP properties if they are below rep:properties-->
<xsl:template xmlns:rep="http://sapportals.com/xmlns/cm/reporting" match="rep:properties//*">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:apply-templates select="@*|node()" /></xsl:element>
</xsl:template> 
		
<xsl:template match="D:ishidden/@dt:*"><xsl:copy /></xsl:template>

<xsl:template match="@dt:*" />

<!-- sort response elements if param says so -->
<xsl:template match="/D:multistatus">
  <xsl:choose>
    <xsl:when test="$sortResponses=true()">
      <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
        <xsl:apply-templates select="@*" />
        <xsl:apply-templates select="D:response">
          <xsl:sort select="D:href" />
        </xsl:apply-templates>
        <xsl:apply-templates select="node()[not(self::D:response)]" />
      </xsl:element>
    </xsl:when>
    <xsl:otherwise>
      <xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:apply-templates select="@*|node()" /></xsl:element>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- sort every inside DAV:version-set properties -->
<xsl:template match="D:response/D:propstat/D:prop/D:version-set">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    <xsl:apply-templates select="@*" />
    <xsl:apply-templates select="D:response">
      <xsl:sort select="D:href" />
      <!-- fallback: sort by content length of all property elements -->
      <xsl:sort select="string(D:propstat)" />
    </xsl:apply-templates>
    <xsl:apply-templates select="node()[not(self::D:response)]" />
  </xsl:element>
</xsl:template>

<!-- lockroot in lockdiscovery ignored -->
<xsl:template match="D:lockdiscovery/D:activelock/D:lockroot" />

<!-- things we don't wont to see in sharepoint responses -->

<xsl:template xmlns:p="urn:schemas-microsoft-com:publishing:" xmlns:D="DAV:" xmlns:ex="urn:schemas-microsoft-com:exch-data:"
  match="D:prop/D:href | D:prop/D:parentname |
         D:prop/p:BaseDoc | D:prop/p:WorkspaceName | D:prop/p:LastUnapprovedVersion |
         D:prop/ex:schema-collection-ref"/>

<!-- sort privileges -->

<xsl:template match="D:acl/D:ace/D:grant">
  <grant xmlns="DAV:">
  	<xsl:apply-templates select="D:privilege">
      <xsl:sort select="namespace-uri(*)" /> 
      <xsl:sort select="local-name(*)" /> 
    </xsl:apply-templates>
  	<xsl:apply-templates select="*[not(self::D:privilege)]"/>
  </grant>
</xsl:template>
	
<!-- sort expanded names under properties (report result) -->

<xsl:template match="wd:results/rpt:properties">
  <properties xmlns="http://sapportals.com/xmlns/cm/reporting">
  	<xsl:copy-of select="@*"/>
    <xsl:apply-templates select="sdt:expanded-name">
      <xsl:sort select="."/>
    </xsl:apply-templates>
  </properties>
</xsl:template>
		
<!-- identity transform -->

<xsl:template match="comment()|processing-instruction()|text()|@*"><xsl:copy><xsl:apply-templates select="node()|@*" /></xsl:copy></xsl:template>

<xsl:template match="*">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
	  <xsl:for-each select="@*">
      <xsl:sort select="namespace-uri()" />
      <xsl:sort select="local-name()" />
	    <xsl:apply-templates select="." />
		</xsl:for-each>
  	<xsl:apply-templates select="node()" />
	</xsl:element>
</xsl:template>

<xsl:template match="/">
	<xsl:copy><xsl:apply-templates select="node()" /></xsl:copy>
</xsl:template>

<xsl:template match="text()[normalize-space(.)='']" />

</xsl:transform>
