package com.sapportals.wcm.protocol.webdav.server.webtests;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import com.sapportals.wcm.util.html.HTMLException;
import com.sapportals.wcm.util.html.HTMLFilterImpl;
import com.sapportals.wcm.util.html.HTMLReaderFactory;
import com.sapportals.wcm.util.html.HTMLStreamWriter;
import com.sapportals.wcm.util.html.IHTMLElementStart;
import com.sapportals.wcm.util.html.IHTMLReader;
import com.sapportals.wcm.util.http.ContentType;
import com.sapportals.wcm.util.http.IResponse;

/**
 * Various tests for GET method.
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: WebGetTest.java,v 1.7 2005/05/30 11:58:39 sei Exp $
 */

public class WebGetTest extends TestMember
{
  private static final String[] desc=new String[] {
    "00 GET on /web/www.greenbytes.de",
    "01 GET on /web/www.greenbytes.de/stylesheet.css",
    "02 GET on /web/www.heise.de",
    "03 GET on /web/www.heise.de/newsticker/",
    "04 GET on /web/www.suse.de",
    "05 GET on /webcrawl/arabic.html",
    "06 GET on /webcrawl/china.html",
    "07 GET on /webcrawl/cnn.html",
    "08 GET on /webcrawl/greenbytes.html",
    "09 GET on /webcrawl/japanese.html",
    "10 GET on /webcrawl/selfhtml.html",
    "11 GET on /webcrawl/spiegel.html",
    "12 GET on /webcrawl/thainews.html",
    "13 GET and compare on /webcrawl/arabic.html",
    "14 GET and compare on /webcrawl/arabic.html/",
    "15 GET and compare on /webcrawl/china.html",
    "16 GET and compare on /webcrawl/china.html/",
    "17 GET and compare on /webcrawl/cnn.html",
    "18 GET and compare on /webcrawl/cnn.html/",
    "19 GET and compare on /webcrawl/greenbytes.html",
    "20 GET and compare on /webcrawl/greenbytes.html/",
    "21 GET and compare on /webcrawl/japanese.html",
    "22 GET and compare on /webcrawl/japanese.html/",
    "23 GET and compare on /webcrawl/selfhtml.html",
    "24 GET and compare on /webcrawl/selfhtml.html/",
    "25 GET and compare on /webcrawl/spiegel.html",
    "26 GET and compare on /webcrawl/spiegel.html/",
    "27 GET and compare on /webcrawl/thainews.html",
    "28 GET and compare on /webcrawl/thainews.html/",
    "29 GET and compare on /webcrawl",
    "30 GET and compare on /webcrawl/sql92.html",
    "31 GET and compare on /webcrawl/sap-error1.html",
    "32 GET on /sgb (https://www.greenbytes.de/)",
    "33 GET on /web/cvs.greenbytes.de/SPORT/",
    "34 GET and compare on /webcrawl2/synopsys.html",
    "35 GET and compare on /webcrawl/SPIEGEL.html",
    "36 GET content-length check",
    "37 GET 2nd content-length check",
    "38 HEAD content-length check",
    "39 GET root page with parameter",
  };

  public static int NrOfTests = desc.length;

  public WebGetTest(Integer nr)
  {
    super(nr, desc[nr.intValue()]);
  }

  protected void runTest()
  {
    try
    {
      switch (nr) {
        case 0:
          checkContent("/www/spiegel.de", "text/html");
          checkContent("/web/www.greenbytes.de", "text/html");
          break;
        case 1:
          checkContent("/web/www.greenbytes.de/stylesheet.css", "text/css");
          break;
        case 2:
          checkContent("/web/www.heise.de", "text/html");
          break;
        case 3:
          checkContent("/web/www.heise.de/newsticker/", "text/html");
          break;
        case 4:
          // removed: SuSE change site layout and is returning 302
          //checkContent("/web/www.suse.de", "text/html");
          break;
        case 5:
          checkContent("/webcrawl/arabic.html", "text/html");
          break;
        case 6:
          checkContent("/webcrawl/china.html", "text/html");
          break;
        case 7:
          checkContent("/webcrawl/cnn.html", "text/html");
          break;
        case 8:
          checkContent("/webcrawl/greenbytes.html", "text/html");
          break;
        case 9:
          checkContent("/webcrawl/japanese.html", "text/html");
          break;
        case 10:
          checkContent("/webcrawl/selfhtml.html", "text/html");
          break;
        case 11:
          checkContent("/webcrawl/spiegel.html", "text/html");
          break;
        case 12:
          checkContent("/webcrawl/thainews.html", "text/html");
          break;
        case 13:
          checkResult("/webcrawl/arabic.html", "arabic.html");
          break;
        case 14:
          checkResult("/webcrawl/arabic.html/", "arabic2.html");
          break;
        case 15:
          checkResult("/webcrawl/china.html", "china.html");
          break;
        case 16:
          checkResult("/webcrawl/china.html/", "china2.html");
          break;
        case 17:
          checkResult("/webcrawl/cnn.html", "cnn.html");
          break;
        case 18:
          checkResult("/webcrawl/cnn.html/", "cnn2.html");
          break;
        case 19:
          checkResult("/webcrawl/greenbytes.html", "greenbytes.html");
          break;
        case 20:
          checkResult("/webcrawl/greenbytes.html/", "greenbytes2.html");
          break;
        case 21:
          checkResult("/webcrawl/japanese.html", "japanese.html");
          break;
        case 22:
          checkResult("/webcrawl/japanese.html/", "japanese2.html");
          break;
        case 23:
          checkResult("/webcrawl/selfhtml.html", "selfhtml.html");
          break;
        case 24:
          checkResult("/webcrawl/selfhtml.html/", "selfhtml2.html");
          break;
        case 25:
          checkResult("/webcrawl/spiegel.html", "spiegel.html");
          break;
        case 26:
          checkResult("/webcrawl/spiegel.html/", "spiegel2.html");
          break;
        case 27:
          checkResult("/webcrawl/thainews.html", "thainews.html");
          break;
        case 28:
          checkResult("/webcrawl/thainews.html/", "thainews2.html");
          break;
        case 29:
          checkContent("/webcrawl", "text/html");
          break;
        case 30:
          checkResult("/webcrawl/sql92.html", "sql92.html");
          break;
        case 31:
          checkResult("/webcrawl/sap-error1.html", "sap-error1.html");
          break;
        case 32:
          checkContent("/sgb/", "text/html");
          break;
        case 33:
          clear(m_request);
          m_request.setMethod("GET");
          m_request.setURI(m_localpart + "/web/cvs.greenbytes.de/SPORT/");
          IResponse response = m_requester.perform( m_request );
          checkResponseCode(response, 404);
          break;
        case 34:
          checkResult("/webcrawl2/synopsys.html", "synopsys.html");
          break;
        case 35:
          response = checkResult("/webcrawl/SPIEGEL.html", "spiegel3.html");
          String tmp = response.getHeader("Content-Location");
          if (tmp != null) {
            assertTrue("unexpected content-location header, expected something ending in"
                +" .../webcrawl/spiegel.html, got " + tmp,
                tmp.endsWith("/webcrawl/spiegel.html"));
          }
          break;
        case 36:
          clear(m_request);
          m_request.setMethod("GET");
          m_request.setURI(m_localpart + "/webcrawl/servlet2_2.pdf");
          response = m_requester.perform(m_request);
          checkResponseCode(response, 200);
          tmp = response.getHeader("Content-Length");
          assertTrue("content-length header missing", tmp != null);
          System.out.print("{len: "+tmp+"}");
          break;
        case 37:
          clear(m_request);
          m_request.setMethod("GET");
          m_request.setURI(m_localpart + "/webcrawl/servlet2_2.pdf");
          response = m_requester.perform(m_request);
          checkResponseCode(response, 200);
          tmp = response.getHeader("Content-Length");
          assertTrue("content-length header missing", tmp != null);
          System.out.print("{len: "+tmp+"}");
          break;
        case 38:
          clear(m_request);
          m_request.setMethod("HEAD");
          m_request.setURI(m_localpart + "/webcrawl/servlet2_2.pdf");
          response = m_requester.perform(m_request);
          checkResponseCode(response, 200);
          tmp = response.getHeader("Content-Length");
          assertTrue("content-length header missing", tmp != null);
          System.out.print("{len: "+tmp+"}");
          break;
        case 39:
          clear(m_request);
          m_request.setMethod("GET");
          m_request.setURI(m_localpart + "/web/cvs.greenbytes.de?test=true&Whatever");
          response = m_requester.perform( m_request );
          checkResponseCode(response, 200, 404);
          break;
      }
    }
    catch (Exception ex) {
      java.io.StringWriter sw = new java.io.StringWriter();
      ex.printStackTrace(new java.io.PrintWriter(sw));
      fail(sw.getBuffer().toString());
    }
  }

  private void checkContent(String uri, String expectedType) throws Exception {
    checkContent(uri, expectedType, -1);
  }

  private void checkContent(String uri, String expectedType, int expectedSize) throws Exception {
    clear(m_request);
    m_request.setMethod("GET");
    m_request.setURI(m_localpart + uri);
    m_request.expectsResponseStream(true);
    IResponse response = m_requester.perform( m_request );
    checkResponseCode(response, 200);

    String ctype = response.getHeader("Content-Type");
    assertTrue("unexpected content-type: "+ctype,
               ctype != null && ctype.startsWith(expectedType));

    int len = length(response.getStream());
    if (expectedSize >= 0) {
      assertTrue("unexpected length: "+len, len == expectedSize);
    }
    else {
      assertTrue("unexpected length: "+len, len > 0);
    }
  }

  private IResponse checkResult(String uri, String reference) throws Exception {
    clear(m_request);
    m_request.setMethod("GET");
    m_request.setURI(m_localpart + uri);
    m_request.expectsResponseStream(true);
    IResponse response = m_requester.perform( m_request );
    checkResponseCode(response, 200);
    byte[] content = getNormalizedContent(response);
    checkResponseContent(content, reference);
    return response;
  }

  private int length(InputStream is) throws IOException {
    if (is == null)
      return 0;

    byte[] buffer = new byte[64 * 1024];
    int len;
    int size = 0;
    while ((len = is.read(buffer)) != -1) {
      size += len;
      // nop
    }
    return size;
  }

  private byte[] getNormalizedContent(IResponse response) throws Exception  {
    String host = this.m_base;
    if (host.startsWith("http://")) {
      host = host.substring("http://".length());
    }
    int index = host.indexOf('/');
    if (index > 0) {
      host = host.substring(0, index);
    }
    
    String subst[] = new String[] { host+this.m_localpart, host};
    String repl[] = new String[] { "localhost:port/prefix", "localhost:port"};
    
    IHTMLReader reader = HTMLReaderFactory.createHTMLReader();
    ContentType ct = ContentType.valueOf(response.getHeader("Content-Type"));
    if (ct.getEncoding() != null) {
      reader.setSource(response.getStream(), ct.getEncoding());
    }
    else {
      reader.setSource(response.getStream());
    }
    reader = new LinkNormalizer(reader, subst, repl);
    ByteArrayOutputStream bos = new ByteArrayOutputStream(64 * 1024);
    reader.setContentHandler(new HTMLStreamWriter(bos, reader.getEncoding()));
    reader.parse();
    return bos.toByteArray();
  }
  
  private final class LinkNormalizer extends HTMLFilterImpl {
    private final String subst[];
    private final String repl[];
    private final StringBuffer sb;
    public LinkNormalizer(IHTMLReader reader, String subst[], String repl[]) {
      super(reader);
      this.subst = subst;
      this.repl = repl;
      this.sb = new StringBuffer(256);
    }
    
    
		public void startElement(IHTMLElementStart element) throws HTMLException {
      replaceAttribute(element, "href");
      replaceAttribute(element, "src");
      replaceAttribute(element, "background");
      replaceAttribute(element, "action");
			super.startElement(element);
		}
    
    private void replaceAttribute(IHTMLElementStart element, String attribute) {
      int index = element.getIndexOf(attribute);
      if (index >= 0) {
        String href = element.getValueOf(index);
        for (int i = 0; i < this.subst.length; ++i) {
          int ix = href.indexOf(this.subst[i]);
          if (ix >= 0) {
            this.sb.setLength(0);
            this.sb.append(href.substring(0, ix));
            this.sb.append(this.repl[i]);
            ix += this.subst[i].length();
            if (ix < href.length()) {
              this.sb.append(href.substring(ix));
            }
            element.setValueOf(index, sb.toString());
            break;
          }
        }
      }
    }
  }
}
