package com.sapportals.wcm.protocol.webdav.server.webtests;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.sapportals.wcm.util.http.*;

/**
 * Generates various tests for a certain class.
 *
 * <p>
 * Copyright (c) SAP AG 2001-2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: WebTestSuite.java,v 1.1.1.1 2004/02/18 12:58:55 sync Exp $
 */

public class WebTestSuite extends TestCase
{
  private static String m_host = null;

  // base collection for tests
  private static String m_base = null;

  private static IRequesterFactory m_factory = null;
  private static IRequester m_requester = null;
  private static IContext m_context = null;

  // private static String m_sourcePath;
  private static boolean m_first = true;

  private static String m_user;
  private static String m_password;

  static {
    m_host = System.getProperty ("host", "http://localhost:8080");
    m_base = System.getProperty ("base", "/wcm/docs");

    m_user = System.getProperty("user", "user");
    m_password = System.getProperty("password", "user");

    try{
      m_factory = RequesterFactory.newInstance();
      m_factory.setBaseURL(m_host);
      m_context = m_factory.getContext("user");

      if (m_context.isNew()) {
        m_context.setUserInfo(UserInfo.valueOf(m_user, m_password));
      }
      m_requester = m_factory.getRequester(m_context);
    }catch (Throwable ex) {
      ex.printStackTrace(System.err);
    }
  }

  public WebTestSuite(String name)
  {
    super(name);
  }

  protected void setUp()
    throws Exception
  {
    SetUp();
  }

  public static void SetUp() throws Exception
  {
    if (m_first) {
      m_first = false;

      WebHelper wd = new WebHelper(m_requester, m_host, m_base + "/test");
      wd.setup();
    }
  }

  public static TestSuite suite(Class memberClass, String name)
  {
    if (!TestMember.class.isAssignableFrom(memberClass))
      throw new IllegalArgumentException("wrong class: "+memberClass);

    TestSuite suite = new TestSuite(name);
    suite.addTest(new WebTestSuite("Setting up..."));

    try{
      Class[] consArgs = new Class[] {Integer.class};
      Constructor cons = memberClass.getConstructor(consArgs);
      Field field = memberClass.getField("NrOfTests");
      int maxTests = field.getInt(null);
      for (int i=0; i<maxTests; i++)
      {
        Object[] args = new Object[] {new Integer(i)};
        TestMember instance = (TestMember)cons.newInstance(args);
        instance.init(m_factory, m_requester, m_host, m_base, m_user);
        suite.addTest(instance);
      }
    }catch (Exception ex) {
      ex.printStackTrace(System.err);
    }
    return suite;
  }

  public static IRequester getRequester() {
    return m_requester;
  }

  public static String getHost() {
    return m_host;
  }

  public static String getBaseURI() {
    return m_base;
  }

  protected void runTest()
  {
  }
}
