<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               version="1.0"
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
               xmlns:D="DAV:"
               xmlns:rnd="http://sapportals.com/xmlns/cm/rendering"
               xmlns:S="http://sapportals.com/xmlns/cm"
               xmlns:dt="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/"
               exclude-result-prefixes="D S xsi dt rnd"
>

<xsl:param name="base" />
<xsl:param name="extbase" />
<xsl:param name="extprefix" />
<xsl:param name="sortResponses" />

<xsl:output indent="yes" encoding="iso-8859-1" />

<!-- sort the children of the following elements -->
<xsl:template match="D:prop|D:response">
	<xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    	<xsl:apply-templates select="@*" />
        <xsl:apply-templates select="node()">
        	<xsl:sort select="namespace-uri()" />
        	<xsl:sort select="local-name()" />            
        </xsl:apply-templates>
    </xsl:element>
</xsl:template>

<!-- sort hrefs in embedded-links -->
<xsl:template match="S:embedded-links">
	<xsl:element namespace="{namespace-uri()}" name="{local-name()}">
    	<xsl:apply-templates select="@*" />
        <xsl:apply-templates select="node()">
        	<xsl:sort select="node()" />
        </xsl:apply-templates>
    </xsl:element>
</xsl:template>

<xsl:template match="D:status">
	<status xmlns="DAV:" xsl:exclude-result-prefixes="D S rnd"><xsl:value-of select="substring(.,0,13)" /></status>
</xsl:template>

<!-- only use this rule if @dt:dt is not present, or it is present and has the correct value -->
<xsl:template match="D:creationdate[node()][not(@dt:dt) or @dt:dt='dateTime.tz']">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789','0000000000')" /></xsl:element>
</xsl:template>

<xsl:template match="D:displayname[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">XXX</xsl:element>
</xsl:template>

<xsl:template match="D:supported-privilege/D:description[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">XXX</xsl:element>
</xsl:template>

<xsl:template match="D:timeout[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789','')" /></xsl:element>
</xsl:template>

<xsl:template match="D:getcontentlength[node()]">
<!--<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789','')" /></xsl:element>
--></xsl:template>

<!-- only use this rule if @dt:dt is not present, or it is present and has the correct value -->
<xsl:template match="D:getlastmodified[node()][not(@dt:dt) or @dt:dt='dateTime.rfc1123']">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz','0000000000AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA')" /></xsl:element>
</xsl:template>

<xsl:template match="S:last-update[node()][not(@dt:dt) or @dt:dt='dateTime.rfc1123']">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:value-of select="translate(.,'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz','0000000000AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA')" /></xsl:element>
</xsl:template>

<xsl:template match="D:locktoken[node()]|S:created[node()]|S:modified[node()]|D:getetag[node()]|D:responsedescription[node()]|D:getcontenttype[node()]">
<xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template match="D:href[not(local-name(..)='orderingtype')]">
  <xsl:choose>
    <xsl:when test="contains(text(),$extprefix)"><href xmlns="DAV:">.../extbase/prefix<xsl:value-of select="substring-after(.,$extprefix)" /></href></xsl:when>
    <xsl:when test="contains(text(),$extbase)"><href xmlns="DAV:">.../extbase<xsl:value-of select="substring-after(.,$extbase)" /></href></xsl:when>
    <xsl:when test="contains(text(),$base)"><href xmlns="DAV:">...<xsl:value-of select="substring-after(.,$base)" /></href></xsl:when>
    <xsl:otherwise><href xmlns="DAV:"><xsl:value-of select="." /></href></xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="rnd:contentlink">
<xsl:if test="contains(text(),$base)"><contentlink xmlns="http://sapportals.com/xmlns/cm/rendering">...<xsl:value-of select="substring-after(.,$base)" /></contentlink></xsl:if>
<xsl:if test="not(contains(text(),$base))"><contentlink xmlns="http://sapportals.com/xmlns/cm/rendering"><xsl:value-of select="." /></contentlink></xsl:if>
</xsl:template>

<xsl:template match="*[namespace-uri(.)='http://schemas.microsoft.com/exchange/' 
                                         and (local-name(.)='foldersize' 
                                           or local-name(.)='defaultrevision'
                                           or local-name(.)='mid'
                                           or local-name(.)='ntsecuritydescriptor'
                                           or local-name(.)='permanenturl')]">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template match="*[namespace-uri(.)='urn:x-opentext-com:ll:properties:']">
</xsl:template>

<xsl:template match="*[namespace-uri(.)='publishing:security:']">
</xsl:template>

<xsl:template match="*[namespace-uri(.)='http://sapportals.com/xmlns/cm/repository.service.rating.RatingManager'
                       or namespace-uri(.)='http://sapportals.com/xmlns/cm/acc_stat']">
</xsl:template>

<xsl:template match="*[namespace-uri(.)='urn:schemas-microsoft-com:publishing:'
                       and (local-name(.)='ShadowFolderUrl' 
			or  local-name(.)='IsInternalData'
			or  local-name(.)='checkedoutby'
			or  local-name(.)='owner'
			or  local-name(.)='RolesChangeTime'
			or  local-name(.)='IsCheckedOut')]">
</xsl:template>
 
<xsl:template match="*[namespace-uri(.)='http://schemas.microsoft.com/exchange/'
                       and (local-name(.)='outlookmessageclass'
                            or local-name(.)='replicationstyle')]">
</xsl:template>

<xsl:template match="*[namespace-uri(.)='http://schemas.microsoft.com/mapi/proptag/']">
</xsl:template>

<xsl:template match="*[namespace-uri(.)='http://schemas.microsoft.com/repl/' 
                                         and (local-name(.)='contenttag' 
                                           or local-name(.)='repl-uid'
                                           or local-name(.)='resourcetag')]">
  <xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template match="D:id[node()]|D:revisionid[node()]|D:revisionlabel[node()]|D:revisionuri[node()]|D:uid[node()]|D:vresourceid[node()]">
	<xsl:element namespace="{namespace-uri()}" name="{local-name()}">...</xsl:element>
</xsl:template>

<xsl:template match="S:readonly/@xsi:type" />
<xsl:template match="S:versioning" />
<xsl:template match="S:versioned" />
<xsl:template match="S:enhancedcollection" />
<xsl:template match="S:createdby" />
<xsl:template match="S:modifiedby" />
<xsl:template match="S:description" />
<xsl:template match="S:resourcetype" />
<xsl:template match="S:config-hash" />

<xsl:template match="comment()|processing-instruction()|text()|@*"><xsl:copy><xsl:apply-templates select="node()|@*" /></xsl:copy></xsl:template>

<xsl:template match="D:ishidden/@dt:*"><xsl:copy /></xsl:template>

<xsl:template match="@dt:*" />

<!-- sort response elements if param says so -->
<xsl:template match="/D:multistatus">
  <xsl:choose>
    <xsl:when test="$sortResponses=true()">
      <xsl:element namespace="{namespace-uri()}" name="{local-name()}">
        <xsl:apply-templates select="@*" />
        <xsl:apply-templates select="D:response">
          <xsl:sort select="D:href" />
        </xsl:apply-templates>
        <xsl:apply-templates select="node()[not(self::D:response)]" />
      </xsl:element>
    </xsl:when>
    <xsl:otherwise>
      <xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:apply-templates select="@*|node()" /></xsl:element>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- identity transform -->

<xsl:template match="*">
	<xsl:element namespace="{namespace-uri()}" name="{local-name()}"><xsl:apply-templates select="@*|node()" /></xsl:element>
</xsl:template>

<xsl:template match="/">
	<xsl:copy><xsl:apply-templates select="node()" /></xsl:copy>
</xsl:template>

<xsl:template match="text()[normalize-space(.)='']" />

</xsl:transform>
