package com.sapportals.wcm.protocol.webdav.sets;

/**
 * Set of Java ints.
 * <p>
 * Copyright (c) SAP AG 2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: IntSet.java,v 1.1.1.1 2004/02/18 13:00:23 sync Exp $
 */
public interface IntSet {
  
  /**
   * Return if set contains i.
   * @param i to check
   * @return if set contains i
   */
	public abstract boolean contains(int i);
  
  /**
   * Add i to the set
   * @param i to add to set
   * @return if i was added first time
   */
	public abstract boolean add(int i);
  
  /**
   * Remove i from the set.
   * @param i to remove from set
   * @return if i was member
   */
	public abstract boolean remove(int i);
  
	/**
	 * Adds all integers [n, m[, starting from n up to,
	 * but not including m to the set
	 * @param n lower bound to add
	 * @param m upper bound (exclusive) to add
	 */
	public abstract void addInterval(int n, int m);
  
	/**
	 * Removes all integers [n, m[, starting from n up to,
	 * but not including m from the set
	 * @param n lower bound to remove
	 * @param m upper bound (exclusive) to remove
	 */
	public abstract void removeInterval(int n, int m);
  
  /**
   * Return number of elements in set
   * @return set size
   */
	public abstract int size();
  
  /**
   * Return if set is empty
   * @return if set is empty
   */
	public abstract boolean isEmpty();
  
  /**
   * Iterators over all int in set 
   * @return iterator over all int values
   */	
  public abstract IntIterator iterator();
  
  /**
   * Remove all members of the set.
   */
	public abstract void clear();
}