package com.sapportals.wcm.protocol.webdav.sets;

/**
 * Set of Java ints, optimized for sparse sets.
 * <p>
 * Copyright (c) SAP AG 2003
 * @author stefan.eissing@greenbytes.de
 * @version $Id: SparseIntSet.java,v 1.1.1.1 2004/02/18 13:00:23 sync Exp $
 */
public class SparseIntSet implements IntSet {
  
  public static SparseIntSet getInstance() {
    return new SparseIntSet();
  }

  private IntSet set;
  
  private SparseIntSet() {
    this.set = ArrayIntSet.getInstance();
  }
  
	public boolean add(int i) {
    boolean retval = this.set.add(i);
    optimize();
    return retval;
	}

	public void addInterval(int n, int m) {
    for (int i = n; i < m; ++i) {
      this.add(i);
    }
	}

	public void clear() {
    this.set.clear();
	}

	public boolean contains(int i) {
    return this.set.contains(i);
	}

	public boolean isEmpty() {
    return this.set.isEmpty();
	}

	public IntIterator iterator() {
    return this.set.iterator();
	}

	public boolean remove(int i) {
    boolean retval = this.set.remove(i);
    optimize();
    return retval;
	}

	public void removeInterval(int n, int m) {
    for (int i = n; i < m; ++i) {
      this.remove(i);
    }
	}

	public int size() {
    return this.set.size();
	}

  public String toString() {
    return "SparseIntSet["+this.set+"]";
  }
  
  private void optimize() {
    if (this.set instanceof ArrayIntSet) {
      if (((ArrayIntSet)this.set).getStorageEfficiency() < 0.4f) {
        this.set = IntervalIntSet.getInstance(this.set);
      }
    }
    else if (this.set instanceof IntervalIntSet) {
      if (((IntervalIntSet)this.set).getStorageEfficiency() < 0.4f) {
        this.set = ArrayIntSet.getInstance(this.set);
      }
    }
  }
}
