/*
 * Copyright (c) 2003 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: EventSenderThread.java,v 1.2 2005/10/11 09:58:23 sei Exp $
 */

package com.sapportals.wcm.util.events;

import com.sap.tc.logging.Location;
import com.sapportals.wcm.util.logging.LoggingFormatter;

/**
 * Consumes events from the queue an calls the receivers received() method.
 */
public class EventSenderThread extends Thread {

  private static Location log = Location.getLocation(EventSenderThread.class);

  private boolean stop;
  private EventQueue queue;
  private IEventReceiver receiver;

  EventSenderThread(EventQueue queue, IEventReceiver receiver) {
    this.queue = queue;
    this.receiver = receiver;
    this.setName("KM RF Event Sender - receiver: " + receiver.getClass().getName());
    this.start();
  }

  /**
   * Do the work
   */
  public void run() {
    while (!terminated()) {
      if (this.receiver instanceof IEventReceiverBulk) {
        IEventReceiverBulk rec = (IEventReceiverBulk) this.receiver;
        int waitTime = rec.getWaitTime();
        int maxSize = rec.getMaxNumberOfEvents();

        // enforce resonable values
        if (maxSize <= 1)
          maxSize = 2;
        if (waitTime < 1000)
          waitTime = 1000;

        IEvent[] events = this.queue.dequeue(maxSize, waitTime);
        if (events.length > 0) {
          if (EventSenderThread.log.beDebug()) {
            EventSenderThread.log.debugT(
              "Calling received() method of receiver=" + rec.getClass().getName() + ", events=" + events.toString());
          }
          try {
            rec.received(events);
          }
          catch (Throwable t) {
            EventSenderThread.log.errorT(
              "Caught Throwable calling received() method of receiver: "
                + this.receiver.getClass().getName()
                + ": "
                + LoggingFormatter.extractCallstack(t));
          }
        }
      }
      else {
        IEvent event = this.queue.dequeue();
        if (event != null) {
          if (EventSenderThread.log.beDebug()) {
            EventSenderThread.log.debugT(
              "calling received() method of async. receiver="
                + this.receiver.getClass().getName()
                + ", event="
                + event);
          }
          long start = System.currentTimeMillis();
          try {
            this.receiver.received(event);
          }
          catch (Throwable t) {
            EventSenderThread.log.errorT("Caught Exception in event receiver: " + LoggingFormatter.extractCallstack(t));
          }
        }
      }
    }
    log.debugT("Terminated");
  }

  /**
   * Will cause the thread to stop (if it is not waiting on a empty queue)
   */
  void terminate() {
    this.stop = true;
    log.debugT("terminate() was called");
  }

  private boolean terminated() {
    return this.stop;
  }
}
