/*
 * Copyright (c) 2003 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: IEventReceiverBulk.java,v 1.1 2005/10/11 09:58:23 sei Exp $
 */
package com.sapportals.wcm.util.events;


/**
 * Defines an asynchronous event receiver that is able to handle bulk resource events.
 * If this receiver is registered for asynchronous events the receive() method
 * of it's base interface {@link IEventReceiver#received(IEvent)} is never called.
 * If it is registered for synchronous events however, only the
 * this received() method for single events will be called.
 *  
 * @author d032974
 */
public interface IEventReceiverBulk extends IEventReceiver {
  /**
   * Returns the maximum number of events to receive in a single call of the received() method
   * (that is the max. size of the event array). The returned number must be > 1.
   * @return 
   */
  public int getMaxNumberOfEvents();
  
  /**
   * Returns the amount of time in milliseconds to wait for the max. number {@link #getMaxNumberOfEvents()} of events to be reached.
   * The returned number should be > 1000. 
   * @return 
   */
  public int getWaitTime();
  
  /**
   * This method will be called by the event broker with an array of events.
   * The size of the array can be smaller than the MaxNumberOfEvents value if the WaitTime
   * has elapsed before the number of queued events has reached MaxNumberOfEvents.
   * But the size of the array will always be > 0 (if the queue is empty after the
   * WaitTime has elapsed this mehtod is not called).
   * @param events
   */
  public void received(IEvent[] events);  
}
