/*
 * Copyright (c) 2003 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: IEventSender.java,v 1.1.1.1 2004/02/18 12:56:46 sync Exp $
 */

package com.sapportals.wcm.util.events;

/**
 * This interface defines an event sender which produces (instanciates) events.
 * Event senders must be registered at a {@link IEventBroker}. After successfull
 * registration this sender instance is allowed to use the broker's {@link
 * IEventBroker#send} method. <p>
 *
 * Copyright (c) SAP AG 2001-2002
 *
 * @author Markus Breitenfelder
 * @version $Id: //javabas/com.sapportals.wcm/dev/src/java/util/api/com/sapportals/wcm/util/events/IEventSender.java#6
 *      $
 * @review-reader Paul Goetz
 * @review-date 10.01.2002
 */
public interface IEventSender {

  /**
   * Returns a list of event templates for all possible events of this sender.
   * The event instances contained in the list are "templates", that means the
   * event parameter is not relevant. The broker will call it to build a list of
   * all possible events of all registered event senders.
   *
   * @return a list of event templates for all possible events of this sender.
   */
  public IEventList getEvents();

}
