/*
 * Copyright (c) 2003 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: IconTable.java,v 1.1.1.1 2004/02/18 12:56:41 sync Exp $
 */

package com.sapportals.wcm.util.icons;
import com.sap.tc.logging.Location;

import com.sapportals.config.fwk.*;

import com.sapportals.wcm.*;
import com.sapportals.wcm.util.config.*;
import com.sapportals.wcm.util.logging.LoggingFormatter;
import com.sapportals.wcm.util.uri.URI;

import java.util.*;

/**
 * TBD: Description of the class.
 */
public class IconTable {

  private final static String CFG_ICONS_MIMETYPE_KEY = "mimetype";
  private final static String CFG_ICONS_IMAGE_KEY = "image";
  private final static String DEFAULT_ICON = "unknown.gif";

  private static com.sap.tc.logging.Location s_log = com.sap.tc.logging.Location.getLocation(com.sapportals.wcm.util.icons.IconTable.class);
  private static Properties s_icons = null;

  static {
    try {
      IConfigurable[] configurables = ConfigurationProxy.getInstance().getPluginConfigurables(ConfigurationProxy.CFG_PLUGIN_CM_UTILITIES_ICONS);
      s_icons = new Properties();
      for (int i = 0; i < configurables.length; i++) {
        Properties properties = ConfigurationProxy.getConfigurableProperties(configurables[i]);
        String mimetype = properties.getProperty(CFG_ICONS_MIMETYPE_KEY);
        String image = properties.getProperty(CFG_ICONS_IMAGE_KEY);
        if (mimetype != null && image != null) {
          if (s_log.beDebug()) {
            s_log.debugT("<cinit>(50)", "icon " + mimetype + " - " + image);
          }
          s_icons.put(mimetype, image);
        }
      }
    }
    catch (Exception ex) {
      ex.printStackTrace(System.out);
    }
  }

  private IconTable() { }

  public static Properties getIcons() {
    return s_icons;
  }

  public static String getIconForMimetype(String mimetype) {
    if (s_icons == null) {
      return DEFAULT_ICON;
    }
    String icon = s_icons.getProperty(mimetype.toLowerCase());
    if (icon != null && icon.length() > 0) {
      return icon.trim();
    }
    else {
      return DEFAULT_ICON;
    }
  }
}
