package com.sapportals.wcm.util.jdbc.idmapper;

import java.sql.Connection;
import java.sql.SQLException;

import com.sapportals.wcm.util.jdbc.connectionpool.JDBCConnectionPool;
import com.sapportals.wcm.util.logging.LoggingFormatter;

/**
 * @param externalID             the external ID
 * @param pool                   @todo: Description of the incoming method parameter
 * @return                       the constant internal ID for an external ID (null if mapping doesn't exists)
 * @exception IDMapperException  a database error occurred
 */
class IDDeadlockHandler {
  /*
  code 1:
  String result = getConstInternalID ( connection, externalID ) ;

  replacement 1: 
  String result = null;
  IDDeadlockHandler deadlockHandler = new IDDeadlockHandler(pool, connection);
  IDMapperException exception;
  do {
    try {
      exception = null;
      //indirect jdbc method throwing IDMapperException
      result = aLocalMethod(connection);
    }
    catch (IDMapperException exception2) {
      exception = exception2;
    }
  } while (deadlockHandler.isTransientDeadlock(exception));


  code 2:
  ps = c.prepareStatement ( PS_REMOVE_MULTIPLE_ID_MAPPINGS2 ) ;
  ps.setString ( PS_REM_MUL2_EXT_CI, je.s + "/%" ) ;
  
  replacement 2:
  IDDeadlockHandler deadlockHandler = new IDDeadlockHandler(pool, connection);
  SQLException exception;
  do {
    try {
      exception = null;
      if (ps != null) {
        //retry handling; the connection has already been rolled back!
        try {
          ps.close();
        }
        catch (SQLException ignored) {
          //$JLIN-EXC
          //forced close of statement as we're retrying with a new statement
        }
      }
      // direct jdbc methods
      ps = c.prepareStatement ( PS_REMOVE_MULTIPLE_ID_MAPPINGS2 ) ;
      ps.setString ( PS_REM_MUL2_EXT_CI, je.s + "/%" ) ;
    }
    catch (SQLException exception2) {
      exception = exception2;
    }
  } while (deadlockHandler.isTransientDeadlock(exception));

  */

  
  private static com.sap.tc.logging.Location log = com.sap.tc.logging.Location
    .getLocation(IDDeadlockHandler.class.getName());
  private int retryCounter = 20;
  private JDBCConnectionPool pool;
  private Connection connection;
  
  IDDeadlockHandler(JDBCConnectionPool pool, Connection connection) {
    this.pool = pool;
    this.connection = connection;
  }
  
  boolean isTransientDeadlock(IDMapperException exception) throws IDMapperException {
    if (exception == null) {
      return false;
    }
    Throwable throwable = exception.getContainedException();
    /*
    * if this is an SQL exception and if it shows the typical signs of a
    * deadlock and if we haven't tried too many times yet then let's return
    * true so that the calling code can loop around a retry block
    * 
    * if any of the conditions isn't met then rethrow the exception so that
    * the calling code can run business as usual
    */
    if (throwable instanceof SQLException
    && pool.isDeadlock((SQLException)throwable)
    && retryCounter-- > 0) {
      if (log.beDebug()) {
        log.debugT("deadlock encountered; " + retryCounter + " retries left");
      }
      try {
        //play it again, Sam.
        connection.rollback();
      }
      catch (SQLException e) {
        log.warningT("rollback failed: " + LoggingFormatter.extractCallstack(e));
        //gush, she's pumping mud; let's get outta here.
        throw exception;
      }
      //waita bit; 75ms on average with a 50ms variation
      long wait = 25 + (long) (Math.random() * 100.0);
      try {
        Thread.sleep(wait);
      }
      catch (InterruptedException e) {
        //don't mind if we haven't gotten our full nap
      }
      return true;
    }
    throw exception;
  }
  
  boolean isTransientDeadlock(SQLException exception) throws SQLException {
    if (exception == null) {
      return false;
    }
    if (pool.isDeadlock(exception) && retryCounter-- > 0) {
      if (log.beDebug()) {
        log.debugT("deadlock encountered; " + retryCounter + " retries left");
      }
      try {
        //play it again, Sam.
        connection.rollback();
      }
      catch (SQLException e) {
        log.warningT("rollback failed: " + LoggingFormatter.extractCallstack(e));
        //gush, she's pumping mud; let's get outta here.
        throw exception;
      }
      //waita bit; 75ms on average with a 50ms variation
      long wait = 25 + (long) (Math.random() * 100.0);
      try {
        Thread.sleep(wait);
      }
      catch (InterruptedException e) {
        //don't mind if we haven't gotten our full nap
      }
      return true;
    }
    throw exception;
  }
}