/*
 * Copyright (c) 2003 by SAP AG. All Rights Reserved.
 *
 * SAP, mySAP, mySAP.com and other SAP products and
 * services mentioned herein as well as their respective
 * logos are trademarks or registered trademarks of
 * SAP AG in Germany and in several other countries all
 * over the world. MarketSet and Enterprise Buyer are
 * jointly owned trademarks of SAP AG and Commerce One.
 * All other product and service names mentioned are
 * trademarks of their respective companies.
 *
 * @version $Id: ResourceBundles.java,v 1.1.1.1 2004/02/18 12:56:48 sync Exp $
 */

package com.sapportals.wcm.util.resource;

import com.sap.tc.logging.Location;

import com.sapportals.wcm.crt.*;
import com.sapportals.wcm.util.logging.LoggingFormatter;
import java.lang.ClassLoader;
import java.text.MessageFormat;
import java.util.ArrayList;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;

import java.util.ResourceBundle;

/**
 * A helper class to load resource bundles.
 *
 * @author Jens Kaiser
 * @version $Id: //javabas/com.sapportals.wcm/dev/src/java/util/api/com/sapportals/wcm/util/resource/ResourceBundles.java#11
 *      $
 */
public class ResourceBundles {
  private String m_baseName;

  private static Location log = Location.getLocation(ResourceBundles.class);

  private static HashMap m_bundles = new HashMap();
  private static HashMap m_failures = new HashMap();

  private ResourceBundles(String baseName) {
    m_baseName = baseName;
  }

  /**
   * Invalidates the cached resource bundles
   */
  public static void invalidate() {
    m_bundles.clear();
    m_failures.clear();
  }

  public String getString(String key, Locale locale, Object[] objs) throws MissingResourceException {
    String bundle = getBundle(m_baseName, locale).getString(key);

    if (objs != null) {
      MessageFormat msg = new MessageFormat(bundle);
      msg.setLocale(locale);
      StringBuffer replacement = new StringBuffer();
      msg.format(objs, replacement, null);
      bundle = replacement.toString();
    }
    return bundle;
  }

  public String getString(String key, Object[] objs) throws MissingResourceException {
    return getString(key, Locale.getDefault(), objs);
  }

  public String getString(String key, Object obj) throws MissingResourceException {
    return getString(key, Locale.getDefault(), new Object[] { obj });
  }

  public String getString(String key, Object obj1, Object obj2) throws MissingResourceException {
    return getString(key, Locale.getDefault(), new Object[] { obj1, obj2 });
  }

  public String getString(String key, Locale locale) throws MissingResourceException {
    return getBundle(m_baseName, locale).getString(key);
  }

  public String getString(String key, Locale locale, Object obj) throws MissingResourceException {
    return getString(key, locale, new Object[] { obj });
  }

  public String getString(String key, Locale locale, Object obj1, Object obj2) throws MissingResourceException {
    return getString(key, locale, new Object[] { obj1, obj2 });
  }

  public String getString(String key) throws MissingResourceException {
    return this.getString(key, Locale.getDefault());
  }

  public boolean exists(String key, Locale locale) {
    try {
      this.getString(key, locale);
    }
    catch (Exception ex) {
      return false;
    }
    return true;
  }

  public boolean exists(String key) {
    try {
      this.getString(key);
    }
    catch (Exception ex) {
      return false;
    }
    return true;
  }

  /**
   * Deprectated. This method has no effect.
   * @param cl
   */
  public static void registerClassLoader(ClassLoader cl) {
  }

  public static ResourceBundles getBundle(String baseName) {
    return new ResourceBundles(baseName);
  }

  private static ResourceBundle getBundle(String baseName, Locale locale) {
    ResourceBundle res = null;
    MissingResourceException x = (MissingResourceException)m_failures.get(baseName + "_" + locale);
    if (x == null) {
      try {
        HashMap baseMap = (HashMap)m_bundles.get(baseName);
        if (baseMap == null) {
          res = ResourceBundle.getBundle(baseName, locale, CrtClassLoaderRegistry.getClassLoader());
          baseMap = new HashMap();
          baseMap.put(locale, res);
          m_bundles.put(baseName, baseMap);
        }
        else {
          res = (ResourceBundle)baseMap.get(locale);
          if (res == null) {
            res = ResourceBundle.getBundle(baseName, locale, CrtClassLoaderRegistry.getClassLoader());
            baseMap.put(locale, res);
          }
        }
      }
      catch (MissingResourceException mrx) {
        m_failures.put(baseName + "_" + locale, mrx);
        throw mrx;
      }
    }
    else {
      throw x;
    }

    return res;
  }
}
